<?php

/**
 * WooCommerce B2B Sales Agents Admin Orders set-up Class
 *
 * @version 1.3.1
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Admin_Orders Class
 */
class WCB2BSA_Admin_Orders {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'woocommerce_hidden_order_itemmeta', array( $this, 'hide_item_commission' ) );
        
        add_filter( 'pre_get_posts', array( $this, 'query_filters' ) );
        add_action( 'restrict_manage_posts', array( $this, 'render_filters' ) );
        add_action( 'woocommerce_process_shop_order_meta', array( $this, 'update_order_meta' ) );
        add_action( 'woocommerce_before_save_order_items', array( $this, 'update_order_item_meta' ), 10, 2 );
        add_action( 'woocommerce_admin_order_data_after_order_details', array( $this, 'display_order_meta' ) );
        add_action( 'woocommerce_admin_order_totals_after_total', array( $this, 'display_order_commission_amount' ) );
        add_action( 'woocommerce_admin_order_item_headers', array( $this, 'item_commission_headers' ) );
        add_action( 'woocommerce_admin_order_item_values', array( $this, 'item_commission_values' ), 10, 3 );
        add_action( 'woocommerce_after_order_itemmeta', array( $this, 'display_custom_price_meta' ), 10, 3 );

        if ( get_option( 'wcb2bsa_commission_column' ) === 'yes' ) {
            add_filter( 'manage_edit-shop_order_columns', array( $this, 'render_columns' ) );
            add_filter( 'manage_shop_order_posts_custom_column', array( $this, 'render_columns_value' ), 10, 2 );
            add_filter( 'manage_edit-shop_order_sortable_columns', array( $this, 'make_columns_sortable' ) );
        }
    }

    /**
     * Add parameters to filter orders
     *
     * @param WP_Query $query The WP_Query instance (passed by reference)
     */
    public function query_filters( $query ) {
        global $typenow;

        if ( $typenow == 'shop_order' ) {
            if ( isset( $_GET['wcb2bsa_sales_agent_search'] ) && $_GET['wcb2bsa_sales_agent_search'] !== '' ) {
                $meta_query[] = array(
                    'key'   => 'wcb2bsa_sales_agent',
                    'value' => (int)$_GET['wcb2bsa_sales_agent_search']
                );
            }
            if ( ! empty( $meta_query ) ) {
                $query->set( 'meta_query', $meta_query );
            }    
        }
    }

    /**
     * Add filters to orders
     */
    public function render_filters() {
        global $typenow;

        if ( in_array( $typenow, wc_get_order_types( 'order-meta-boxes' ), true ) ) {
            $sales_agents = wcb2bsa_get_sales_agents();

            $filters = '<select class="wc-customer-search" name="wcb2bsa_sales_agent_search" id="wcb2bsa_sales_agent_search" data-placeholder="' . esc_html__( 'Filter by sales agent', 'woocommerce-b2b-sales-agents' ) . '" data-allow_clear="true">';
            $filters .= '<option value=""></option>';

            if ( ! empty( $sales_agents->get_results() ) ) {
                foreach ( $sales_agents->get_results() as $sales_agent ) {
                    $filters .= sprintf(
                        '<option value="%1$s" %2$s>%3$s</option>',
                        $sales_agent->ID,
                        ( isset( $_GET['wcb2bsa_sales_agent_search'] ) && $_GET['wcb2bsa_sales_agent_search'] != '' && (int)$_GET['wcb2bsa_sales_agent_search'] === $sales_agent->ID ? ' selected="selected"' : '' ),
                        $sales_agent->display_name
                    );
                }
            }
            wp_reset_postdata();

            $filters .= '</select>';
            echo $filters;
        }
    }

    /**
     * Update orders meta data
     *
     * @param int $order_id The processing order ID
     */
    public function update_order_meta( $order_id ) {
        $sales_agent_id = isset( $_POST['wcb2bsa_sales_agent'] ) ? $_POST['wcb2bsa_sales_agent'] : '';
        $customer_id = isset( $_POST['customer_user'] ) ? $_POST['customer_user'] : '';
        $order_item_types_eligible_for_commission = isset( $_POST['wcb2bsa_order_item_types_eligible_for_commission'] ) ? $_POST['wcb2bsa_order_item_types_eligible_for_commission'] : '';
        $order_created_by = isset( $_POST['wcb2bsa_created_by'] ) ? $_POST['wcb2bsa_created_by'] : '';

        if ( $_POST['save'] == 'Create' ) {
            if ( empty( $sales_agent_id ) && ! empty( $customer_id ) ) {
                $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $customer_id );
            }
            if ( empty( $order_item_types_eligible_for_commission ) ) {
                $order_item_types_eligible_for_commission = get_option( 'wcb2bsa_order_item_types_eligible_for_commission' );
            }
        }

        update_post_meta( $order_id, '_wcb2bsa_order_version', str_replace( '.', '', WCB2BSA()->version ) );
        update_post_meta( $order_id, 'wcb2bsa_sales_agent', $sales_agent_id );
        update_post_meta( $order_id, 'wcb2bsa_order_item_types_eligible_for_commission', $order_item_types_eligible_for_commission );
        if ( ! empty( $order_created_by ) ) {
            update_post_meta( $order_id, 'wcb2bsa_created_by', $order_created_by );
        }
    }

    /**
     * Update order item meta on item edit
     * 
     * @param int $order_id Order ID
     * @param array $items Order items posted values
     */
    public function update_order_item_meta( $order_id, $items ) {
        if ( ! is_admin() ) { return false; }
        $order = new WC_Order( $order_id );

        // Check if order has assigned sales agent
        $sales_agent_id = get_post_meta( $order_id, 'wcb2bsa_sales_agent', true );
        if ( isset( $items['wcb2bsa_sales_agent'] ) && $items['wcb2bsa_sales_agent'] ) {
            $sales_agent_id = $items['wcb2bsa_sales_agent'];
        }

        // Create order items list
        $order_items = array();
        if ( isset( $items['order_item_id'] ) ) {
            $order_items = array_merge( $order_items, $items['order_item_id'] );
        }
        if ( isset( $items['shipping_method_id'] ) ) {
            $order_items = array_merge( $order_items, $items['shipping_method_id'] );
        }

        $order_item_types_eligible_for_commission = get_post_meta( $order_id, 'wcb2bsa_order_item_types_eligible_for_commission', true );

        if ( ! empty( $order_items ) ) {
            foreach ( $order_items as $item_id ) {
                $item = WC_Order_Factory::get_order_item( absint( $item_id ) );

                if ( $order_item_types_eligible_for_commission && isset($order_item_types_eligible_for_commission[$item->get_type()]) ) {

                    if ( ! $sales_agent_id ) {
                        // If there isn't a sales agent assigned, reset all item commissions
                        $item_metas = $item->get_meta_data();
                        foreach ( $item_metas as $item_meta ) {
                            if ( 0 === stripos( $item_meta->key, 'wcb2bsa_item_commission' ) ) {
                                wc_update_order_item_meta( $item_id, $item_meta->key, 0 );
                            }
                        }
                        continue;
                    }

                    /* Only if has a sales agent assigned... */

                    if ( isset( $items['wcb2bsa_item_commission'][$item_id] ) ) {
                        // Get from POST fields...
                        $item_commission = $items['wcb2bsa_item_commission'][$item_id];
                    } else {
                        // Get by item
                        $item_commission = wcb2bsa_get_item_commission( $item, $sales_agent_id );
                    }
                    // Save primary commission (to calculate total amount)
                    wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission', $item_commission );

                    // Calculate primary commission amount and save
                    $item_commission_amount = wcb2bsa_order_item_commission_amount_evaluate( $item, $order );
                    wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount', $item_commission_amount );

                    // Walk hierarchy (ascendant)
                    $parents = array();
                    $parents = wcb2bsa_get_parents_hierarchy_by_sales_agent( $sales_agent_id, $parents );
                    foreach ( $parents as $parent_id ) {
                        // Save parent subcommission percentage
                        $item_subcommission = (float)get_the_author_meta( 'wcb2bsa_subcommission', $parent_id );
                        wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_' . $parent_id, $item_subcommission );

                        // Calculate parent subcommission amount and save
                        $item_subcommission_amount = $item_commission_amount * $item_subcommission / 100;
                        wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount_' . $parent_id, $item_subcommission_amount );
                        
                        // Subtracts current subcommission amout from total commission amount
                        $item_commission_amount -= $item_subcommission_amount;
                    }
                    // Save primary sales agent remaining commission and percentage
                    wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_' . $sales_agent_id, $item_commission );
                    wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount_' . $sales_agent_id, $item_commission_amount );
                }
            }
        }
        // Update order total commission amount
        update_post_meta( $order_id, 'wcb2bsa_commission_amount', wcb2bsa_order_commission_amount_evaluate( $order_id ) );
    }

    /**
     * Add order meta fields in administration page
     *
     * @param WC_Order $order The processing WC_Order instance
     */
    public function display_order_meta( $order ) {
        include WCB2BSA_ABSPATH . 'includes/views/html-admin-order.php';   
    }

    /**
     * Display order total commission amount
     * 
     * @param int $order_id Order ID
     */
    public function display_order_commission_amount( $order_id ) {
        include WCB2BSA_ABSPATH . 'includes/views/html-admin-order-commission.php';   
    }

    /**
     * Hiding item meta commission. Useful to manually update item meta
     * 
     * @param array $hidden_meta Array of all meta data to hide.
     * @return array
     */
    public function hide_item_commission( $hidden_meta ) {
        $hidden_meta[] = 'wcb2bsa_custom_price';
        $hidden_meta[] = 'wcb2bsa_item_commission';
        $hidden_meta[] = 'wcb2bsa_item_commission_amount';
        $sales_agents = wcb2bsa_get_sales_agents();
        foreach ( $sales_agents->get_results() as $sales_agent ) {
            $hidden_meta[] = 'wcb2bsa_item_commission_' . $sales_agent->ID;
            $hidden_meta[] = 'wcb2bsa_item_commission_amount_' . $sales_agent->ID;
        }
        return $hidden_meta;
    }

    /**
     * Add commission headers to order products table
     *
     * @param WC_Order $order Current order instance
     */
    public function item_commission_headers( $order ) {
        printf( '<th class="item_cost sortable no-break" data-sort="float">%s</th>', esc_html__( 'Commission (%)', 'woocommerce-b2b-sales-agents' ) );
        printf( '<th class="item_cost sortable no-break" data-sort="float">%s</th>', esc_html__( 'Commission amount', 'woocommerce-b2b-sales-agents' ) );
    }

    /**
     * Display order item commission percentage and amount
     * 
     * @param WC_Product $product Product class instance
     * @param WC_Order_item $item The item being displayed
     * @param int $item_id The item id being displayed
     */
    public function item_commission_values( $product, $item, $item_id ) {
        include WCB2BSA_ABSPATH . 'includes/views/html-admin-order-item-commission.php';   
    }

    /**
     * Add notice to advice if item price has been edited by sales agent
     *
     * @param integer $item_id Current item ID
     * @param object $item Current item
     * @param object $product Current item product
     */
    public function display_custom_price_meta( $item_id, $item, $product ) {
        if ( wc_get_order_item_meta( $item_id, 'wcb2bsa_custom_price', true ) ) {
            printf( '<em class="wcb2b-help">%s</em>', __( '(Repriced by sales agent)', 'woocommerce-b2b-sales-agents' ) );
        }
    }

    /**
     * Add new orders columns in list
     * 
     * @param  array $columns Array of column name => label
     * @return array
     */
    public function render_columns( $columns ) {
        foreach ( $columns as $column_name => $column_info ) {
            if ( 'order_total' === $column_name ) {
                $_columns['wcb2bsa_sales_agent'] = esc_html__( 'Sales agent', 'woocommerce-b2b-sales-agents' );
            }
            $_columns[ $column_name ] = $column_info;
        }
        $_columns['wcb2bsa_commission_amount'] = esc_html__( 'Commission amount', 'woocommerce-b2b-sales-agents' );
        return $_columns;
    }

    /**
     * Retrieve orders column value in list
     *
     * @param string $column_name Column name
     * @param int $order_id ID of the currently-listed order
     */
    public function render_columns_value( $column_name, $order_id ) {
        if ( $column_name === 'wcb2bsa_sales_agent' ) {
            if ( $sales_agent_id = get_post_meta( $order_id, 'wcb2bsa_sales_agent', true ) ) {
                $sales_agent = get_userdata( $sales_agent_id );
                if ( false !== $sales_agent && wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) {
                    echo $sales_agent->display_name;
                }
            }
        }
        if ( $column_name === 'wcb2bsa_commission_amount' ) {
            $order = new WC_Order( $order_id );
            echo wc_price( $order->get_meta( 'wcb2bsa_commission_amount', true ), array( 'currency' => $order->get_currency() ) );
        }
    }

    /**
     * Make new columns sortable in order list
     * 
     * @param  array $columns Array of column name => label
     * @return array
     */
    public function make_columns_sortable( $columns ) {
        $columns['wcb2bsa_sales_agent'] = 'wcb2bsa_sales_agent';
        $columns['wcb2bsa_commission_amount'] = 'wcb2bsa_commission_amount';
        return $columns;
    }

}

return new WCB2BSA_Admin_Orders();