<?php

/**
 * WooCommerce B2B Sales Agents Tools
 *
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Admin_Tools
 */
class WCB2BSA_Admin_Tools {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'export_filters', array( $this, 'export_commissions_option' ) );
        add_filter( 'export_filters', array( $this, 'export_sales_agents_option' ) );
        add_filter( 'export_wp', array( $this, 'export_commissions' ) );
        add_filter( 'export_wp', array( $this, 'export_sales_agents' ) );
    }

    /**
     * Add commissions exporter to WordPress tools
     */
    public function export_commissions_option() {
        ?>
        <fieldset>
            <p>
                <label>
                    <input type="radio" name="content" value="commissions"><?php esc_html_e( 'Commissions', 'woocommerce-b2b-sales-agents' ); ?>
                </label>
            </p>
            <p class="description"><?php esc_html_e( 'In CSV format for statistics purposes', 'woocommerce-b2b-sales-agents' ); ?></p>
        </fieldset>
        <?php
    }

    /**
     * Add sales agents exporter to WordPress tools
     */
    public function export_sales_agents_option() {
        ?>
        <fieldset>
            <p>
                <label>
                    <input type="radio" name="content" value="sales_agents"><?php esc_html_e( 'Sales agents', 'woocommerce-b2b-sales-agents' ); ?>
                </label>
            </p>
            <p class="description"><?php esc_html_e( 'In CSV format for statistics purposes', 'woocommerce-b2b-sales-agents' ); ?></p>
        </fieldset>
        <?php
    }

    /**
     * Export commissions
     * 
     * @param array $args Export tools arguments
     */
    public function export_commissions( $args ) {
        if ( 'commissions' == $args['content'] ) {
            $fields = array(
                __('Order id', 'woocommerce-b2b-sales-agents'),
                __('Order number', 'woocommerce-b2b-sales-agents'),
                __('Customer', 'woocommerce-b2b-sales-agents'),
                __('Sales_agent', 'woocommerce-b2b-sales-agents'),
                __('Date', 'woocommerce-b2b-sales-agents'),
                __('Status', 'woocommerce-b2b-sales-agents'),
                __('Total', 'woocommerce-b2b-sales-agents'),
                __('Commission amount', 'woocommerce-b2b-sales-agents'),
                __('Commission status', 'woocommerce-b2b-sales-agents'),
                __('Created by', 'woocommerce-b2b-sales-agents')
            );

            $args = array(
                'type'                    => 'shop_order',
                'wcb2bsa_has_sales_agent' => true,
                'limit' => -1
            );
            $orders = wc_get_orders( $args );

            // If any customer is retrieved, skip
            if ( ! $orders ) {
                $referer = add_query_arg( 'error', 'empty', wp_get_referer() );
                wp_redirect( $referer );
                exit;
            }

            // Prepare headers to download CSV file
            header( 'Content-Description: File Transfer' );
            header( 'Content-Disposition: attachment; filename=' . sanitize_key( get_bloginfo( 'name' ) ) . '-commissions-' . date( 'YmdHis' ) . '.csv' );
            header( 'Content-Type: text/csv; charset=' . get_option( 'blog_charset' ), true );

            echo strtoupper( str_replace( '_', ' ', implode( ';', $fields ) ) ) . "\n";
            foreach ( $orders as $order ) {
                $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent', true );
                $sales_agent = get_userdata( $sales_agent_id );
                if ( ! ( false !== $sales_agent && wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) ) { continue; }

                $order_id = $order->get_id();
                $data['order_id'] = $order_id;
                $data['order_number'] = $order->get_order_number();
                $data['customer'] = sprintf('%s %s', $order->get_billing_last_name(), $order->get_billing_first_name() );
                $data['sales_agent'] = sprintf( '%s %s', $sales_agent->first_name, $sales_agent->last_name );
                $data['date'] = wc_format_datetime( $order->get_date_created(), 'd-m-Y' );
                $data['status'] = wc_get_order_status_name( $order->get_status() );
                $data['total'] = $order->get_total();
                $data['commission_amount'] = wcb2bsa_get_order_commission_amount_by_sales_agent( $order_id, $sales_agent_id );
                $data['commission_status'] = wcb2bsa_get_order_commission_payout_by_sales_agent( $order_id, $sales_agent_id );
                $data['created_by'] = wcb2bsa_get_order_author( $order );

                echo implode( ';', $data ) . "\n";

                $parents = array();
                $parents = wcb2bsa_get_parents_hierarchy_by_sales_agent( $sales_agent_id, $parents, true );
                foreach ( $parents as $parent_id ) {
                    $parent = get_userdata( $parent_id );
                    if ( ! ( false !== $parent && wcb2bsa_has_role( $parent_id, 'sales_agent' ) ) ) { continue; }

                    $data['order_id'] = false;
                    $data['order_number'] = false;
                    $data['customer'] = false;
                    $data['sales_agent'] = sprintf( '%s %s', $parent->first_name, $parent->last_name );
                    $data['date'] = false;
                    $data['status'] = false;
                    $data['total'] = false;
                    $data['commission_amount'] = wcb2bsa_get_order_commission_amount_by_sales_agent( $order_id, $parent_id );
                    $data['commission_status'] = wcb2bsa_get_order_commission_payout_by_sales_agent( $order_id, $parent_id );
                    $data['created_by'] = false;

                    echo implode( ';', $data ) . "\n";
                }
            }
            exit;
        }
    }

    /**
     * Export sales agents
     * 
     * @param array $args Export tools arguments
     */
    public function export_sales_agents( $args ) {
        if ( 'sales_agents' == $args['content'] ) {
            // Requested fields
            $fields = array(
                'ID',
                'user_login',
                'user_email',
                'user_registered',
                'first_name',
                'last_name',
                'billing_first_name',
                'billing_last_name',
                'billing_company',
                'billing_vat',
                'billing_address_1',
                'billing_address_2',
                'billing_city',
                'billing_postcode',
                'billing_country',
                'billing_state',
                'billing_phone',
                'billing_email',
                'shipping_first_name',
                'shipping_last_name',
                'shipping_company',
                'shipping_address_1',
                'shipping_address_2',
                'shipping_city',
                'shipping_postcode',
                'shipping_country',
                'shipping_state',
                'wcb2bsa_sales_agent_parent',
                'manager'
            );

            // Get users by role (sales_agent)
            $sales_agents = get_users( array(
                'role__in'  => get_option( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ),
                'fields'    => array(
                    'ID',
                    'user_login',
                    'user_email',
                    'user_registered'
                )
            ) );

            // If any sales agent is retrieved, skip
            if ( ! $sales_agents ) {
                $referer = add_query_arg( 'error', 'empty', wp_get_referer() );
                wp_redirect( $referer );
                exit;
            }

            // Prepare headers to download CSV file
            header( 'Content-Description: File Transfer' );
            header( 'Content-Disposition: attachment; filename=' . sanitize_key( get_bloginfo( 'name' ) ) . '-sales_agents-' . date( 'YmdHis' ) . '.csv' );
            header( 'Content-Type: text/csv; charset=' . get_option( 'blog_charset' ), true );

            echo strtoupper( str_replace( '_', ' ', implode( ';', $fields ) ) ) . "\n";
            foreach ( $sales_agents as $sales_agent ) {
                $data = array_flip( $fields );
                $data = array_fill_keys( $fields, null );
                $data = array_merge( $data, (array)$sales_agent, wp_list_pluck( get_user_meta( $sales_agent->ID ), 0 ) );
                $data = array_intersect_key( $data, array_flip( $fields ) );
                
                $data['manager_id'] = $data['wcb2bsa_sales_agent_parent'];
                $data['manager'] = $data['manager_id'] ? get_user_meta( $data['manager_id'], 'display_name' ) : '';
                $data['user_registered'] = date_i18n( get_option( 'date_format' ), strtotime( $data['user_registered'] ) );

                unset( $data['wcb2bsa_sales_agent_parent'] );

                echo implode( ';', $data ) . "\n";
            }
            exit;
        }
    }

}

return new WCB2BSA_Admin_Tools();