<?php

/**
 * WooCommerce B2B Sales Agents Admin set-up Class
 *
 * @version 1.3.1
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Admin Class
 */
class WCB2BSA_Admin {

    /**
     * Constructor
     */
    public function __construct() {
        $this->includes();
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_action( 'admin_enqueue_scripts', array( $this, 'print_styles' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'print_scripts' ) );
    }

    /**
     * Include any classes we need within admin
     */
    public function includes() {
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-admin-orders.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-admin-posts.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-admin-users.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-admin-products.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-admin-tools.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-admin-reports.php';
    }

    /**
     * Print styles
     */
    public function print_styles() {
        wp_register_style( 'wcb2bsa_admin', plugins_url( 'assets/css/admin.min.css', WCB2BSA_PLUGIN_FILE ), false, '1.2.1' );
        wp_enqueue_style( 'wcb2bsa_admin' );
    }

    /**
     * Print scripts
     */
    public function print_scripts() {
        $sales_agents = wcb2bsa_get_sales_agents();

        wp_register_script( 'wcb2bsa_admin', plugins_url( 'assets/js/admin.min.js', WCB2BSA_PLUGIN_FILE ), false, '1.0.1', true );
        wp_enqueue_script( 'wcb2bsa_admin' );
        wp_localize_script( 'wcb2bsa_admin', 'wcb2bsa_users_parameters', array(
            'wcb2bsa_sales_agents'  => json_encode( wp_list_pluck( $sales_agents->get_results(), 'display_name', 'ID' ) ),
            'wcb2bsa_statuses'      => json_encode( WCB2BSA_Configuration::get_sales_agents_statuses() )
        ) );
    }

}

return new WCB2BSA_Admin();