<?php

/**
 * WooCommerce B2B Sales Agents Common hooks to improve compatibility with third parts assets
 *
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Compatibility Class
 */
class WCB2BSA_Compatibility {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'wc_get_template', array( $this, 'redis_cache_tweak' ), 10, 5 ); // Redis
    }

    /**
     * Used to fix Redis cache problem and improve cache management
     *
     * @param string $template Located template
     * @param string $template_name Template name
     * @param array $args Arguments
     * @param string $template_path Template path
     * @param string $default_path Default path
     */
    public function redis_cache_tweak( $template, $template_name, $args, $template_path, $default_path ) {
        if ( apply_filters( 'disabled_cache_tweak', false ) ) { return $template; }

        if ( function_exists( 'wcb2bsa_has_role' ) ) {
            if ( get_the_ID() == get_option( 'wcb2bsa_dashboard_page' ) && is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
                return wcb2bsa_locate_template( $template_name );
            }
        }
        return $template;
    }

}

return new WCB2BSA_Compatibility();