<?php

/**
 * WooCommerce B2B Sales Agents Frontend MyAccount set-up Class
 *
 * @version 1.3.1
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Frontend_Myaccount Class
 */
class WCB2BSA_Frontend_Myaccount {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'woocommerce_order_item_get_formatted_meta_data', array( $this, 'format_item_meta' ), 10, 2 );
        add_action( 'woocommerce_save_account_details', array( $this, 'save_account_details' ) );
        add_filter( 'woocommerce_locate_template', array( $this, 'locate_template' ), 10, 3 );
        add_action( 'template_include', array( $this, 'template_include' ) );
        add_action( 'template_redirect', array( $this, 'logout_redirect' ) );
        add_filter( 'is_active_sidebar', array( $this, 'remove_sidebar' ), 10, 2 );
        add_filter( 'woocommerce_get_query_vars', array( $this, 'add_endpoints' ), 10 );
        add_action( 'wp_footer', array( $this, 'add_switchbar' ) );
        add_action( 'clear_auth_cookie', array( $this, 'clear_cookie' ) );
        add_action( 'wp', array( $this, 'template_loader' ) );
        add_action( 'wp', array( $this, 'link_to_dashboard' ) );
        add_action( 'init', array( $this, 'get_endpoints_title' ) );
        add_action( 'init', array( $this, 'switch_to_id' ) );
        add_action( 'woocommerce_register_form_start', array( $this, 'add_custom_fields' ) );
        add_filter( 'woocommerce_new_customer_data', array( $this, 'update_user_data' ) );
        add_action( 'woocommerce_created_customer', array( $this, 'new_customer' ), 10, 3 );
        add_filter( 'the_content', array( $this, 'disable_pagebuilder_on_sales_agents_area' ) );

        if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) {
            add_filter( 'authenticate', array( $this, 'prevent_login' ), 98, 2 );
            add_filter( 'woocommerce_registration_auth_new_customer', array( $this, 'prevent_autologin' ), 98, 2 );
        }

        if ( get_option( 'wcb2bsa_registration_notice' ) === 'yes' ) {
            add_action( 'woocommerce_created_customer', array( $this, 'notify_admin' ) );
        }

        if ( 'yes' == get_option( 'wcb2bsa_enable_affiliation' ) ) {
            add_action( 'init', array( $this, 'set_affiliate_cookie' ) );
            add_action( 'woocommerce_created_customer', array( $this, 'new_customer_from_affiliation' ), 11, 3 );
        }

        if ( 'yes' == get_option( 'wcb2bsa_allow_new_customers', false ) ) {
            add_action( 'woocommerce_created_customer', array( $this, 'new_customer_from_sales_agent' ), 12, 3 );
        }

        if ( 'yes' == get_option( 'wcb2bsa_allow_new_subagents', false ) ) {
            add_action( 'woocommerce_created_customer', array( $this, 'new_subagent_from_sales_agent' ), 12, 3 );
        }

        if (
            'yes' == get_option( 'wcb2bsa_extend_new_customers_form' )
            || 
            'yes' == get_option( 'wcb2bsa_extend_new_subagents_form' )
        ) {

            if ( 'yes' == get_option( 'wcb2bsa_extend_new_customers_form' ) ) {
                add_action( 'wcb2bsa_new_customer_form', array( $this, 'add_extended_fields' ) );
            }
            if ( 'yes' == get_option( 'wcb2bsa_extend_new_subagents_form' ) ) {
                add_action( 'wcb2bsa_new_subagent_form', array( $this, 'add_extended_fields' ) );
            }

            add_action( 'wcb2bsa_print_scripts', array( $this, 'add_dependencies_scripts' ) );
            add_action( 'woocommerce_register_post', array( $this, 'validate_extended_fields' ), 10, 3 );
            add_action( 'woocommerce_created_customer', array( $this, 'save_extended_fields' ) );
            add_filter( 'woocommerce_get_country_locale_base', array( $this, 'force_not_required_fields' ) );
        }

        if ( 'yes' == get_option( 'wcb2bsa_update_order_status' ) ) {
            add_action( 'wp_loaded', array( $this, 'process_update_order_status' ) );
        }

        if ( 'yes' == get_option( 'wcb2bsa_add_order_note' ) ) {
            add_action( 'wp_loaded', array( $this, 'process_add_order_note' ) );
        }

        if ( 'yes' == get_option( 'wcb2bsa_myaccount_data' ) ) {
            add_action( 'woocommerce_account_dashboard', array( $this, 'sales_agent_in_myaccount' ), 99 );
        }
    }

    /**
     * Format item meta for display
     * 
     * @param array $formatted_meta Contains all meta
     * @param WC_Order_Item $item Current order item instance
     * @return array
     */
    public function format_item_meta( $formatted_meta, $item ) {
        if ( is_array( $formatted_meta ) ) {
            foreach( $formatted_meta as $key => $meta ) {
                if ( in_array( $meta->key, array( 'wcb2bsa_custom_price' ) ) ) {
                    unset( $formatted_meta[$key] );
                }
                if ( 0 === stripos( $meta->key, 'wcb2bsa_item_commission' ) ) {
                    unset( $formatted_meta[$key] );
                }
            }
        }
        return $formatted_meta;
    }

    /**
     * Save order notification option
     *   
     * @param int $user_id User ID
     */
    public function save_account_details( $user_id ) {
        $telephone = isset( $_POST['billing_phone'] ) ? sanitize_text_field( $_POST['billing_phone'] ) : '';
        $notification = isset( $_POST['wcb2bsa_order_notification'] ) ? (int)$_POST['wcb2bsa_order_notification'] : 0;
        $payment_method = isset( $_POST['wcb2bsa_payment_method'] ) ? sanitize_text_field( $_POST['wcb2bsa_payment_method'] ) : '';
        $payment_method_info = isset( $_POST['wcb2bsa_payment_method_info'] ) ? sanitize_textarea_field( $_POST['wcb2bsa_payment_method_info'] ) : '';
        update_user_meta( $user_id, 'billing_phone', $telephone );
        update_user_meta( $user_id, 'wcb2bsa_order_notification', $notification );
        update_user_meta( $user_id, 'wcb2bsa_payment_method', $payment_method );
        update_user_meta( $user_id, 'wcb2bsa_payment_method_info', $payment_method_info );
        if ( is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
            wp_safe_redirect( esc_url_raw( wc_get_endpoint_url( 'sales-agent-edit-account', '', get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) ) ) );
            exit;
        }
    }

    /**
     * Override sales agents myaccount files, to locate theme from theme/plugin
     *
     * @param string $template_name Template name
     * @param string $template_path Template path
     * @param string $default_path  Default path
     * @return string
     */
    public function locate_template( $template, $template_name, $template_path ) {
        if ( is_user_logged_in() ) {
            $check = false;
            $override = array();
            if ( wcb2bsa_has_role( get_current_user_id(), 'customer' ) ) {
                $override = apply_filters( 'wcb2bsa_template_override_customer', array(
                    'global/switchbar.php'
                ) );
                $check = true;
            }

            if ( wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
                $override = apply_filters( 'wcb2bsa_template_override_sales_agent', array(
                    'sales_agent-page.php',
                    'global/header.php',
                    'global/footer.php',
                    'global/sidebar.php',
                    'global/breadcrumb.php',
                    'global/pagination.php',
                    'myaccount/my-account.php',
                    'myaccount/dashboard.php',
                    'myaccount/customers.php',
                    'myaccount/view-customer.php',
                    'myaccount/orders.php',
                    'myaccount/view-order.php',
                    'myaccount/earnings.php',
                    'myaccount/commissions.php',
                    'myaccount/notifications.php',
                    'myaccount/statistics.php',
                    'myaccount/subagents.php',
                    'myaccount/view-subagent.php',
                    'myaccount/edit-account.php',
                    'myaccount/form-new-customer.php',
                    'myaccount/form-new-subagent.php',
                    'myaccount/form-update-order-status.php',
                    'myaccount/form-add-order-note.php',
                    'notices/success.php',
                    'notices/error.php',
                    'notices/info.php'
                ) );
                $check = is_page( get_option( 'wcb2bsa_dashboard_page' ) );
            }

            if ( $check && in_array( $template_name, $override ) ) {
                $template = wcb2bsa_locate_template( $template_name );
            }
        }
        return $template;
    }

    /**
     * Change main page template for sales agents
     * 
     * @param string $template Template name
     * @return string
     */
    public function template_include( $template ) {
        if ( is_page( get_option( 'wcb2bsa_dashboard_page' ) ) && is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
            add_filter( 'woocommerce_account_menu_item_classes', array( $this, 'clean_menu_item_classes' ), 10, 2 );
            return wc_locate_template( 'sales_agent-page.php' );
        }
        return $template;
    }

    /**
     * Remove unused classes from menu items
     * 
     * @param array $classes Classes to be applied to menu item
     * @param string $endpoint Endpoint
     * @return array
     */
    public function clean_menu_item_classes( $classes, $endpoint ) {
        return array_intersect( str_replace( 'is-active', 'active', $classes ), array( 'active' ) );
    }

    /**
     * Add logout management for sales agents
     */
    public function logout_redirect() {
        global $wp_query, $wp;

        // Logout
        if ( isset( $wp->query_vars['sales-agent-logout'] ) && ! empty( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_REQUEST['_wpnonce'] ), 'sales-agent-logout' ) ) {
            wp_safe_redirect( str_replace( '&amp;', '&', wp_logout_url( get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) ) ) );
            exit;
        }

        // Redirect to the correct logout endpoint
        if ( isset( $wp->query_vars['sales-agent-logout'] ) && 'true' === $wp->query_vars['sales-agent-logout'] ) {
            wp_safe_redirect( esc_url_raw( wc_get_endpoint_url( 'sales-agent-logout', '', get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) ) ) );
            exit;
        }
    }

    /**
     * Remove sidebar from sales agents area
     * 
     * @param  bool $is_active_sidebar Sidebar will be displayed?
     * @return bool
     */
    public function remove_sidebar( $is_active_sidebar ) {
        if ( is_page( get_option( 'wcb2bsa_dashboard_page' ) ) && is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
            $is_active_sidebar = false;
        }
        return $is_active_sidebar;
    }

    /**
     * Add sales agents area endpoints
     * 
     * @param array $query_vars Query variables
     */
    public function add_endpoints( $query_vars ) {
        $query_vars['sales-agent-customers']     = get_option( 'wcb2bsa_account_customers_endpoint', 'sales-agent-customers' );
        $query_vars['sales-agent-view-customer'] = get_option( 'wcb2bsa_account_view_customer_endpoint', 'sales-agent-view-customer' );
        $query_vars['sales-agent-orders']        = get_option( 'wcb2bsa_account_orders_endpoint', 'sales-agent-orders' );
        $query_vars['sales-agent-view-order']    = get_option( 'wcb2bsa_account_view_order_endpoint', 'sales-agent-view-order' );
        $query_vars['sales-agent-earnings']   = get_option( 'wcb2bsa_account_earnings_endpoint', 'sales-agent-earnings' );
        $query_vars['sales-agent-commissions']   = get_option( 'wcb2bsa_account_commissions_endpoint', 'sales-agent-commissions' );
        $query_vars['sales-agent-notifications'] = get_option( 'wcb2bsa_account_notifications_endpoint', 'sales-agent-notifications' );
        $query_vars['sales-agent-statistics']    = get_option( 'wcb2bsa_account_statistics_endpoint', 'sales-agent-statistics' );
        $query_vars['sales-agent-subagents']    = get_option( 'wcb2bsa_account_subagents_endpoint', 'sales-agent-subagents' );
        $query_vars['sales-agent-view-subagent'] = get_option( 'wcb2bsa_account_view_subagent_endpoint', 'sales-agent-view-subagent' );
        $query_vars['sales-agent-edit-account']  = get_option( 'wcb2bsa_account_edit_account_endpoint', 'sales-agent-edit-account' );
        $query_vars['sales-agent-logout']        = get_option( 'wcb2bsa_account_logout_endpoint', 'sales-agent-logout' );
        return $query_vars;
    }

    public function get_endpoints_title() {
        add_filter( 'woocommerce_endpoint_sales-agent-customers_title', array( $this, 'get_customers_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-orders_title', array( $this, 'get_orders_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-earnings_title', array( $this, 'get_earnings_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-commissions_title', array( $this, 'get_commissions_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-notifications_title', array( $this, 'get_notifications_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-statistics_title', array( $this, 'get_statistics_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-subagents_title', array( $this, 'get_subagents_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-view-customer_title', array( $this, 'get_view_customer_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-view-order_title', array( $this, 'get_view_order_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-view-subagent_title', array( $this, 'get_view_subagent_endpoint_title' ), 10, 2 );
        add_filter( 'woocommerce_endpoint_sales-agent-edit-account_title', array( $this, 'get_edit_account_endpoint_title' ), 10, 2 );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_customers_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Customers', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_orders_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Orders', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_earnings_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Earnings', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_commissions_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Commissions', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_notifications_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Notifications', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_statistics_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Statistics', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_subagents_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Subagents', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_view_customer_endpoint_title( $title, $endpoint ) {
        global $wp;
        $customer = get_userdata( $wp->query_vars['sales-agent-view-customer'] );

        return ( $customer ? sprintf( esc_html__( 'Customer #%s', 'woocommerce-b2b-sales-agents' ), $customer->ID ) : '' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_view_order_endpoint_title( $title, $endpoint ) {
        global $wp;
        $order = wc_get_order( $wp->query_vars['sales-agent-view-order'] );

        return ( $order ? sprintf( esc_html__( 'Order #%s', 'woocommerce-b2b-sales-agents' ), $order->get_order_number() ) : '' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_view_subagent_endpoint_title( $title, $endpoint ) {
        global $wp;
        $subagent = get_userdata( $wp->query_vars['sales-agent-view-subagent'] );

        return ( $subagent ? sprintf( esc_html__( 'Subagent #%s', 'woocommerce-b2b-sales-agents' ), $subagent->ID ) : '' );
    }

    /**
     * Get endpoint title: 
     * 
     * @param string $title Endpoint title
     * @param string $endpoint Endpoint name
     * @return string
     */
    public function get_edit_account_endpoint_title( $title, $endpoint ) {
        return esc_html__( 'Account details', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Sales agents area hooks to load custom templates
     */
    public function template_loader() {
        if ( is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {

            // Save notification read status
            if ( isset( $_POST['notification'], $_POST['submit'] ) ) {
                if ( ! empty( $_POST['woocommerce-notifications-nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['woocommerce-notifications-nonce'] ), 'woocommerce-notifications' ) ) {
                    update_user_meta( get_current_user_id(), 'wcb2bsa_read_notification_' . $_POST['notification'], (boolean)$_POST['submit'] );
                }
            }

            // Remove default contents - Prevent deprecated notice
            if ( is_page( get_option( 'wcb2bsa_dashboard_page' ) ) ) {
                remove_action( 'woocommerce_account_content', 'woocommerce_account_content' );
            }

            add_action( 'wcb2bsa_account_sidebar', array( $this, 'output_sidebar' ) );
            add_action( 'wcb2bsa_account_header', array( $this, 'output_header' ) );
            add_action( 'wcb2bsa_account_footer', array( $this, 'output_footer' ) );
            add_action( 'wcb2bsa_account_content', array( $this, 'output_dashboard_page' ) );

            add_action( 'wcb2bsa_sales-agent-customers_endpoint', array( $this, 'output_customers_page' ) );
            add_action( 'wcb2bsa_sales-agent-orders_endpoint', array( $this, 'output_orders_page' ) );
            add_action( 'wcb2bsa_sales-agent-earnings_endpoint', array( $this, 'output_earnings_page' ) );
            add_action( 'wcb2bsa_sales-agent-commissions_endpoint', array( $this, 'output_commissions_page' ) );
            add_action( 'wcb2bsa_sales-agent-notifications_endpoint', array( $this, 'output_notifications_page' ) );
            add_action( 'wcb2bsa_sales-agent-statistics_endpoint', array( $this, 'output_statistics_page' ) );
            add_action( 'wcb2bsa_sales-agent-subagents_endpoint', array( $this, 'output_subagents_page' ) );
            add_action( 'wcb2bsa_sales-agent-view-customer_endpoint', array( $this, 'output_view_customer_page' ) );
            add_action( 'wcb2bsa_sales-agent-view-order_endpoint', array( $this, 'output_view_order_page' ) );
            add_action( 'wcb2bsa_sales-agent-view-subagent_endpoint', array( $this, 'output_view_subagent_page' ) );
            add_action( 'wcb2bsa_sales-agent-edit-account_endpoint', array( $this, 'output_edit_account_page' ) );
        }
    }

    /**
     * Display dashboard link in user area
     */
    public function link_to_dashboard() {
        if ( is_account_page() && !is_wc_endpoint_url() && ! is_page( get_option( 'wcb2bsa_dashboard_page' ) ) ) {
            if ( is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
                if ( get_option( 'wcb2bsa_dashboard_page' ) !== get_option( 'woocommerce_myaccount_page_id' ) ) {
                    $link = get_permalink( get_option( 'wcb2bsa_dashboard_page' ) );
                    $button = __( 'View dashboard', 'woocommerce-b2b-sales-agents' );
                    $text = __( 'Looking for your sales agent dashboard?', 'woocommerce-b2b-sales-agents' );
                    if ( $notice = apply_filters( 'wcb2bsa_link_to_dashboard', sprintf(
                        '<a href="%s" class="button wc-forward">%s</a> %s',
                        $link,
                        $button,
                        $text
                    ), $link, $button, $text ) ) {
                        wc_add_notice( $notice, 'notice' );
                    }
                }
            }
        }
    }

    /**
     * Global - Sales agent area sidebar and navigation menu
     */
    public function output_sidebar() {
        wc_get_template( 'global/sidebar.php', array(
            'sales_agent' => get_userdata( get_current_user_id() )
        ) );
    }

    /**
     * Global - Sales agent area header
     */
    public function output_header() {
        wc_get_template( 'global/header.php', array(
            'sales_agent' => get_userdata( get_current_user_id() )
        ) );
    }

    /**
     * Global - Sales agent area footer
     */
    public function output_footer() {
        wc_get_template( 'global/footer.php', array(
            'sales_agent' => get_userdata( get_current_user_id() )
        ) );
    }

    /**
     * Sales agent area dashboard
     */
    public function output_dashboard_page() {
        global $wp;
        
        if ( ! empty( $wp->query_vars ) ) {
            foreach ( $wp->query_vars as $key => $value ) {
                if ( 'pagename' === $key ) { continue; }

                if ( has_action( 'wcb2bsa_' . $key . '_endpoint' ) ) {
                    do_action( 'wcb2bsa_' . $key . '_endpoint', $value );
                    return;
                }
            }
        }

        $sales_agent_id = get_current_user_id();
        $dashboard = WCB2BSA_Configuration::get_widgets( $sales_agent_id );
        $dashboard['widgets'] = array_intersect_key( $dashboard['widgets'], get_option( 'wcb2bsa_dashboard_widgets', array() ) );
        $dashboard['stats'] = array_intersect_key( $dashboard['stats'], get_option( 'wcb2bsa_dashboard_charts', array() ) );
        $dashboard['manager'] = false;
        if ( $parent_id = get_the_author_meta( 'wcb2bsa_sales_agent_parent', $sales_agent_id ) ) {
            $dashboard['manager'] = get_userdata( $parent_id );
        }
        wc_get_template( 'myaccount/dashboard.php', $dashboard );
    }

    /**
     * Sales agent area assigned customers
     * 
     * @param int $current_page Current page number used in pagination
     */
    public function output_customers_page( $current_page ) {
        $current_page = empty( $current_page ) ? 1 : absint( $current_page );
        if ( $interactive = apply_filters( 'wcb2bsa_enable_interactive_dashboard', true ) ) {
            $current_page = false;
        }
        $customers = wcb2bsa_get_customers_by_sales_agent( get_current_user_id(), $current_page );
        wc_get_template( 'myaccount/customers.php', array(
            'interactive'   => $interactive,
            'customers'     => $customers,
            'has_customers' => $customers->total_users > 0 ? true : false
        ) );
        if ( ! $interactive ) {
            wc_get_template( 'global/pagination.php', array(
                'pages'        => ceil( $customers->total_users / apply_filters( 'wcb2bsa_dashboard_pagination', 20 )),
                'current_page' => $current_page
            ) );
        }

        if ( 'yes' == get_option( 'wcb2bsa_allow_new_customers', false ) ) {
            wc_get_template( 'myaccount/form-new-customer.php' );
        }
    }

    /**
     * Sales agent area assigned customers orders
     * 
     * @param int $current_page Current page number used in pagination
     */
    public function output_orders_page( $current_page ) {
        $current_page = empty( $current_page ) ? 1 : absint( $current_page );
        if ( $interactive = apply_filters( 'wcb2bsa_enable_interactive_dashboard', true ) ) {
            $current_page = false;
        }
        $orders = wcb2bsa_get_orders_by_sales_agent( get_current_user_id(), $current_page );
        wc_get_template( 'myaccount/orders.php', array(
            'interactive'   => $interactive,
            'orders'        => $orders,
            'has_orders'    => $orders->total > 0 ? true : false
        ) );

        if ( ! $interactive ) {
            wc_get_template( 'global/pagination.php', array(
                'pages'        => ceil( $orders->total / apply_filters( 'wcb2bsa_dashboard_pagination', 20 )),
                'current_page' => $current_page
            ) );
        }
    }

    /**
     * Sales agent area earnings
     * 
     * @param int $current_page Current page number used in pagination
     */
    public function output_earnings_page( $current_page ) {
        $current_page = empty( $current_page ) ? 1 : absint( $current_page );
        if ( $interactive = apply_filters( 'wcb2bsa_enable_interactive_dashboard', true ) ) {
            $current_page = false;
        }
        $orders = wcb2bsa_get_orders_by_sales_agent( get_current_user_id(), $current_page );
        wc_get_template( 'myaccount/earnings.php', array(
            'interactive'   => $interactive,
            'orders'        => $orders,
            'has_orders'    => $orders->total > 0 ? true : false
        ) );

        if ( ! $interactive ) {
            wc_get_template( 'global/pagination.php', array(
                'pages'        => ceil( $orders->total / apply_filters( 'wcb2bsa_dashboard_pagination', 20 )),
                'current_page' => $current_page
            ) );
        }
    }

    /**
     * Sales agent area commissions
     * 
     * @param int $current_page Current page number used in pagination
     */
    public function output_commissions_page( $current_page ) {
        $current_page = empty( $current_page ) ? 1 : absint( $current_page );
        if ( $interactive = apply_filters( 'wcb2bsa_enable_interactive_dashboard', true ) ) {
            $current_page = false;
        }
        $args = array(
            'post_type' => 'product',
            'limit'     => -1,
            'page'      => 1,
            'status'    => 'publish',
            'return'    => 'ids',
            'order'     => 'ASC',
            'orderby'   => 'name'
        );
        // Add pagination
        if ( $current_page !== false ) {
            $args['limit']    = apply_filters( 'wcb2bsa_dashboard_pagination', 20 );
            $args['page']     = $current_page;
        }
        $products = wc_get_products( $args );

        wc_get_template( 'myaccount/commissions.php', array(
            'interactive'   => $interactive,
            'products'      => $products,
            'has_products'  => count( $products ) > 0 ? true : false
        ) );

        if ( ! $interactive ) {
            wc_get_template( 'global/pagination.php', array(
                'pages'        => ceil( count( $products ) / apply_filters( 'wcb2bsa_dashboard_pagination', 20 )),
                'current_page' => $current_page
            ) );
        }
    }

    /**
     * Sales agent area notifications
     * 
     * @param int $current_page Current page number used in pagination
     */
    public function output_notifications_page( $current_page ) {
        $current_page = empty( $current_page ) ? 1 : absint( $current_page );
        $notifications = wcb2bsa_get_notifications_by_sales_agent( get_current_user_id(), $current_page );
        wc_get_template( 'myaccount/notifications.php', array(
            'current_page'  => $current_page,
            'notifications' => $notifications,
            'has_notifications' => count( $notifications ) > 0 ? true : false
        ) );
    }

    /**
     * Sales agent area statistics
     * 
     * @param int $current_page Current page number used in pagination
     */
    public function output_statistics_page( $current_page ) {
        $current_page = empty( $current_page ) ? 1 : absint( $current_page );
        $statistics = false;

        // Save notification read status
        if ( isset( $_POST['wcb2bsa-statistics-type'], $_POST['wcb2bsa-statistics-groupby'], $_POST['wcb2bsa-statistics-daterange'], $_POST['submit'] ) ) {
            if ( ! empty( $_POST['woocommerce-statistics-nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['woocommerce-statistics-nonce'] ), 'woocommerce-statistics' ) ) {
                $statistics = wcb2bsa_get_stats( get_current_user_id(), $_POST['wcb2bsa-statistics-type'], $_POST['wcb2bsa-statistics-data'], $_POST['wcb2bsa-statistics-groupby'], $_POST['wcb2bsa-statistics-daterange'] );
            }
        }

        wc_get_template( 'myaccount/statistics.php', array(
            'current_page'  => $current_page,
            'statistics'    => $statistics,
            'has_statistics' => $statistics ? true : false
        ) );
    }

    /**
     * Sales agent area subagents
     * 
     * @param int $current_page Current page number used in pagination
     */
    public function output_subagents_page( $current_page ) {
        $current_page = empty( $current_page ) ? 1 : absint( $current_page );
        if ( $interactive = apply_filters( 'wcb2bsa_enable_interactive_dashboard', true ) ) {
            $current_page = false;
        }
        $subagents = wcb2bsa_get_subagents_by_sales_agent( get_current_user_id(), $current_page );
        wc_get_template( 'myaccount/subagents.php', array(
            'interactive'   => $interactive,
            'subagents'     => $subagents,
            'has_subagents' => $subagents->total_users > 0 ? true : false
        ) );
        if ( ! $interactive ) {
            wc_get_template( 'global/pagination.php', array(
                'pages'        => ceil( $subagents->total_users / apply_filters( 'wcb2bsa_dashboard_pagination', 20 )),
                'current_page' => $current_page
            ) );
        }

        if ( 'yes' == get_option( 'wcb2bsa_allow_new_subagents', false ) ) {
            wc_get_template( 'myaccount/form-new-subagent.php' );
        }

    }

    /**
     * Sales agent area single customer details
     * 
     * @param int $customer_id Current user ID
     */
    public function output_view_customer_page( $customer_id ) {
        $customer = new WC_Customer( $customer_id );
        $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $customer_id );
        wc_get_template( 'myaccount/view-customer.php', array(
            'customer'      => $customer,
            'has_permission'=> wcb2bsa_can_read( $sales_agent_id, get_current_user_id() )
        ) );
    }

    /**
     * Sales agent area single order details
     * 
     * @param int $order_id Current order ID
     */
    public function output_view_order_page( $order_id ) {
        $order = new WC_Order( $order_id );
        $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent', true );
        wc_get_template( 'myaccount/view-order.php', array(
            'order'         => $order,
            'has_permission'=> wcb2bsa_can_read( $sales_agent_id, get_current_user_id() )
        ) );

        if ( 'yes' == get_option( 'wcb2bsa_update_order_status', false ) ) {
            wc_get_template( 'myaccount/form-update-order-status.php', array(
                'order' => $order
            ) );
        }

        if ( 'yes' == get_option( 'wcb2bsa_add_order_note' ) ) {
            wc_get_template( 'myaccount/form-add-order-note.php', array(
                'order' => $order
            ) );
        }
    }
    /**
     * Sales agent area single subagent details
     * 
     * @param int $subagent_id Current subagent ID
     */
    public function output_view_subagent_page( $subagent_id ) {
        $subagent = new WC_Customer( $subagent_id );
        wc_get_template( 'myaccount/view-subagent.php', array(
            'subagent'      => $subagent,
            'customers'     => wcb2bsa_get_customers_by_sales_agent( $subagent_id ),
            'orders'        => wcb2bsa_get_orders_by_sales_agent( $subagent_id ),
            'has_permission'=> in_array( $subagent_id, wcb2bsa_get_subagents_hierarchy_by_sales_agent( get_current_user_id() ) )
        ) );
    }

    /**
     * Sales agent area personal profile
     */
    public function output_edit_account_page() {
        wc_get_template( 'myaccount/edit-account.php', array(
            'sales_agent' => get_user_by( 'id', get_current_user_id() ),
            'payment_methods' => wcb2bsa_get_payment_methods()
        ) );
    }

    /**
     * Force WooCommerce shortcode in sales agents area
     *
     * @param string $content The page content
     * @return string
     */
    public function disable_pagebuilder_on_sales_agents_area( $content ) {
        if ( get_the_ID() == get_option( 'wcb2bsa_dashboard_page' ) && is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
            return '[woocommerce_my_account]';
        }   
        return $content;
    }

    /**
     * New customers registration - default
     *
     * @param integer $customer_id Customer ID
     * @param array $new_customer_data All new customer data
     * @param boolean $password_generated Password generated
     */
    public function new_customer( $customer_id, $new_customer_data, $password_generated ) {
        if ( is_user_logged_in() ) { return true; }
        if ( isset( $_POST['wcb2bsa_role'] ) && 'sales_agent' == $_POST['wcb2bsa_role'] ) { return true; }

        $sales_agent_id = get_option( 'wcb2bsa_default_sales_agent', false );
        // Random
        if ( 'random' == $sales_agent_id ) {
            $sales_agents = wcb2bsa_get_sales_agents();
            if ( $sales_agents->get_total() ) {
                $sales_agents = $sales_agents->get_results();
                $sales_agent_id = $sales_agents[array_rand( $sales_agents )]->data->ID;
            }
        }
        if ( 'choose' == $sales_agent_id ) {
            // Do nothing, the checkout class takes care of it! 
        }
        if ( is_numeric( $sales_agent_id ) ) {
            update_user_meta( $customer_id, 'wcb2bsa_sales_agent', $sales_agent_id );

            // Default B2B group support for sales agents
            $this->set_default_group( $customer_id, $sales_agent_id );
        }
    }

    /**
     * New customer registration - from sales agent
     *
     * @param integer $customer_id Customer ID
     * @param array $new_customer_data All new customer data
     * @param boolean $password_generated Password generated
     */
    public function new_customer_from_sales_agent( $customer_id, $new_customer_data, $password_generated ) {
        if ( is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
            // Delay notices and update data to have query vars endpoint available 
            add_action( 'wp', function() use ( $customer_id ) {
                // Registration is from dashboard customers page endpoint
                if ( is_wc_endpoint_url( 'sales-agent-customers' ) ) {
                    $sales_agent_id = get_current_user_id();
                    // Assign sales agent
                    update_user_meta( $customer_id, 'wcb2bsa_sales_agent', $sales_agent_id );

                    // Default B2B group support for sales agents
                    $this->set_default_group( $customer_id, $sales_agent_id );
                    // Custom notice
                    wc_clear_notices();
                    // Check autoswitch checkbox
                    if ( isset( $_POST['autoswitch'] ) ) {
                        $this->switch_to_id( $customer_id );
                    } else {
                        wc_add_notice( __( 'Customer account was created successfully and a password has been sent to his email address.', 'woocommerce-b2b-sales-agents' ), 'success' );
                    }
                }
            } );

            // On registration, prevent autologin
            add_filter( 'woocommerce_registration_auth_new_customer', '__return_false', 99 );
        }
    }

    /**
     * New subagent registration - from sales agent
     *
     * @param integer $subagent_id Subagent ID
     * @param array $new_subagent_data All new subagent data
     * @param boolean $password_generated Password generated
     */
    public function new_subagent_from_sales_agent( $subagent_id, $new_subagent_data, $password_generated ) {
        if ( is_user_logged_in() && wcb2bsa_has_role( get_current_user_id(), 'sales_agent' ) ) {
            // Delay notices and update data to have query vars endpoint available 
            add_action( 'wp', function() use ( $subagent_id ) {
                // Registration is from dashboard subagents page endpoint
                if ( is_wc_endpoint_url( 'sales-agent-subagents' ) ) {
                    $sales_agent_id = get_current_user_id();
                    // Assign role
                    wp_update_user( array(
                        'ID'    => $subagent_id,
                        'role'  => 'sales_agent'
                    ) );
                    // Assign sales agent manager
                    update_user_meta( $subagent_id, 'wcb2bsa_sales_agent_parent', $sales_agent_id );
                    // Custom notice
                    wc_clear_notices();
                    wc_add_notice( __( 'Subagent account was created successfully and a password has been sent to his email address.', 'woocommerce-b2b-sales-agents' ), 'success' );
                }
            } );

            // On registration, prevent autologin
            add_filter( 'woocommerce_registration_auth_new_customer', '__return_false', 99 );
        }
    }

    /**
     * Send new registration notice to admin by email
     * 
     * @param int $user_id Current user ID
     */
    public function notify_admin( $user_id ) {
        if ( wcb2bsa_has_role( $user_id, 'sales_agent' ) ) {
            wp_new_user_notification( $user_id, null, 'admin' );
        }
    }

    /**
     * New customer registration - by affiliation
     *
     * @param integer $customer_id Customer ID
     * @param array $new_customer_data All new customer data
     * @param boolean $password_generated Password generated
     */
    public function new_customer_from_affiliation( $customer_id, $new_customer_data, $password_generated ) {
        $parameter = apply_filters( 'wcb2bsa_affiliate_parameter', 'wcb2bsa_affiliate' );
        if ( ! is_admin() && isset( $_COOKIE[$parameter] ) ) {
            $sales_agent_id = $_COOKIE[$parameter];
            if ( wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) {
                update_user_meta( $customer_id, 'wcb2bsa_sales_agent', $sales_agent_id );

                // Default B2B group support for sales agents
                $this->set_default_group( $customer_id, $sales_agent_id );
            }
        }
    }

    /**
     * WooCommerce B2B compatibility, assign default group to new customers
     *
     * @param integer $$customer_id Customer ID
     * @param integer $sales_agent_id Sales agent ID
     */
    public function set_default_group( $customer_id, $sales_agent_id ) {
        if ( wcb2b_is_active() ) {
            $default_group_id = get_the_author_meta( 'wcb2bsa_customers_default_group', $sales_agent_id );
            if ( ! empty( $default_group_id ) && 'choose' != $default_group_id ) {
                update_user_meta( $customer_id, 'wcb2b_group', $default_group_id );

                // To fix caches
                wc_delete_product_transients();
            }
        }
    }

    /**
     * Force all fields to be unrequired in extended registration forms in sales agents area
     * 
     * @param array $fields Fields parameters
     * @return array
     */
    public function force_not_required_fields( $fields ) {
        if ( is_wc_endpoint_url( 'sales-agent-customers' ) ) {
            foreach ( $fields as $field => $parameters ) {
                $fields[$field]['required'] = false;
            }
        }
        if ( is_wc_endpoint_url( 'sales-agent-subagents' ) ) {
            foreach ( $fields as $field => $parameters ) {
                $fields[$field]['required'] = false;
            }
        }
        return $fields;
    }

    /**
     * Add WooCommerce scripts required for extended form
     */
    public function add_dependencies_scripts() {
        WC_Frontend_Scripts::load_scripts();
        wp_enqueue_script( 'wc-country-select' );
        wp_enqueue_script( 'wc-address-i18n' );
        wp_enqueue_script( 'wc-checkout' );
        wp_localize_script( 'wc-country-select', 'wc_country_select_params', array(
            'countries'                 => wp_json_encode( array_merge( WC()->countries->get_allowed_country_states(), WC()->countries->get_shipping_country_states() ) ),
            'i18n_select_state_text'    => esc_attr__( 'Select an option&hellip;', 'woocommerce-b2b-sales-agents' ),
            'i18n_no_matches'           => _x( 'No matches found', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_ajax_error'           => _x( 'Loading failed', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_input_too_short_1'    => _x( 'Please enter 1 or more characters', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_input_too_short_n'    => _x( 'Please enter %qty% or more characters', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_input_too_long_1'     => _x( 'Please delete 1 character', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_input_too_long_n'     => _x( 'Please delete %qty% characters', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_selection_too_long_1' => _x( 'You can only select 1 item', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_selection_too_long_n' => _x( 'You can only select %qty% items', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_load_more'            => _x( 'Loading more results&hellip;', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
            'i18n_searching'            => _x( 'Searching&hellip;', 'enhanced select', 'woocommerce-b2b-sales-agents' ),
        ) );

        wp_localize_script( 'wc-address-i18n', 'wc_address_i18n_params', array(
            'locale'             => wp_json_encode( WC()->countries->get_country_locale() ),
            'locale_fields'      => wp_json_encode( WC()->countries->get_country_locale_field_selectors() ),
            'i18n_required_text' => esc_attr__( 'required', 'woocommerce-b2b-sales-agents' ),
            'i18n_optional_text' => esc_html__( 'optional', 'woocommerce-b2b-sales-agents' ),
        ) );

        wp_localize_script( 'wc-checkout', 'wc_checkout_params', array(
            'ajax_url'                  => WC()->ajax_url(),
            'wc_ajax_url'               => WC_AJAX::get_endpoint( '%%endpoint%%' ),
            'update_order_review_nonce' => wp_create_nonce( 'update-order-review' ),
            'apply_coupon_nonce'        => wp_create_nonce( 'apply-coupon' ),
            'remove_coupon_nonce'       => wp_create_nonce( 'remove-coupon' ),
            'option_guest_checkout'     => get_option( 'woocommerce_enable_guest_checkout' ),
            'checkout_url'              => WC_AJAX::get_endpoint( 'checkout' ),
            'is_checkout'               => is_checkout() && empty( $wp->query_vars['order-pay'] ) && ! isset( $wp->query_vars['order-received'] ) ? 1 : 0,
            'debug_mode'                => WP_DEBUG,
            'i18n_checkout_error'       => esc_attr__( 'Error processing checkout. Please try again.', 'woocommerce-b2b-sales-agents' ),
        ) );

        wp_print_scripts( array( 'wc-country-select', 'wc-address-i18n', 'wc-checkout' ) );
    }

    /**
     * Add extended fields to customer/subagent registration in sales agents area
     */
    public function add_extended_fields( $type ) {
        $checkout = WC_Checkout::instance();
        $fields = $this->get_fields( $checkout );

        foreach ( $fields as $key => $field ) {
            // Prevent email field replication
            if ( $key == 'billing_email' ) { continue; }

            if ( isset( $field['country_field'], $fields[ $field['country_field'] ] ) ) {
                $field['country'] = $checkout->get_value( $field['country_field'] );
            }
            wcb2bsa_form_field( $type, $key, $field );
        }

        // WCB2B compatibility: Allow to choose B2B group support for sales agents
        if ( 'customer' == $type && wcb2b_is_active() ) {
            if ( 'choose' == get_the_author_meta( 'wcb2bsa_customers_default_group', get_current_user_id() ) ) {
                $groups = wcb2b_get_groups();
                if ( $groups->found_posts ) {
                    wcb2bsa_form_field( $type, 'wcb2b_group', array(
                        'type'          => 'select',
                        'class'         => array( 'form-group' ),
                        'input_class'   => array( 'form-control' ),
                        'label'         => esc_html__( 'Choose customer group', 'woocommerce-b2b-sales-agents' ),
                        'required'      => false,
                        'options'       => array_reduce( $groups->posts, function( $result, $item ) {
                            $item = (array)$item;
                            $result[$item['ID']] = $item['post_title'];
                            return $result;
                        } )
                    ) );
                }
            }
        }
    }

    /**
     * Validate extended registration fields in sales agents area
     * 
     * @param string $username Post registering username
     * @param string $email Post registering email
     * @param array $errors List of errors
     * @return array
     */
    public function validate_extended_fields( $username, $email, $errors ) {
        if ( defined( 'REST_REQUEST' ) ) { return $errors; }

        $checkout = WC_Checkout::instance();
        $fields = $this->get_fields( $checkout );
        $data = $_POST;

        foreach ( $fields as $key => $field ) {
            if ( $key == 'billing_email' ) { continue; }

            if ( ! isset( $data[ $key ] ) ) {
                continue;
            }

            $required    = ! empty( $field['required'] );
            $format      = array_filter( isset( $field['validate'] ) ? (array) $field['validate'] : array() );
            $field_label = isset( $field['label'] ) ? $field['label'] : '';

            if ( in_array( 'postcode', $format ) ) {
                $country      = isset( $data[ 'billing_country' ] ) ? $data[ 'billing_country' ] : WC()->customer->{"get_billing_country"}();
                $data[ $key ] = wc_format_postcode( $data[ $key ], $country );

                if ( '' !== $data[ $key ] && ! WC_Validation::is_postcode( $data[ $key ], $country ) ) {
                    switch ( $country ) {
                        case 'IE':
                            /* translators: %1$s: field name, %2$s finder.eircode.ie URL */
                            $postcode_validation_notice = sprintf( __( '%1$s is not valid. You can look up the correct Eircode <a target="_blank" href="%2$s">here</a>.', 'woocommerce-b2b-sales-agents' ), '<strong>' . esc_html( $field_label ) . '</strong>', 'https://finder.eircode.ie' );
                            break;
                        default:
                            /* translators: %s: field name */
                            $postcode_validation_notice = sprintf( __( '%s is not a valid postcode / ZIP.', 'woocommerce-b2b-sales-agents' ), '<strong>' . esc_html( $field_label ) . '</strong>' );
                    }
                    $errors->add( 'validation', apply_filters( 'woocommerce_checkout_postcode_validation_notice', $postcode_validation_notice, $country, $data[ $key ] ), array( 'id' => $key ) );
                }
            }

            if ( in_array( 'phone', $format ) ) {
                $data[ $key ] = wc_format_phone_number( $data[ $key ] );

                if ( '' !== $data[ $key ] && ! WC_Validation::is_phone( $data[ $key ] ) ) {
                    /* translators: %s: phone number */
                    $errors->add( 'validation', sprintf( __( '%s is not a valid phone number.', 'woocommerce-b2b-sales-agents' ), '<strong>' . esc_html( $field_label ) . '</strong>' ), array( 'id' => $key ) );
                }
            }

            if ( in_array( 'email', $format ) && '' !== $data[ $key ] ) {
                $data[ $key ] = sanitize_email( $data[ $key ] );
                if ( ! is_email( $data[ $key ] ) ) {
                    /* translators: %s: email address */
                    $errors->add( 'validation', sprintf( __( '%s is not a valid email address.', 'woocommerce-b2b-sales-agents' ), '<strong>' . esc_html( $field_label ) . '</strong>' ), array( 'id' => $key ) );
                    continue;
                }
            }

            if ( '' !== $data[ $key ] && in_array( 'state', $format ) ) {
                $country      = isset( $data[ 'billing_country' ] ) ? $data[ 'billing_country' ] : WC()->customer->{"get_billing_country"}();
                $valid_states = WC()->countries->get_states( $country );

                if ( ! empty( $valid_states ) && is_array( $valid_states ) && count( $valid_states ) > 0 ) {
                    $valid_state_values = array_map( 'wc_strtoupper', array_flip( array_map( 'wc_strtoupper', $valid_states ) ) );
                    $data[ $key ]       = wc_strtoupper( $data[ $key ] );

                    if ( isset( $valid_state_values[ $data[ $key ] ] ) ) {
                        // With this part we consider state value to be valid as well, convert it to the state key for the valid_states check below.
                        $data[ $key ] = $valid_state_values[ $data[ $key ] ];
                    }

                    if ( ! in_array( $data[ $key ], $valid_state_values ) ) {
                        /* translators: 1: state field 2: valid states */
                        $errors->add( 'validation', sprintf( __( '%1$s is not valid. Please enter one of the following: %2$s', 'woocommerce-b2b-sales-agents' ), '<strong>' . esc_html( $field_label ) . '</strong>', implode( ', ', $valid_states ) ), array( 'id' => $key ) );
                    }
                }
            }

            if ( $required && '' === $data[ $key ] ) {
                /* translators: %s: field name */
                $errors->add( 'required-field', apply_filters( 'woocommerce_checkout_required_field_notice', sprintf( __( '%s is a required field.', 'woocommerce-b2b-sales-agents' ), '<strong>' . esc_html( $field_label ) . '</strong>' ), $field_label ), array( 'id' => $key ) );
            }
        }
        return $errors;
    }

    /**
     * Save extended fields to customer/subagents registration in sales agents area
     * 
     * @param int $customer_id Current customer ID
     */
    public function save_extended_fields( $customer_id ) {
        if ( defined( 'REST_REQUEST' ) ) { return true; }

        $checkout = WC_Checkout::instance();
        $fields = $this->get_fields( $checkout );

        foreach ( $fields as $key => $field ) {
            // Prevent email field replication
            if ( $key == 'billing_email' ) { continue; }

            if ( ! isset( $_POST[$key] ) ) {
                continue;
            }

            update_user_meta( $customer_id, $key, sanitize_text_field( $_POST[$key] ) );
            if ( $key == 'billing_first_name' ) {
                update_user_meta( $customer_id, 'first_name', sanitize_text_field( $_POST[$key] ) );
            }
            if ( $key == 'billing_last_name' ) {
                update_user_meta( $customer_id, 'last_name', sanitize_text_field( $_POST[$key] ) );
            }
        }

        if ( isset( $_POST['wcb2b_group'] ) && $_POST['wcb2b_group'] ) {
            update_user_meta( $customer_id, 'wcb2b_group', intval( $_POST['wcb2b_group'] ) );
        }
    }

    /**
     * Get billing fields to add to new customer/subagents form in sales agents area
     *   
     * @return array
     */
    private function get_fields( $checkout ) {
        $fields = $checkout->get_checkout_fields( 'billing' );
        $fields = array_map( function( $field ) {
            $field['required'] = false;
            $field['input_class'] = array( 'form-control bg-light' );
            $field['class'] = array( 'form-group' );
            return $field;
        }, $fields );

        if ( is_wc_endpoint_url( 'sales-agent-customers' ) ) {
            return apply_filters( 'wcb2bsa_new_customer_form_fields', $fields );
        }
        if ( is_wc_endpoint_url( 'sales-agent-subagents' ) ) {
            return apply_filters( 'wcb2bsa_new_subagent_form_fields', $fields );
        }
        return $fields;
    }

    /**
     * Add switch to customer bar
     */
    public function add_switchbar() {
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                wc_get_template( 'global/switchbar.php', array(
                    'customer'      => get_userdata( get_current_user_id() ),
                    'switchto_id'   => $switchto_id
                ) );
            }
        }
    }

    /**
     * Handle sales agent role field in registration
     */
    public function add_custom_fields() {
        if ( is_page( get_option( 'wcb2bsa_dashboard_page' ) ) && ! is_account_page() ) {
            woocommerce_form_field( 'wcb2bsa_role', array(
                'type' => 'hidden',
                'required' => true,
                'custom_attributes' => array(
                    'readonly' => true
                )
            ), 'sales_agent' );
        }
    }

    /**
     * Add status to sales agent on new registration
     *
     * @param array $data User data
     */
    public function update_user_data( $data ) {
        if ( isset( $_POST['wcb2bsa_role'] ) && 'sales_agent' == $_POST['wcb2bsa_role'] ) {
            // Enabled
            $data['role'] = 'sales_agent';
            $data['meta_input']['wcb2bsa_status'] = (int)('yes' !== get_option( 'wcb2bsa_moderate_registration', false ));
        }
        return $data;
    }

    /**
     * Prevent login in wp-login page
     * 
     * @param object $user Current user instance
     * @param string $username Current user username
     * @return object
     */
    public function prevent_login( $user, $username ) {
        if ( ! is_null( $user ) && ! is_wp_error( $user ) ) {
            // If user isn't a sales_agent, skip. Only customers could be inactive
            if ( wcb2bsa_has_role( $user->ID, 'sales_agent' ) ) {
                $status = get_the_author_meta( 'wcb2bsa_status', $user->ID );
                if ( 0 == (int)$status ) {
                    $message = __( 'Your account is now under review. You will receive an email as soon as it is activated.', 'woocommerce-b2b-sales-agents' );
                    $user = new WP_Error( 'authentication_failed', $message );
                }
            }
        }
        return $user;
    }

    /**
     * Prevent autologin on new registration
     * 
     * @param boolean $auth Prevent autologin?
     * @param integer $sales_agent_id Current sales agent ID
     * @return boolean
     */
    public function prevent_autologin( $auth, $sales_agent_id ) {
        if ( ! is_null( $sales_agent_id ) && ! is_wp_error( $sales_agent_id ) ) {
            // If user isn't a customer, skip. Only customers could be inactive
            if ( wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) {
                $status = get_the_author_meta( 'wcb2bsa_status', $sales_agent_id );
                if ( 0 == (int)$status ) {
                    $message = __( 'Your account is now under review. You will receive an email as soon as it is activated.', 'woocommerce-b2b-sales-agents' );
                    wc_add_notice( apply_filters( 'wcb2bsa_waiting_approvation', $message ), 'notice' );
                    return false;
                }
            }
        }
        return $auth;
    }

    /**
     * Handle switch between sales agent and customer
     */
    public function switch_to_id( $switchto_id = false ) {
        if ( is_user_logged_in() ) {
            if ( false == $switchto_id ) {
                $switchto_id = isset( $_POST['wcb2bsa_switch_to'] ) ? base64_decode( $_POST['wcb2bsa_switch_to'] ) : false;
            }
            if ( $switchto_id ) {
                $can_switch = false;
                $user_id = get_current_user_id();

                if ( wcb2bsa_has_role( $user_id, 'sales_agent' ) ) {
                    $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $switchto_id );
                    $can_switch = wcb2bsa_can_write( $sales_agent_id, $user_id );
                    if ( $can_switch ) {
                        $redirect = apply_filters( 'wcb2bsa_switch_to_customer_redirect', wc_get_page_permalink( 'shop' ) );
                        set_transient( 'wcb2bsa_switched_' . $switchto_id, $user_id, 86400 );
                        setcookie( 'wcb2bsa_switched', 'true', 0, '/', COOKIE_DOMAIN, is_ssl(), true );
                    }
                }

                if ( wcb2bsa_has_role( $user_id, 'customer' ) ) {
                    $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $user_id );
                    $can_switch = wcb2bsa_can_write( $sales_agent_id, $switchto_id );
                    if ( $can_switch ) {
                        $redirect = apply_filters( 'wcb2bsa_switch_to_sales_agent_redirect', get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) );
                        delete_transient( 'wcb2bsa_switched_' . $user_id );
                        setcookie( 'wcb2bsa_switched', '', time() - 3600, '/' );
                    }
                }

                if ( $can_switch && isset( $redirect ) ) {
                    wp_set_current_user( $switchto_id );
                    wp_set_auth_cookie( $switchto_id );
                    WC()->cart->empty_cart( apply_filters( 'wcb2bsa_empty_cart_on_switch', true ) );
                    wp_redirect( $redirect );
                    exit;
                } else {
                    wc_add_notice( __( 'Sorry, your profile is not authorized to make orders for this customer', 'woocommerce-b2b-sales-agents' ), 'error' );
                }
            }
        }
    }

    /**
     * Clear switch transient on logout
     */
    public function clear_cookie() {
        delete_transient( 'wcb2bsa_switched_' . get_current_user_id() );
        setcookie( 'wcb2bsa_switched', '', time() - 3600, '/' );
    }

    /**
     * Set affiliation cookie
     */
    public function set_affiliate_cookie() {
        // Check for token
        $parameter = apply_filters( 'wcb2bsa_affiliate_parameter', 'wcb2bsa_affiliate' );
        if ( isset( $_GET[$parameter] ) ) {
            $users = get_users(array(
                'meta_key' => 'wcb2bsa_affiliate_id',
                'meta_value' => sanitize_text_field( $_GET[$parameter] )
            ));
            if ( 1 == count( $users ) ) {
                $duration = apply_filters( 'wcb2bsa_affiliate_cookie_duration', 3600 );
                setcookie( $parameter, $users[0]->ID, time() + $duration, '/' );
            }
        }
    }

    /**
     * Change order status and display a message
     */
    public function process_update_order_status() {
        if ( isset( $_POST['order_id'], $_POST['update-order-status-nonce'] ) ) {
            // Check security nonce
            if ( ! wp_verify_nonce( sanitize_key( $_POST['update-order-status-nonce'] ), 'update-order-status' ) ) {
                wc_add_notice( __( 'An error has occurred, please try again', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }
            $order = wc_get_order( absint( $_POST['order_id'] ) );
            if ( ! is_a( $order, 'WC_Order' ) ) {
                wc_add_notice( __( 'An error has occurred processing order, please try again', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }

            $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent', true );
            if ( ! wcb2bsa_can_write( $sales_agent_id, get_current_user_id() ) ) {
                wc_add_notice( __( 'This order is not from a customer assigned to you', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }

            if ( ! $order->update_status( $_POST['order_status'], __( 'Order updated by sales agent', 'woocommerce-b2b-sales-agents') ) ) {
                wc_add_notice( __( 'Error updating status for order, please try again', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }
            wc_add_notice( __( 'Order has been updated', 'woocommerce-b2b-sales-agents' ), 'success' );
        }
    }

    /**
     * Add order notes
     */
    public function process_add_order_note() {
        if ( isset( $_POST['order_id'], $_POST['add-order-note-nonce'] ) ) {
            // Check security nonce
            if ( ! wp_verify_nonce( sanitize_key( $_POST['add-order-note-nonce'] ), 'add-order-note' ) ) {
                wc_add_notice( __( 'An error has occurred, please try again', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }
            $order = wc_get_order( absint( $_POST['order_id'] ) );
            if ( ! is_a( $order, 'WC_Order' ) ) {
                wc_add_notice( __( 'An error has occurred processing order, please try again', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }
            if ( $order->get_meta( 'wcb2bsa_sales_agent', true ) != get_current_user_id() ) {
                wc_add_notice( __( 'This order is not from a customer assigned to you', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }

            if ( ! $order->add_order_note( $_POST['order_note'], $_POST['order_note_type'], true ) ) {
                wc_add_notice( __( 'Error adding note to order, please try again', 'woocommerce-b2b-sales-agents' ), 'error' );
                return false;
            }
            wc_add_notice( __( 'Note has been added', 'woocommerce-b2b-sales-agents' ), 'success' );
        }
    }

    /**
     * Add sales agent data in myaccount
     */
    public function sales_agent_in_myaccount() {
        if ( wcb2bsa_has_role( get_current_user_id(), 'customer' ) ) {
            if ( $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', get_current_user_id() ) ) {
                if ( $sales_agent = get_userdata( $sales_agent_id ) ) {
                    $template = wcb2bsa_locate_template( 'myaccount/sales-agent-data.php' );
                    include $template;
                }
            }
        }
    }

}

return new WCB2BSA_Frontend_Myaccount();