<?php

/**
 * WooCommerce B2B Sales Agents Common hooks set-up
 *
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Hooks Class
 */
class WCB2BSA_Hooks {

    /**
     * Constructor
     */
    public function __construct() {
        $this->includes();
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'option_wcb2bsa_has_role_customer', array( $this, 'has_role_customer' ), 99, 2 );
        add_filter( 'option_wcb2bsa_has_role_sales_agent', array( $this, 'has_role_sales_agent' ), 99, 2 );
        add_filter( 'is_protected_meta', array( $this, 'hide_protected_meta' ), 10, 3 );
        add_filter( 'init', array( $this, 'add_endpoints_rewrite' ), 10 );
        add_action( 'woocommerce_new_order_item', array( $this, 'save_order_item_meta' ), 10, 3 );
        add_action( 'woocommerce_after_order_object_save', array( $this, 'save_total_commission' ), 10, 2 );
        add_filter( 'woocommerce_order_item_get_formatted_meta_data', array( $this, 'remove_item_commissions' ), 10, 2 );
        add_filter( 'woocommerce_admin_settings_sanitize_option', array( $this, 'sanitize_settings' ), 10, 3 );
        add_filter( 'woocommerce_payment_gateways', array( $this, 'add_waiting_gateway' ) );
        add_action( 'woocommerce_register_shop_order_post_statuses', array( $this, 'register_waiting_status' ) );
        add_filter( 'wc_order_statuses', array( $this, 'add_waiting_status' ) );
        add_filter( 'wc_order_is_editable', array( $this, 'make_waiting_order_editable' ), 10, 2 );
        add_filter( 'woocommerce_email_actions', array( $this, 'set_emails_actions' ) );
        add_filter( 'woocommerce_email_classes', array( $this, 'set_waiting_order_emails' ) );
        add_filter( 'woocommerce_email_recipient_new_order', array( $this, 'prevent_new_order_email' ), 20, 2 );
        add_filter( 'woocommerce_valid_order_statuses_for_payment', array( $this, 'waiting_order_can_be_payed' ) );
        add_filter( 'woocommerce_valid_order_statuses_for_cancel', array( $this, 'waiting_order_can_be_cancelled' ) );
        add_filter( 'woocommerce_email_classes', array( $this, 'pending_account_email' ) );
        add_action( 'woocommerce_email_footer', array( $this, 'new_account_email_message' ) );

        if ( get_option( 'wcb2bsa_email_data' ) === 'yes' ) {
            add_action( 'woocommerce_email_customer_details', array( $this, 'sales_agent_in_email' ), 99, 4 );
        }
    }

    /**
     * Include any classes we need
     */
    public function includes() {
        include_once WC()->plugin_path() . '/includes/admin/reports/class-wc-admin-report.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/reports/class-wcb2bsa-report-sales-by-sales-agent.php';

        add_action( 'plugins_loaded', function() {
            include_once WCB2BSA_ABSPATH . 'includes/classes/gateways/class-wcb2bsa-payment-gateway-waiting.php';
        }, 11 );
    }

    /**
     * Apply hook to consider other roles as "customer"
     * 
     * @param mixed $pre_option The value to return instead of the option value
     * @param string $option_name Option name
     * @param mixed $default The fallback value to return if the option does not exist
     * @return mixed
     */
    public function has_role_customer( $value, $option ) {
        return apply_filters( 'wcb2bsa_has_role_customer', $value );
    }

    /**
     * Apply hook to consider other roles as "sales_agent"
     * 
     * @param mixed $pre_option The value to return instead of the option value
     * @param string $option_name Option name
     * @param mixed $default The fallback value to return if the option does not exist
     * @return mixed
     */
    public function has_role_sales_agent( $value, $option ) {
        return apply_filters( 'wcb2bsa_has_role_sales_agent', $value );
    }

    /**
     * Hide protected meta$protected, $meta_key, $meta_type
     *
     * @param boolean $protected Is meta protected?
     * @param string $meta_key Meta key name
     * @param string $meta_type Meta type
     * @return boolean
     */
    public function hide_protected_meta( $protected, $meta_key, $meta_type ) {
        $meta = array(
            'wcb2bsa_created_by',
            'wcb2bsa_sales_agent',
            'wcb2bsa_min_price',
            'wcb2bsa_commissions',
            'wcb2bsa_commission',
            'wcb2bsa_commission_amount',
            'wcb2bsa_order_item_types_eligible_for_commission'
        );
        if ( in_array( $meta_key, $meta ) ) {
            $protected = true;
        }
        return $protected;
    }

    public function add_endpoints_rewrite() {
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_subagents_endpoint', 'sales-agent-subagents' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_view_subagent_endpoint', 'sales-agent-view-subagent' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_customers_endpoint', 'sales-agent-customers' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_view_customer_endpoint', 'sales-agent-view-customer' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_orders_endpoint', 'sales-agent-orders' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_view_order_endpoint', 'sales-agent-view-order' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_earnings_endpoint', 'sales-agent-earnings' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_commissions_endpoint', 'sales-agent-commissions' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_notifications_endpoint', 'sales-agent-notifications' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_statistics_endpoint', 'sales-agent-statistics' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_edit_account_endpoint', 'sales-agent-edit-account' ), EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( get_option( 'wcb2bsa_account_logout_endpoint', 'sales-agent-logout' ), EP_ROOT | EP_PAGES );
        flush_rewrite_rules();
    }

    /**
     * On new order item saving, add custom meta data
     * 
     * @param int $item_id Order item ID
     * @param WC_Order_Item $item Order item class instance
     * @param int $order_id Current order ID
     */
    public function save_order_item_meta( $item_id, $item, $order_id ) {
        $order = new WC_Order( $order_id );

        $sales_agent_id = 0;
        if ( ! empty( $order->get_customer_id() ) ) {
            $sales_agent_id = get_the_author_meta( 'wcb2bsa_sales_agent', $order->get_customer_id() );
        }
        if ( ! empty( $_POST['wcb2bsa_sales_agent'] ) ) {
            $sales_agent_id = (int)$_POST['wcb2bsa_sales_agent'];
        } elseif ( is_admin() ) {
            $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent', true );
        }

        $order_item_types_eligible_for_commission = get_post_meta( $order_id, 'wcb2bsa_order_item_types_eligible_for_commission', true );
        if ( $order_item_types_eligible_for_commission && isset($order_item_types_eligible_for_commission[$item->get_type()]) ) {

            // Retrieve commission percentage (product/global) and save
            $item_commission = wcb2bsa_get_item_commission( $item, $sales_agent_id );
            wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission', $item_commission );

            // Calculate commission amount and save
            $item_commission_amount = wcb2bsa_order_item_commission_amount_evaluate( $item, $order );
            wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount', $item_commission_amount );

            // Walk hierarchy (ascendant)
            $parents = array();
            $parents = wcb2bsa_get_parents_hierarchy_by_sales_agent( $sales_agent_id, $parents );
            foreach ( $parents as $parent_id ) {
                // Retrieve parent subcommission percentage and save
                $item_subcommission = get_the_author_meta( 'wcb2bsa_subcommission', $parent_id );
                wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_' . $parent_id, $item_subcommission );

                // Calculate parent subcommission amount and save
                $item_subcommission_amount = $item_commission_amount * (float)$item_subcommission / 100;
                wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount_' . $parent_id, $item_subcommission_amount );
                
                // Subtracts current subcommission amout from total commission amount
                $item_commission_amount -= $item_subcommission_amount;
            }
            // Save primary sales agent remaining commission and percentage
            wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_' . $sales_agent_id, $item_commission );
            wc_update_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount_' . $sales_agent_id, $item_commission_amount );
        }
    }

    /**
     * Calculate and save total order commission amount
     * 
     * @param object $order Current order instance
     * @param array $data Current order data
     */
    public function save_total_commission( $order, $data ) {
        $commissions = wcb2bsa_get_order_commission_amount_by_sales_agent( $order->get_id() );
        update_post_meta( $order->get_id(), 'wcb2bsa_commission_amount', $commissions );
    }

    /**
     * Hiding item meta commission
     * 
     * @param array $formatted_meta Array of all formatted meta.
     * @return array
     */
    public function remove_item_commissions( $formatted_meta ) {
        if ( is_wc_endpoint_url() ) { return $formatted_meta; }

        foreach ( $formatted_meta as $key => $meta ) {
            if ( in_array( $meta->key, array( 'wcb2bsa_custom_price' ) ) ) {
                unset( $formatted_meta[$key] );
            }
            if ( 0 === stripos( $meta->key, 'wcb2bsa_item_commission' ) ) {
                unset( $formatted_meta[$key] );
            }
        }
        return $formatted_meta;
    }

    /**
     * Sanitize multicheckbox settings
     *
     * @param mixed $value The value
     * @param array $option Option data
     * @param mixed $raw_value Raw value
     * @return mixed
     */
    public function sanitize_settings( $value, $option, $raw_value ) {
        if ( 'multicheckbox' == $option['type'] ) {
            if ( is_null( $value ) ) {
                return array();
            }
        }
        return $value;
    }
    /**
     * Add waiting payment gateway
     * 
     * @param array $gateways All payment gateways
     * @return array
     */
    public function add_waiting_gateway( $gateways ) {
        if ( ! is_wc_endpoint_url( 'order-pay' ) ) {
            if ( is_user_logged_in() ) {
                $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
                if ( is_admin() || ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) ) {
                    $gateways[] = 'WCB2BSA_Gateway_Waiting';
                }
            }
        }
        return $gateways;
    }

    /**
     * Register waiting payment status
     * 
     * @param array $order_statuses All statuses settings list
     * @return array
     */
    public function register_waiting_status( $order_statuses ) {
        $order_statuses['wc-waiting'] = array(
            'label'                     => __( 'Waiting payment', 'woocommerce-b2b-sales-agents' ),
            'public'                    => true,
            'show_in_admin_status_list' => true,
            'show_in_admin_all_list'    => true,
            'exclude_from_search'       => false,
            'label_count'               => _n_noop( 'Waiting payment <span class="count">(%s)</span>', 'Waiting payment <span class="count">(%s)</span>', 'woocommerce-b2b-sales-agents' )
        );
        return $order_statuses;
    }

    /**
     * Add new statuses to list
     * 
     * @param array $order_statuses All statuses list
     * @return array
     */
    public function add_waiting_status( $order_statuses ) {
        $order_statuses['wc-waiting'] = __( 'Waiting payment', 'woocommerce-b2b-sales-agents' );
        return $order_statuses;
    }

    /**
     * Make waiting payment status editable
     * 
     * @param boolean $editable Order can be edit?
     * @param object $order Current order instance
     * @return boolean
     */
    public function make_waiting_order_editable( $editable, $order ) {
        if ( $order->get_status() == 'waiting' ) {
            return true;
        }
        return $editable;
    }

    /**
     * Add transactional email actions for waiting payment
     * 
     * @param array $emails Emails settings
     * @return array
     */
    public function set_emails_actions( $emails ) {
        $emails[] = 'woocommerce_order_status_waiting';
        return $emails;
    }

    /**
     * Add transactional email for waiting payment
     * 
     * @param array $emails Emails settings
     * @return array
     */
    public function set_waiting_order_emails( $emails ) {
        $emails['WCB2BSA_Email_Customer_Waiting_Order']   = include WCB2BSA_ABSPATH . 'includes/classes/emails/class-wcb2bsa-email-customer-waiting-order.php';
        return $emails;
    }

    /**
     * Prevent sending new order email if is a waiting payment
     * 
     * @param string $recipient Email recipient
     * @param object $order Current order object
     * @return string
     */
    public function prevent_new_order_email( $recipient, $order ) {
        if ( is_object( $order ) && $order->has_status( 'waiting' ) ) {
            return '';
        }
        return $recipient;
    }

    /**
     * Mark waiting payment as payable
     * 
     * @param array $statuses Payable statuses
     * @return array
     */
    public function waiting_order_can_be_payed( $statuses ) {
        $statuses[] = 'waiting';
        return $statuses;
    }

    /**
     * Mark waiting payment as cancellable
     * 
     * @param array $statuses Cancellable statuses
     * @return array
     */
    public function waiting_order_can_be_cancelled( $statuses ) {
        $statuses[] = 'waiting';
        return $statuses;
    }

    /**
     * Include email classes
     * 
     * @param array $emails Email classes list
     * @return array
     */
    public function pending_account_email( $emails ) {
        if ( ! array_key_exists( 'WCB2BSA_Email_Sales_Agent_Status_Notification', $emails ) ) {
            $emails['WCB2BSA_Email_Sales_Agent_Status_Notification'] = include_once WCB2BSA_ABSPATH . 'includes/classes/emails/class-wcb2bsa-email-sales_agent-status-notification.php';
        }

        return $emails;
    }

    /**
     * Add message into new account email to inform customers to wait activation
     * 
     * @param object $email Current email instance
     */
    public function new_account_email_message( $email ) {
        // Customize new account email
        if ( is_object( $email ) && $email->id == 'customer_new_account' ) {
            $status = get_the_author_meta( 'wcb2bsa_status', $email->object->ID );
            if ( 0 == (int)$status ) {
                echo '<p>' . apply_filters( 'wcb2bsa_new_account_email', __( 'We are checking your account, please wait for the activation confirmation email before trying to login', 'woocommerce-b2b-sales-agents' ) ) . '</p>';
            }
        }
    }

    /**
     * Add sales agent data in email
     * 
     * @param object $order Current order instance
     * @param int $sent_to_admin Is an admin email?
     * @param int $plain_text Is plain text email?
     * @param string $email Recipient
     */
    public function sales_agent_in_email( $order, $sent_to_admin, $plain_text, $email ) {
        $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent', true );
        $sales_agent = get_userdata( $sales_agent_id );

        $template = wcb2bsa_locate_template( '/emails/' . ( $plain_text ? 'plain/' : false ) . 'email-sales-agent.php' );
        include $template;
    }

}

return new WCB2BSA_Hooks();