<?php

/**
 * WooCommerce B2B Sales Agents Admin Post types set-up Class
 *
 * @version 1.2.1
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Post_Types Class
 */
class WCB2BSA_Post_Types {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_action( 'init', array( $this, 'register_notification_post_type' ), 0 );
        add_action( 'add_meta_boxes_wcb2bsa_notification', array( $this, 'add_metabox' ) );
        add_action( 'save_post_wcb2bsa_notification', array( $this, 'save_meta_box_fields' ) );

        // WCB2B compatibility
        add_action( 'add_meta_boxes_wcb2b_group', array( $this, 'add_metabox_group' ) );
        add_action( 'save_post_wcb2b_group', array( $this, 'save_meta_box_fields_group' ) );
    }

    /**
     * Register custom notification post type
     */
    public function register_notification_post_type() {
        $labels = array(
            'name'                  => _x( 'Notifications', 'Post Type General Name', 'woocommerce-b2b-sales-agents' ),
            'singular_name'         => _x( 'Notification', 'Post Type Singular Name', 'woocommerce-b2b-sales-agents' ),
            'menu_name'             => __( 'Notifications', 'woocommerce-b2b-sales-agents' ),
            'name_admin_bar'        => __( 'Notification', 'woocommerce-b2b-sales-agents' ),
            'archives'              => __( 'Notification archives', 'woocommerce-b2b-sales-agents' ),
            'attributes'            => __( 'Notification attributes', 'woocommerce-b2b-sales-agents' ),
            'parent_item_colon'     => __( 'Parent notification:', 'woocommerce-b2b-sales-agents' ),
            'all_items'             => __( 'All Notifications', 'woocommerce-b2b-sales-agents' ),
            'add_new_item'          => __( 'Add new notification', 'woocommerce-b2b-sales-agents' ),
            'add_new'               => __( 'Add new', 'woocommerce-b2b-sales-agents' ),
            'new_item'              => __( 'New notification', 'woocommerce-b2b-sales-agents' ),
            'edit_item'             => __( 'Edit notification', 'woocommerce-b2b-sales-agents' ),
            'update_item'           => __( 'Update notification', 'woocommerce-b2b-sales-agents' ),
            'view_item'             => __( 'View notification', 'woocommerce-b2b-sales-agents' ),
            'view_items'            => __( 'View notifications', 'woocommerce-b2b-sales-agents' ),
            'search_items'          => __( 'Search notification', 'woocommerce-b2b-sales-agents' ),
            'not_found'             => __( 'Not found', 'woocommerce-b2b-sales-agents' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'woocommerce-b2b-sales-agents' ),
            'featured_image'        => __( 'Featured image', 'woocommerce-b2b-sales-agents' ),
            'set_featured_image'    => __( 'Set featured image', 'woocommerce-b2b-sales-agents' ),
            'remove_featured_image' => __( 'Remove featured image', 'woocommerce-b2b-sales-agents' ),
            'use_featured_image'    => __( 'Use as featured image', 'woocommerce-b2b-sales-agents' ),
            'insert_into_item'      => __( 'Insert into notification', 'woocommerce-b2b-sales-agents' ),
            'uploaded_to_this_item' => __( 'Uploaded to this notification', 'woocommerce-b2b-sales-agents' ),
            'items_list'            => __( 'Notifications list', 'woocommerce-b2b-sales-agents' ),
            'items_list_navigation' => __( 'Notifications list navigation', 'woocommerce-b2b-sales-agents' ),
            'filter_items_list'     => __( 'Filter notifications list', 'woocommerce-b2b-sales-agents' ),
        );
        $args = array(
            'label'                 => __( 'Notification', 'woocommerce-b2b-sales-agents' ),
            'description'           => __( 'Notifications to share with sales agents', 'woocommerce-b2b-sales-agents' ),
            'labels'                => $labels,
            'supports'              => array( 'title', 'editor' ),
            'hierarchical'          => false,
            'public'                => false,
            'show_ui'               => true,
            'show_in_menu'          => 'users.php',
            'menu_position'         => 25,
            'menu_icon'             => false,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => false,
            'can_export'            => false,
            'has_archive'           => false,
            'exclude_from_search'   => true,
            'publicly_queryable'    => true,
            'rewrite'               => false,
            'capability_type'       => 'product',
        );
        register_post_type( 'wcb2bsa_notification', $args );
    }

    /**
     * Add notification post type metabox with custom fields
     * 
     * @param WP_Post $post The current WP_Post object
     */
    public function add_metabox( $post ) {
        // Recipient meta box
        add_meta_box( 'wcb2bsa_notification-recipient-meta_box', __( 'Display notification to', 'woocommerce-b2b-sales-agents' ), function( $post ) {
            // Make sure the form request comes from WordPress
            wp_nonce_field( basename( __FILE__ ), 'wcb2bsa_notification-recipients-nonce' );

            // Retrieve current value
            $recipients = get_post_meta( $post->ID, 'wcb2bsa_recipients', true );
            $sales_agents = wcb2bsa_get_sales_agents();

            if ( ! empty( $sales_agents->get_results() ) ) {
                foreach ( $sales_agents->get_results() as $sales_agent ) {
                    // Display fields
                    printf( '<p><input type="checkbox" name="wcb2bsa_recipients[]" value="%s" %s /> %s</p>',
                        $sales_agent->ID,
                        is_array( $recipients ) ? checked( in_array( $sales_agent->ID, $recipients ), true, false ) : false,
                        $sales_agent->display_name
                    );
                }
            }
        }, 'wcb2bsa_notification', 'side', 'low' );
    }

    /**
     * Store custom field meta box data
     * 
     * @param int $post_id The current post ID
     */
    public function save_meta_box_fields( $post_id ) {
        // Only ShopManager can edit
        if ( ! current_user_can( 'manage_woocommerce' ) ) { return; }

        // Verify meta box nonce
        if ( ! isset( $_POST['wcb2bsa_notification-recipients-nonce'] ) || ! wp_verify_nonce( $_POST['wcb2bsa_notification-recipients-nonce'], basename( __FILE__ ) ) ) { return; }
        
        // Return if autosave
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { return; }

        // Store notice recipients
        $recipients = isset( $_POST['wcb2bsa_recipients'] ) ? $_POST['wcb2bsa_recipients'] : '';
        update_post_meta( $post_id, 'wcb2bsa_recipients', $recipients );
    }

    /**
     * Add commission metabox to WCB2B group
     * 
     * @param WP_Post $post The current WP_Post object
     */
    public function add_metabox_group( $post ) {
        // Check if WCB2B is active
        if ( ! wcb2b_is_active() ) { return; }
        // Discount meta box
        add_meta_box( 'wcb2bsa_group-commission-meta_box', __( 'Sales agent commission (%)', 'woocommerce-b2b-sales-agents' ), function( $post ) {
            // Make sure the form request comes from WordPress
            wp_nonce_field( basename( __FILE__ ), 'wcb2bsa_group-commission-nonce' );

            // Retrieve group discount current value
            $group_commission = get_post_meta( $post->ID, 'wcb2bsa_group_commission', true );

            // Display fields
            printf( '<p><input type="text" name="wcb2bsa_group_commission" value="%s" pattern="[0-9]+([\.][0-9]+)?" title="' . __( 'This should be a number with up to 2 decimal places, with (.) as decimal separator', 'woocommerce-b2b-sales-agents' ) . '" /></p>',
                $group_commission
            );
        }, 'wcb2b_group', 'side', 'low' );
    }

    /**
     * Store custom field meta box data
     * 
     * @param int $post_id The current post ID
     */
    public function save_meta_box_fields_group( $post_id ) {
        // Check if WCB2B is active
        if ( ! wcb2b_is_active() ) { return; }
        // Only ShopManager can edit
        if ( ! current_user_can( 'manage_woocommerce' ) ) { return; }

        // Verify meta box nonce
        if ( ! isset( $_POST['wcb2bsa_group-commission-nonce'] ) || ! wp_verify_nonce( $_POST['wcb2bsa_group-commission-nonce'], basename( __FILE__ ) ) ) { return; }
        
        // Return if autosave
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { return; }

        // Store commissions
        $group_commission = isset( $_POST['wcb2bsa_group_commission'] ) ? $_POST['wcb2bsa_group_commission'] : '';
        update_post_meta( $post_id, 'wcb2bsa_group_commission', $group_commission );
    }

}

return new WCB2BSA_Post_Types();