<?php

/**
 * WooCommerce B2B Sales Agents REST API support
 *
 * @version 1.3.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * WCB2BSA_Rest_Controller Class
 */
class WCB2BSA_Rest_Controller {

    /**
     * Constructor
     */
    public function __construct() {
        $this->init_hooks();
    }

    /**
     * Init current class hooks
     */
    public function init_hooks() {
        add_filter( 'woocommerce_rest_prepare_customer', array( $this, 'add_customer_meta' ), 90, 2 );
        add_filter( 'woocommerce_rest_prepare_shop_order_object', array( $this, 'add_order_meta' ), 90, 2 );
        add_filter( 'woocommerce_rest_prepare_product_object', array( $this, 'add_product_meta' ), 90, 2 );
        add_filter( 'woocommerce_rest_prepare_product_variation_object', array( $this, 'add_product_meta' ), 90, 2 );
    }

    /**
     * Add customer fields to REST API
     *
     * @param object $response REST API response object
     * @param object $customer Current customer object
     * @return object
     */
    public function add_customer_meta( $response, $customer ) {
        // Sales agent
        if ( wcb2bsa_has_role( $customer->ID, 'customer' ) ) {
            $response->data['wcb2bsa_sales_agent'] = (int)get_user_meta( $customer->ID, 'wcb2bsa_sales_agent', true );
        }
        // Commission
        if ( wcb2bsa_has_role( $customer->ID, 'sales_agent' ) ) {
            $response->data['wcb2bsa_status'] = get_user_meta( $customer->ID, 'wcb2bsa_status', true );
            $response->data['wcb2bsa_affiliate_id'] = get_user_meta( $customer->ID, 'wcb2bsa_affiliate_id', true );
            $response->data['wcb2bsa_affiliate_link'] = wcb2bsa_get_affiliate_link( $customer->ID );
            $response->data['wcb2bsa_sales_agent'] = get_user_meta( $customer->ID, 'wcb2bsa_sales_agent', true );
            $response->data['wcb2bsa_parents'] = wcb2bsa_get_parents_hierarchy_by_sales_agent( $customer->ID );
            $response->data['wcb2bsa_subagents'] = wcb2bsa_get_subagents_hierarchy_by_sales_agent( $customer->ID );
            $response->data['wcb2bsa_commission'] = get_user_meta( $customer->ID, 'wcb2bsa_commission', true );
            $response->data['wcb2bsa_order_notification'] = (int)get_user_meta( $customer->ID, 'wcb2bsa_order_notification', true );
            $response->data['wcb2bsa_customers_default_group'] = get_user_meta( $customer->ID, 'wcb2bsa_customers_default_group', true );
        }
        return $response;
    }

    /**
     * Add order fields to REST API
     *
     * @param object $response REST API response object
     * @param object $order Current order object
     * @return object
     */
    public function add_order_meta( $response, $order ) {
        // Sales agent
        $response->data['wcb2bsa_sales_agent'] = (int)$order->get_meta( 'wcb2bsa_sales_agent', true );
        $response->data['wcb2bsa_order_item_types_eligible_for_commission'] = $order->get_meta( 'wcb2bsa_order_item_types_eligible_for_commission', true );
        $response->data['wcb2bsa_created_by'] = $order->get_meta( 'wcb2bsa_created_by', true );
        return $response;
    }

    /**
     * Add product fields to REST API
     *
     * @param object $response REST API response object
     * @param object $customer Current product object
     * @return object
     */
    public function add_product_meta( $response, $product ) {
        // Commissions
        $response->data['wcb2bsa_commissions'] = get_post_meta( $product->get_ID(), 'wcb2bsa_commissions' );
        return $response;
    }

}

return new WCB2BSA_Rest_Controller();