<?php

/**
 * WooCommerce B2B Sales Agents setup
 * 
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * Main WooCommerce B2B Sales Agents Class
 *
 * @class WooCommerceB2BSalesAgents
 */
final class WooCommerceB2BSalesAgents {

    /**
     * Plugin name
     *
     * @var string
     */
    public $plugin_name = 'WooCommerce B2B Sales Agents';

    /**
     * WooCommerce B2B Sales Agents version
     *
     * @var string
     */
    public $version = '1.3.2';

    /**
     * WooCommerce B2B Sales Agents Envato ID
     *
     * @var string
     */
    public $envato_id = '25720432';

    /**
     * Useful external links
     *
     * @var array
     */
    protected $links = array(
        'support'   => 'https://support.woocommerce-b2b.com/',
        'docs'      => 'https://woocommerce-b2b.com/sales-agents/documentation/',
        'snippets'  => 'https://woocommerce-b2b.com/sales-agents/snippets/',
        'changelog' => 'https://woocommerce-b2b.com/sales-agents/#changelog',
        'faq'       => 'https://woocommerce-b2b.com/sales-agents/#faq'
    );

    /**
     * The single instance of the class
     *
     * @var WooCommerceB2BSalesAgents
     */
    protected static $_instance = null;

    /**
     * Main WooCommerce B2B Sales Agents Instance
     *
     * Ensures only one instance of WooCommerce B2B Sales Agents is loaded or can be loaded
     * 
     * @static
     * @return WooCommerceB2BSalesAgents - Main instance
     */
    public static function instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * WooCommerce B2B Sales Agents Constructor
     */
    public function __construct() {
        if ( ! array_key_exists( 'woocommerce/woocommerce.php', get_site_option( 'active_sitewide_plugins', array() ) ) && ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
            add_action( 'admin_notices', array( $this, 'dependencies_notice' ) );
            add_action( 'admin_init', array( $this, 'dependencies_disable' ) );
            return false;
        }
        $this->define_constants();
        $this->init_hooks();
        $this->includes();
    }

    /**
     * Define WooCommerce B2B Sales Agents Constants
     */
    private function define_constants() {
        $this->define( 'WCB2BSA_ABSPATH', dirname( WCB2BSA_PLUGIN_FILE ) . '/' );
        $this->define( 'WCB2BSA_VERSION', $this->version );
        $this->define( 'WCB2BSA_ENVATO_ID', $this->envato_id );
        $this->define( 'WCB2BSA_OVERRIDES', 'woocommerce-b2b-sales-agents' );
    }

    /**
     * Include required core files used in admin and on the frontend
     */
    public function includes() {
        /* Upgrades */
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-upgrade.php';
        /* Common */        
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-configuration.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-install.php';
        include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-post_types.php';

        if ( $this->is_request( 'admin' ) ) {
            include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-admin.php';
        }
        if ( $this->is_request( 'frontend' ) ) {
            include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-frontend.php';
        }
        if ( $this->is_rest_api_request() ) {
            include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-rest_api.php';
        }

        add_action( 'plugins_loaded', function() {
            include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-hooks.php';
            include_once WCB2BSA_ABSPATH . 'includes/classes/class-wcb2bsa-compatibility.php';
        } );
    }

    /**
     * Hook into actions and filters
     */
    private function init_hooks() {
        register_activation_hook( WCB2BSA_PLUGIN_FILE, array( $this, 'activation' ) );
        register_deactivation_hook( WCB2BSA_PLUGIN_FILE, array( $this, 'deactivation' ) );

        add_action( 'admin_menu', array( $this, 'add_menu_page' ), 99 );
        add_filter( 'woocommerce_order_data_store_cpt_get_orders_query', array( $this, 'handle_orders_meta' ), 10, 2 );
        add_action( 'woocommerce_admin_field_multicheckbox', array( $this, 'display_settings_multicheckbox_field' ) );
        add_filter( 'woocommerce_get_settings_pages', array( $this, 'settings' ) );
        add_filter( 'plugin_row_meta', array( $this, 'plugin_row_meta' ), 10, 2 );
        add_filter( 'plugin_action_links_' . plugin_basename( WCB2BSA_PLUGIN_FILE ), array( $this, 'settings_link' ) );
        add_action( 'plugins_loaded', array( $this, 'on_plugins_loaded' ), -1 );
        add_action( 'after_setup_theme', array( $this, 'pluggable' ) );
        add_action( 'init', array( $this, 'init' ), 0 );
        add_action( 'customize_register', array( $this, 'upload_logo' ) );
        // Generators
        add_filter( 'get_the_generator_html', array( $this, 'add_generator_tag' ) );
        add_filter( 'get_the_generator_xhtml', array( $this, 'add_generator_tag' ) );
    }

    /**
     * Add pluggable support to functions
     */
    public function pluggable() {
        include_once WCB2BSA_ABSPATH . 'includes/wcb2bsa-deprecated-functions.php';
        include_once WCB2BSA_ABSPATH . 'includes/wcb2bsa-functions.php';
    }

    /**
     * Fired on plugin activation
     */
    public function activation() {
        if ( ! current_user_can( 'activate_plugins' ) ) { return; }

        $plugin = isset( $_REQUEST[ 'plugin' ] ) ? $_REQUEST[ 'plugin' ] : null;
        check_admin_referer( 'activate-plugin_' . $plugin );
    }

    /**
     * Fired on plugin deactivation
     */
    public function deactivation() {
        if ( ! current_user_can( 'activate_plugins' ) ) { return; }

        $plugin = isset( $_REQUEST[ 'plugin' ] ) ? $_REQUEST[ 'plugin' ] : null;
        check_admin_referer( 'deactivate-plugin_' . $plugin );
    }

    /**
     * Add page to admin menu
     */
    public function add_menu_page() {
        add_submenu_page(
            'users.php',
            esc_html__( 'Commissions', 'woocommerce-b2b-sales-agents' ),
            esc_html__( 'Commissions', 'woocommerce-b2b-sales-agents' ),
            'manage_woocommerce',
            'wcb2bsa-commissions',
            array( $this, 'add_menu_page_callback' ),
            99
        );
    }

    /**
     * Add page to admin menu callback
     */
    public function add_menu_page_callback() {
        include WCB2BSA_ABSPATH . 'includes/views/html-admin--page-commissions.php';
    }

    /**
     * Handle a custom 'wcb2bsa_sales_agent' and 'wcb2bsa_commission_payment' query var to get orders with the 'wcb2bsa_sales_agent' meta or 'wcb2bsa_commission_payment' meta
     * @param WP_Query $query The WP_Query instance (passed by reference)
     * @param array $query_vars The wp query args
     * @return WP_Query
     */
    public function handle_orders_meta( $query, $query_vars ) {
        if ( ! empty( $query_vars['wcb2bsa_has_sales_agent'] ) ) {
            $query['meta_query'][] = array(
                'key' => 'wcb2bsa_sales_agent',
                'compare' => 'EXISTS'
            );
            $query['meta_query'][] = array(
                'key' => 'wcb2bsa_sales_agent',
                'compare' => '!=',
                'value' => ''
            );
        }
        if ( ! empty( $query_vars['wcb2bsa_sales_agent'] ) ) {
            $query['meta_query'][] = array(
                'key' => 'wcb2bsa_sales_agent',
                'value' => array_map( function( $value ) {
                    return filter_var( esc_attr( $value ), FILTER_VALIDATE_INT );
                }, $query_vars['wcb2bsa_sales_agent'] ),
                'compare' => 'IN'
            );
        }
        if ( ! empty( $query_vars['wcb2bsa_commission_payment'] ) ) {
            $query['meta_query'][] = array(
                'key' => 'wcb2bsa_commission_payment',
                'compare' => esc_attr( $query_vars['wcb2bsa_commission_payment'] ) == 'paid' ? 'EXISTS' : 'NOT EXISTS',
            );
        }
        return $query;
    }

    public function display_settings_multicheckbox_field( $field ) {
        $field_description = WC_Admin_Settings::get_field_description( $field );
        $description       = $field_description['description'];
        ?>
        <tr valign="top">
            <th scope="row" class="titledesc"><?php echo esc_html( $field['title'] ); ?></th>
            <td class="forminp forminp-checkbox">
                
                <?php woocommerce_wp_multicheckbox( $field ); ?>

                <?php echo $description; ?>

            </td>
        </tr>
        <?php
    }

    /**
     * Include the settings page classes
     *
     * @param array $settings Array of WooCommerce settings pages
     * @return array
     */
    public function settings( $settings ) {
        $settings[] = include WCB2BSA_ABSPATH . 'includes/classes/settings/class-wcb2bsa-settings.php';
        return $settings;
    }

    /**
     * Add link to configuration page into plugin
     *
     * @param array $links Array of plugin action links
     * @return array
     */
    public function settings_link( $links ) {
        return array_merge( array(
            'settings' => '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=woocommerce-b2b-sales-agents' ) . '">' . esc_html__( 'Settings', 'woocommerce-b2b-sales-agents' ) . '</a>'
        ), $links );
    } 

    /**
     * Add useful links into plugin
     *
     * @param array $links Array of plugin action links
     * @return array
     */
    public function plugin_row_meta( $links, $file ) {
        if ( plugin_basename( WCB2BSA_PLUGIN_FILE ) === $file ) {
            $row_meta = array(
                'docs'     => sprintf( '<a href="%s">%s</a>', $this->links['docs'], esc_html__( 'Docs', 'woocommerce-b2b-sales-agents' ) ),
                'faq'      => sprintf( '<a href="%s">%s</a>', $this->links['faq'], esc_html__( 'Faq', 'woocommerce-b2b-sales-agents' ) ),
                'snippets' => sprintf( '<a href="%s">%s</a>', $this->links['snippets'], esc_html__( 'Snippets', 'woocommerce-b2b-sales-agents' ) ),
                'support'  => sprintf( '<a href="%s">%s</a>', $this->links['support'], esc_html__( 'Support', 'woocommerce-b2b-sales-agents' ) )
            );
            return array_merge( $links, $row_meta );
        }
        return (array)$links;
    }

    /**
     * When WP has loaded all plugins, trigger the `wcb2bsa_loaded` hook
     */
    public function on_plugins_loaded() {
        do_action( 'wcb2bsa_loaded' );
    }

    /**
     * Init WooCommerce B2B Sales Agents when WordPress Initialises
     */
    public function init() {
        // Set up localisation
        $this->load_plugin_textdomain();

        // Before init action
        do_action( 'wcb2bsa_before_init' );

        WCB2BSA_Install::init();

        // Init action
        do_action( 'wcb2bsa_init' );
    }

    /**
     * Generator tag
     *
     * @param string $tag Meta generator tag
     * @return string
     */
    public function add_generator_tag( $tag ) {
        if ( false === apply_filters( 'wcb2bsa_show_generator_tag', true ) ) {
            return $tag;
        }
        return "\n" . sprintf( '<meta name="generator" content="%s %s">',
            esc_attr( $this->plugin_name ),
            esc_attr( $this->version )
        );
    }

    /**
     * Add option to upload custom logo
     */
    public function upload_logo( $wp_customize ) {
        $wp_customize->add_section(
            'wcb2bsa',
            array(
                'title'       => esc_html__( 'WooCommerce B2B Sales Agents', 'woocommerce-b2b-sales-agents' ),
                'priority'    => 10
            )
        );

        $wp_customize->add_setting( 'wcb2bsa_logo' );

        $wp_customize->add_control( new WP_Customize_Cropped_Image_Control( $wp_customize, 'wcb2bsa_logo', array(
            'label'        => __( 'Sales agents area logo', 'woocommerce-b2b-sales-agents' ),
            'section'      => 'wcb2bsa',
            'settings'     => 'wcb2bsa_logo',
            'height'       => 40,
            'width'        => 270,
            'flex_width'   => true,
            'flex_height'  => true
        ) ) );
    }

    /**
     * Load Localisation files
     */
    public function load_plugin_textdomain() {
        load_plugin_textdomain( 'woocommerce-b2b-sales-agents', false, plugin_basename( dirname( WCB2BSA_PLUGIN_FILE ) ) . '/i18n/' );
    }

    /**
     * Returns true if the request is a non-legacy REST API request
     * 
     * @return boolean
     */
    public function is_rest_api_request() {
        if ( empty( $_SERVER['REQUEST_URI'] ) ) {
            return false;
        }

        $rest_prefix         = trailingslashit( rest_get_url_prefix() );
        $is_rest_api_request = ( false !== strpos( $_SERVER['REQUEST_URI'], $rest_prefix ) );

        return apply_filters( 'woocommerce_is_rest_api_request', $is_rest_api_request );
    }

    /**
     * What type of request is this?
     *
     * @param string $type Type of request: admin, ajax, cron or frontend
     * @return bool
     */
    private function is_request( $type ) {
        switch ( $type ) {
            case 'admin':
                return is_admin();
            case 'ajax':
                return defined( 'DOING_AJAX' );
            case 'cron':
                return defined( 'DOING_CRON' );
            case 'frontend':
                return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );
        }
    }

    /**
     * Define constant if not already set
     *
     * @param string $name Constant name
     * @param string|bool $value Constant value
     */
    private function define( $name, $value ) {
        if ( ! defined( $name ) ) {
            define( $name, $value );
        }
    }

    /**
     * Return useful external links
     * 
     * @return array
     */
    public function get_links() {
        return $this->links;
    }

    /**
     * Add admin notice if WooCommerce is disabled
     */
    public function dependencies_notice() {
        printf( '<div class="error"><p>%s</p></div>',
            sprintf( __( '%s requires %s plugin to be installed and active, please check. %s has been disabled.', 'woocommerce-b2b-sales-agents' ),
                '<strong>' . $this->plugin_name . '</strong>',
                '<strong>WooCommerce</strong>',
                '<strong>' . $this->plugin_name . '</strong>'
            )
        );
    }

    /**
     * Disable plugin if WooCommerce is disabled
     */
    public function dependencies_disable() {
        deactivate_plugins( WCB2BSA_PLUGIN_FILE );
        if ( isset( $_GET['activate'] ) ) {
            unset( $_GET['activate'] );
        }
    }
    
}