<?php

/**
 * WooCommerce B2B Sales Agents Settings
 *
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

if ( class_exists( 'WCB2BSA_Settings', false ) ) {
    return new WCB2BSA_Settings();
}

/**
 * WCB2BSA_Settings
 */
class WCB2BSA_Settings extends WC_Settings_Page {

    /**
     * Constructor
     */
    public function __construct() {
        $this->id    = 'woocommerce-b2b-sales-agents';
        $this->label = __( 'Sales Agents', 'woocommerce-b2b-sales-agents' );

        parent::__construct();
    }

    /**
     * Get settings array
     *
     * @return array
     */
    public function get_settings() {
        $sales_agents = wcb2bsa_get_sales_agents();
        $dashboard = WCB2BSA_Configuration::get_widgets();
        $dashboard_widgets_options = array();
        array_walk( $dashboard['widgets'], function( $item, $key ) use ( &$dashboard_widgets_options ) {
            if ( isset( $item['label'] ) ) {
                $dashboard_widgets_options[$key] = $item['label'];
            }
        } );
        $dashboard_charts_options = array();
        array_walk( $dashboard['stats'], function( $item, $key ) use ( &$dashboard_charts_options ) {
            $dashboard_charts_options[$key] = __( ucfirst( $key ), 'woocommerce-b2b-sales-agents' );
        } );
        $roles = wp_roles()->get_names();

        $settings = apply_filters( 'wcb2bsa_settings', array(
            /* GENERAL SETTINGS */
            array(
                'title'    => __( 'General settings', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'title',
                'desc'     => __( 'Manage sales agents global settings to customize your experience.', 'woocommerce-b2b-sales-agents' )
            ),
            array(
                'name'              => __( 'Moderate registration', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'checkbox',
                'default'           => 'no',
                'desc'              => __( 'Enable moderate registration for sales agent accounts', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'          => __( 'Allow sales agents to login only after admin approval.', 'woocommerce-b2b-sales-agents'),
                'id'                => 'wcb2bsa_moderate_registration'
            ),
            array(
                'name'              => __( 'Registration notice', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'checkbox',
                'default'           => 'no',
                'desc'              => __( 'Send email to admin on every new sales agent account registration', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'          => __( 'When a sales agent create an account, an email is send to admin to inform him', 'woocommerce-b2b-sales-agents' ),
                'id'                => 'wcb2bsa_registration_notice'
            ),
            array(
                'title'    => __( 'Sales agent in emails', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Add assigned sales agent data in emails.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_email_data'
            ),
            array(
                'title'    => __( 'Sales agent in MyAccount', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Add assigned sales agent data in customer MyAccount dashboard.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_myaccount_data'
            ),
            array(
                'title'    => __( 'Apply commission to', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'multicheckbox',
                'desc'     => __( 'Choose what to apply the commission to. By changing this option, only the next orders will be affected. You can customize this option for each order on order edit page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_order_item_types_eligible_for_commission',
                'options'  => WCB2BSA_Configuration::get_order_item_types_eligible_for_commission( false )
            ),
            array(
                'title'    => __( 'Only direct orders', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Consider only orders placed directly from sales agent', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'In his dashboard, sales agent can view only orders placed directly by him. Orders placed from his customers are excluded from counts, statistics, commissions calculation.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_only_direct_orders'
            ),
            array(
                'title'    => __( 'Edit customer data', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Allow sales agents to edit customer account and addresses data after account switch', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Sales agents can edit account and addresses data in checkout and in my-account pages when make purchase in behalf of their customers.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_edit_customer_data'
            ),
            array(
                'title'    => __( 'Edit prices in cart', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Allow sales agents to edit product prices in cart page', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Sales agents can edit and adjust product price in cart page when make purchase in behalf of their customers.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_edit_prices'
            ),
            array(
                'title'    => __( 'Orders commission column', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Display commissions column in orders list', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Add commissions column in WooCommerce orders list (in case of a large number of orders, it may slow down the page loading)', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_commission_column'
            ),
            array(
                'name'              => __( 'Default sales agent', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'select',
                'default'           => '0',
                'class'             => 'wc-enhanced-select',
                'id'                => 'wcb2bsa_default_sales_agent',
                'desc'              => __( 'Assign automatically customer to a sales agent on registration.', 'woocommerce-b2b-sales-agents' ) . '<br />' . __( '"Random" option, assign a random sales agent among those available', 'woocommerce-b2b-sales-agents' ) . '<br />' . __( '"Allow to choose" option, add a dropdown menu in checkout, so guest customers or customers without a sales agent can choose sales agent among those available', 'woocommerce-b2b-sales-agents' ),
                'options'           => array_reduce( $sales_agents->get_results(), function( $result, $item ) {
                    $item = (array)$item->data;
                    $result[$item['ID']] = $item['display_name'];
                    return $result;
                }, array(
                    '0'      => __( '-- None --', 'woocommerce-b2b-sales-agents' ),
                    'random' => __( '-- Random --', 'woocommerce-b2b-sales-agents' ),
                    'choose' => __( '-- Allow to choose --', 'woocommerce-b2b-sales-agents' )
                ) )
            ),
            array(
                'type'     => 'sectionend'
            ),

            /* PRODUCTS SETTINGS */

            array(
                'name'              => __( 'Products settings', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'title',
                'desc'              => __( 'This section controls product options to manage additional information.', 'woocommerce-b2b-sales-agents' )
            ),

            // Show "Estimated earnings"
            array(
                'name'              => __( 'Estimated earnings', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'checkbox',
                'default'           => 'no',
                'desc'              => __( 'Show "Estimated earnings" message', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'          => __( 'For each product shows a box on the loop pages with an approximate calculation of commissions estimated earnings.', 'woocommerce-b2b-sales-agents' ),
                'id'                => 'wcb2bsa_show_estimated_earnings'
            ),

            // Show stock
            array(
                'name'              => __( 'Show stock', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'checkbox',
                'default'           => 'no',
                'desc'              => __( 'Show stock availability to sales agents', 'woocommerce-b2b-sales-agents' ),
                'desc_tip'          => __( 'When a sales agent purchase in behalf of customers, show stock availability in product list pages.', 'woocommerce-b2b-sales-agents' ),
                'id'                => 'wcb2bsa_show_stock'
            ),

            array(
                 'type'             => 'sectionend'
            ),

            /* CUSTOMER SETTINGS */

            array(
                'name'              => __( 'Customers settings', 'woocommerce-b2b' ),
                'type'              => 'title',
                'desc'              => __( 'This section controls customers options to manage registration forms and account activation.', 'woocommerce-b2b' )
            ),

            // Roles
            array(
                'name'              => __( 'Roles to consider as customer', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'multiselect',
                'default'           => 'null',
                'class'             => 'wc-enhanced-select',
                'desc'              => __( 'Extends functions reserved for users with "customer" role to other roles.', 'woocommerce-b2b-sales-agents' ),
                'id'                => 'wcb2bsa_has_role_customer',
                'options'           => array_reduce( array_keys( $roles ), function( $result, $role ) use ( $roles ) {
                    $result[$role] = $roles[$role];
                    return $result;
                } )
            ),
            array(
                'name'              => __( 'Roles to considered as sales agent', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'multiselect',
                'default'           => 'null',
                'class'             => 'wc-enhanced-select',
                'desc'              => __( 'Extends functions reserved for users with "sales agent" role to other roles.', 'woocommerce-b2b-sales-agents' ),
                'id'                => 'wcb2bsa_has_role_sales_agent',
                'options'           => array_reduce( array_keys( $roles ), function( $result, $role ) use ( $roles ) {
                    $result[$role] = $roles[$role];
                    return $result;
                } )
            ),

            array(
                 'type'             => 'sectionend'
            ),

            /* DASHBOARD SETTINGS */

            array(
                'title'    => __( 'Sales agents dashboard settings', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'title',
                'desc'     => __( 'Manage sales agents area dashboard by selecting desired widgets and charts.', 'woocommerce-b2b-sales-agents' )
            ),
            array(
                'name'              => __( 'Sales agents dashboard page', 'woocommerce-b2b-sales-agents' ),
                'type'              => 'select',
                'default'           => 'null',
                'class'             => 'wc-enhanced-select',
                'desc'              => sprintf( __( 'Page to use for sales agents dashboard. Check it has the shortcode %s.', 'woocommerce-b2b-sales-agents' ), '<strong>[woocommerce_my_account]</strong>'),
                'id'                => 'wcb2bsa_dashboard_page',
                'options'           => array_reduce( get_pages( array( 'post_status' => array( 'publish', 'draft' ) ) ), function( $result, $item ) {
                    $item = (array)$item;
                    $result[$item['ID']] = $item['post_title'];
                    return $result;
                } )
            ),
            array(
                'title'    => __( 'Enable affiliation', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Allow customer association with affiliation links', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Create an unique affiliation link (displayed in the sales agent dashboard) for each sales agents. When a customer uses the affiliate link to register an account, his profile will be assigned to the agent who owns used affiliate link.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_enable_affiliation'
            ),
            array(
                'title'    => __( 'New subagents', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Allow sales agents to create new subagents who will be automatically assigned to him.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_allow_new_subagents',
                'show_if_checked'   => 'option',
                'checkboxgroup'     => 'start'
            ),
            array(
                'type'     => 'checkbox',
                'desc'     => __( 'Extends new subagent form adding checkout billing fields.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_extend_new_subagents_form',
                'show_if_checked'   => 'yes', 
                'checkboxgroup'     => 'end'
            ),
            array(
                'title'    => __( 'New customers', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Allow sales agents to create new customers who will be automatically assigned to him.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_allow_new_customers',
                'show_if_checked'   => 'option',
                'checkboxgroup'     => 'start'
            ),
            array(
                'type'     => 'checkbox',
                'desc'     => __( 'Extends new customer form adding checkout billing fields.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_extend_new_customers_form',
                'show_if_checked'   => 'yes', 
                'checkboxgroup'     => 'end'
            ),
            array(
                'title'    => __( 'Update order status', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Allow sales agents to change status to their customer orders.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_update_order_status',
                'show_if_checked'   => 'option'
            ),
            array(
                'title'    => __( 'Add order note', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'checkbox',
                'desc'     => __( 'Allow sales agents to add private note to their customer orders.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_add_order_note',
                'show_if_checked'   => 'option'
            ),
            array(
                'title'    => __( 'Enable widgets', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'multicheckbox',
                'desc'     => __( 'Choose widgets you want to display to your sales agents.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_dashboard_widgets',
                'options'  => $dashboard_widgets_options
            ),
            array(
                'title'    => __( 'Enable charts', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'multicheckbox',
                'desc'     => __( 'Choose charts you want to display to your sales agents.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_dashboard_charts',
                'options'  => $dashboard_charts_options
            ),
            array(
                'type'     => 'sectionend'
            ),

            array(
                'title'    => __( 'Endpoints settings', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'title',
                'desc'     => __( 'Endpoints are appended to your page URLs to handle specific actions on the sales agents account pages. They should be unique.', 'woocommerce-b2b-sales-agents' )
            ),

            // Subagents
            array(
                'title'    => __( 'Subagents', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-subagents',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Subagents" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_subagents_endpoint'
            ),

            // Customers
            array(
                'title'    => __( 'Customers', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-customers',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Customers" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_customers_endpoint'
            ),

            // Orders
            array(
                'title'    => __( 'Orders', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-orders',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Orders" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_orders_endpoint'
            ),

            // Earnings
            array(
                'title'    => __( 'Earnings', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-earnings',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Earnings" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_earnings_endpoint'
            ),

            // Commissions
            array(
                'title'    => __( 'Commissions', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-commissions',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Commissions" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_commissions_endpoint'
            ),

            // Notifications
            array(
                'title'    => __( 'Notifications', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-notifications',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Notifications" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_notifications_endpoint'
            ),

            // Statistics
            array(
                'title'    => __( 'Statistics', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-statistics',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Statistics" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_statistics_endpoint'
            ),

            // View subagent
            array(
                'title'    => __( 'View subagent', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-view-subagent',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; View subagent" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_view_subagent_endpoint'
            ),

            // View customer
            array(
                'title'    => __( 'View customer', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-view-customer',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; View customer" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_view_customer_endpoint'
            ),

            // View order
            array(
                'title'    => __( 'View order', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-view-order',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; View order" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_view_order_endpoint'
            ),

            // Edit account
            array(
                'title'    => __( 'Edit account', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-edit-account',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Edit account" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_edit_account_endpoint'
            ),

            // Logout
            array(
                'title'    => __( 'Logout', 'woocommerce-b2b-sales-agents' ),
                'type'     => 'text',
                'default'  => 'sales-agent-logout',
                'desc'     => __( '', 'woocommerce-b2b-sales-agents' ),
                'desc_tip' => __( 'Endpoint for the "My account &rarr; Logout" page.', 'woocommerce-b2b-sales-agents' ),
                'id'       => 'wcb2bsa_account_logout_endpoint'
            ),
            
            'wcb2bsa-section_end-wcb2bsa_endpoints' => array(
                 'type'    => 'sectionend'
            )
        ) );
        return apply_filters( 'woocommerce_get_settings_' . $this->id, $settings );
    }

    /**
     * Output the settings
     */
    public function output() {
        global $current_section, $hide_save_button;
        $hide_save_button = true;

        $chunks = array(
            'html',
            'admin',
            'settings',
            empty( $current_section ) ? 'general' : $current_section
        );
        $page = implode('-', array_filter($chunks));

        include_once WCB2BSA_ABSPATH . 'includes/views/settings/html-admin-settings.php';
    }

    /**
     * Save settings
     */
    public function save() {
        $settings = $this->get_settings();
        WC_Admin_Settings::save_fields( $settings );
        flush_rewrite_rules();
    }

}

return new WCB2BSA_Settings();