<?php

/**
 * Admin View: Commission statistics page
 *
 * @version 1.2.0
 */

defined( 'ABSPATH' ) || exit;

?>

<?php
    global $wpdb, $wp_locale;
    // Build orders query arguments
    $args = array(
        'type'                    => 'shop_order',
        'limit'                   => 10,
        'paginate'                => true,
        'page'                    => 1,
        'wcb2bsa_has_sales_agent' => true
    );
    
    // Add pagination
    if ( isset( $_GET['pagination'] ) && ! empty( $_GET['pagination'] ) ) {
        $args['page'] = (int)$_GET['pagination'];
    }
    // Filter by Commission payment status
    if ( isset( $_GET['wcb2bsa_commission_payment_search'] ) && ! empty( $_GET['wcb2bsa_commission_payment_search'] ) ) {
        $args['wcb2bsa_commission_payment'] = sanitize_text_field( $_GET['wcb2bsa_commission_payment_search'] );
    }
    // Filter by Sales agent
    if ( isset( $_GET['wcb2bsa_sales_agent_search'] ) && ! empty( $_GET['wcb2bsa_sales_agent_search'] ) ) {
        $args['wcb2bsa_sales_agent'] = array( (int)$_GET['wcb2bsa_sales_agent_search'] );
    }
    // Filter by date
    if ( isset( $_GET['wcb2bsa_date_search'] ) && ! empty( $_GET['wcb2bsa_date_search'] ) ) {
        $date = explode( '-', $_GET['wcb2bsa_date_search'] );
        $args['date_query']['month'] = (int)$date[0];
        $args['date_query']['year'] = (int)$date[1];
    }
    // Filter by creator
    if ( isset( $_GET['wcb2bsa_creator_search'] ) && ! empty( $_GET['wcb2bsa_creator_search'] ) ) {
        $args['meta_key'] = 'wcb2bsa_created_by';
        $args['meta_value'] = sanitize_text_field( $_GET['wcb2bsa_creator_search'] );
        if ( 'admin' == $_GET['wcb2bsa_creator_search'] ) {
            $args['meta_key'] = '_created_via';
            $args['meta_value'] = 'admin';
        }
    }
    // Filter by status
    if ( isset( $_GET['wcb2bsa_status_search'] ) && ! empty( $_GET['wcb2bsa_status_search'] ) ) {
        $args['status'] = array( sanitize_text_field( $_GET['wcb2bsa_status_search'] ) );
    }

    $results = wc_get_orders( apply_filters( 'wcb2bsa_admin_commissions_page_args', $args ) );
    $orders = $results->orders;

    $dates = $wpdb->get_results("
        SELECT DISTINCT YEAR( post_date ) AS year, MONTH( post_date ) AS month
        FROM $wpdb->posts
        WHERE post_type = 'shop_order'
        ORDER BY post_date DESC"
    );

    // Mark as paid or not
    if ( isset( $_POST['wcb2bsa_commissions_orders'] ) ) {
        if ( ! empty( $_POST['wcb2bsa_commissions_orders'] ) && wp_verify_nonce( $_POST['_wpnonce'], 'page-commissions-nonce' ) ) {
            foreach ( $_POST['wcb2bsa_commissions_orders'] as $order_id => $commissions ) {
                $payments = get_post_meta( $order_id, 'wcb2bsa_commission_payment', true );
                if ( empty( $payments ) ) {
                    $payments = array();
                }
                foreach ( $commissions as $sales_agent_id => $commission_amount ) {
                    if ( isset( $_POST['paid'] ) ) {
                        $payments[$sales_agent_id] = array(
                            'amount'    => $commission_amount,
                            'datetime'  => date( 'Y-m-d H:i:s' )
                        );
                    }
                    if ( isset( $_POST['unpaid'] ) ) {
                        if ( isset( $payments[$sales_agent_id] ) ) {
                            unset( $payments[$sales_agent_id] );
                        }
                    }
                }
                update_post_meta( $order_id, 'wcb2bsa_commission_payment', $payments );
            }
        }
    }

    $orders_total = 0;
    $commissions_total = 0;
?>

<div class="wrap">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Sales agents commissions', 'woocommerce-b2b-sales-agents' ); ?></h1>
    <hr class="wp-header-end">
    <p><?php esc_html_e( 'Here all sales agents commissions list. You can bulk mark selected commissions as paid or unpaid.', 'woocommerce-b2b-sales-agents' ); ?></p>
    <form method="get">
        <input type="hidden" name="page" value="<?php echo $_GET['page']; ?>" readonly>
        <div class="tablenav top">
            <div class="alignleft actions">

                <?php $sales_agents = wcb2bsa_get_sales_agents(); ?>

                <select class="wc-customer-search" name="wcb2bsa_sales_agent_search" id="wcb2bsa_sales_agent_search">
                    <option value=""><?php esc_html_e( 'All sales agents', 'woocommerce-b2b-sales-agents' ); ?></option>
                    
                    <?php
                        if ( ! empty( $sales_agents->get_results() ) ) {
                            foreach ( $sales_agents->get_results() as $sales_agent ) {
                                printf(
                                    '<option value="%1$s" %2$s>%3$s</option>',
                                    $sales_agent->ID,
                                    ( isset( $_GET['wcb2bsa_sales_agent_search'] ) && $_GET['wcb2bsa_sales_agent_search'] != '' && (int)$_GET['wcb2bsa_sales_agent_search'] === $sales_agent->ID ? ' selected="selected"' : '' ),
                                    $sales_agent->display_name
                                );
                            }
                        }
                    ?>

                </select>
                
                <?php wp_reset_postdata(); ?>
                
                <select class="wc-customer-search" name="wcb2bsa_commission_payment_search" id="wcb2bsa_commission_payment_search">
                    <option value=""><?php esc_html_e( 'All payment status', 'woocommerce-b2b-sales-agents' ); ?></option>

                    <?php
                        foreach ( array( 'paid' => esc_html__( 'Paid', 'woocommerce-b2b-sales-agents' ), 'unpaid' => esc_html__( 'Unpaid', 'woocommerce-b2b-sales-agents' ) ) as $key => $label ) {
                            printf(
                                '<option value="%1$s" %2$s>%3$s</option>',
                                $key,
                                ( isset( $_GET['wcb2bsa_commission_payment_search'] ) && $_GET['wcb2bsa_commission_payment_search'] != '' && $_GET['wcb2bsa_commission_payment_search'] == $key ? ' selected="selected"' : '' ),
                                $label
                            );
                        }
                    ?>
                </select>

                <select class="wc-customer-search" name="wcb2bsa_date_search" id="wcb2bsa_date_search">
                    <option value=""><?php esc_html_e( 'All dates', 'woocommerce-b2b-sales-agents' ); ?></option>

                    <?php
                        foreach ( $dates as $date ) {
                            $year  = $date->year;
                            $month = zeroise( $date->month, 2 );
                            printf(
                                '<option value="%1$s" %2$s>%3$s</option>',
                                esc_attr( $month . '-' . $year ),
                                ( isset( $_GET['wcb2bsa_date_search'] ) && $_GET['wcb2bsa_date_search'] != '' && $_GET['wcb2bsa_date_search'] == $month . '-' . $year ? ' selected="selected"' : '' ),
                                sprintf( __( '%1$s %2$d' ), $wp_locale->get_month( $month ), $year )
                            );
                        }
                    ?>
                </select>

                <select class="wc-customer-search" name="wcb2bsa_creator_search" id="wcb2bsa_creator_search">
                    <option value=""><?php esc_html_e( 'All creators', 'woocommerce-b2b-sales-agents' ); ?></option>

                    <?php
                        foreach ( WCB2BSA_Configuration::get_order_created_by() as $key => $label ) {
                            printf(
                                '<option value="%1$s" %2$s>%3$s</option>',
                                $key,
                                ( isset( $_GET['wcb2bsa_creator_search'] ) && $_GET['wcb2bsa_creator_search'] != '' && $_GET['wcb2bsa_creator_search'] == $key ? ' selected="selected"' : '' ),
                                $label
                            );
                        }
                    ?>
                </select>

                <select class="wc-customer-search" name="wcb2bsa_status_search" id="wcb2bsa_status_search">
                    <option value=""><?php esc_html_e( 'All statuses', 'woocommerce-b2b-sales-agents' ); ?></option>

                    <?php
                        foreach ( wc_get_order_statuses() as $key => $label ) {
                            printf(
                                '<option value="%1$s" %2$s>%3$s</option>',
                                $key,
                                ( isset( $_GET['wcb2bsa_status_search'] ) && $_GET['wcb2bsa_status_search'] != '' && $_GET['wcb2bsa_status_search'] == $key ? ' selected="selected"' : '' ),
                                $label
                            );
                        }
                    ?>
                </select>

                <?php echo get_submit_button( esc_html__( 'Filter', 'woocommerce-b2b-sales-agents' ), '', 'filter', false ); ?>
            </div>
        </div>
    </form>
    <form method="post">
        <table class="wp-list-table widefat fixed striped posts" data-render="datatables">
            <thead class="thead-dark">
                <tr>
                    <th width="20px"></th>
                    <th width="80px"><?php esc_html_e( 'Order', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Sales agent', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Customer', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Order Date', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Order Status', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Order amount', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Commission amount', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Commission payment', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php esc_html_e( 'Created by', 'woocommerce-b2b-sales-agents' ); ?></th>
                </tr>
            </thead>
            <tbody>
            
                <?php foreach ( $orders as $order ) : ?>

                    <?php
                        $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent', true );
                        $sales_agent = get_userdata( $sales_agent_id );
                        if ( ! ( false !== $sales_agent && wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) ) { continue; }

                        $parents = array();
                        $parents = wcb2bsa_get_parents_hierarchy_by_sales_agent( $sales_agent_id, $parents, true );

                        $order_id = $order->get_id();
                        $currency = $order->get_currency();

                        $order_commission_amount = wcb2bsa_get_order_commission_amount_by_sales_agent( $order_id, $sales_agent_id );
                    ?>

                    <tr>
                        <td>
                            <?php if ( $order_commission_amount ) : ?>
                            <input type="checkbox" name="wcb2bsa_commissions_orders[<?php echo esc_attr( $order->get_id() ); ?>][<?php echo esc_attr( $sales_agent_id ); ?>]" value="<?php echo esc_attr( $order_commission_amount ); ?>">
                            <?php endif; ?>
                        </td>
                        <td><?php printf( '<a href="%s">#%s</a>', esc_url( get_edit_post_link( $order->get_id() ) ), wp_kses_post( $order->get_order_number() ) ); ?></td>
                        <td><?php printf( '%s %s', wp_kses_post( $sales_agent->first_name ), wp_kses_post( $sales_agent->last_name ) ); ?></td>
                        <td><?php printf( '%s %s', wp_kses_post( $order->get_billing_first_name() ), wp_kses_post( $order->get_billing_last_name() ) ); ?></td>
                        <td><?php printf( '%s', wp_kses_post( wc_format_datetime( $order->get_date_created(), 'd-m-Y' ) ) ); ?></td>
                        <td><?php printf( '%s', wp_kses_post( wc_get_order_status_name( $order->get_status() ) ) ); ?></td>
                        <td><?php printf( '%s', wp_kses_post( $order->get_formatted_order_total( '', false ) ) ); ?></td>
                        <td><?php printf( '%s', wp_kses_post( wc_price( $order_commission_amount ), array( 'currency' => $currency ) ) ); ?></td>
                        <td><?php printf( '%s', wp_kses_post( wcb2bsa_get_order_commission_payout_by_sales_agent( $order_id, $sales_agent_id ) ) ); ?></td>
                        <td><?php printf( '%s', wp_kses_post( wcb2bsa_get_order_author( $order ) ) ); ?></td>
                    </tr>

                    <?php foreach ( $parents as $parent_id ) : ?>

                        <?php
                            $parent = get_userdata( $parent_id );
                            if ( ! ( false !== $parent && wcb2bsa_has_role( $parent_id, 'sales_agent' ) ) ) { continue; }

                            $order_subcommission_amount = wcb2bsa_get_order_commission_amount_by_sales_agent( $order_id, $parent_id );
                            $commissions_total += $order_subcommission_amount;
                        ?>
                        <tr>
                            <td><input type="checkbox" name="wcb2bsa_commissions_orders[<?php echo esc_attr( $order->get_id() ); ?>][<?php echo esc_attr( $parent_id ); ?>]" value="<?php echo esc_attr( $order_subcommission_amount ); ?>"></td>
                            <td></td>
                            <td><?php printf( '%s %s', wp_kses_post( $parent->first_name ), wp_kses_post( $parent->last_name ) ); ?></td>
                            <td colspan="4"></td>
                            <td><?php printf( '%s', wp_kses_post( wc_price( $order_subcommission_amount ), array( 'currency' => $currency ) ) ); ?></td>
                            <td><?php printf( '%s', wp_kses_post( wcb2bsa_get_order_commission_payout_by_sales_agent( $order_id, $parent_id ) ) ); ?></td>
                            <td></td>
                        </tr>

                    <?php endforeach; ?>

                    <?php
                        $orders_total += $order->get_total();
                        $commissions_total += $order_commission_amount;
                    ?>

                <?php endforeach; ?>
                
            </tbody>
            <tfoot>
                <tr>
                    <th colspan="6"><?php esc_html_e( 'Totals', 'woocommerce-b2b-sales-agents' ); ?></th>
                    <th><?php echo wc_price( $orders_total ); ?></th>
                    <th><?php echo wc_price( $commissions_total ); ?></th>
                    <th></th>
                    <th></th>
                </tr>
                <tr>
                    <td colspan="10">
                        <button type="submit" class="button action" name="paid"><?php esc_html_e( 'Mark as paid', 'woocommerce-b2b-sales-agents' ); ?></button>
                        <button type="submit" class="button action" name="unpaid"><?php esc_html_e( 'Mark as unpaid', 'woocommerce-b2b-sales-agents' ); ?></button>
                    </td>
                </tr>
            </tfoot>
        </table>
        <div class="tablenav bottom">
            <div class="tablenav-pages">
                <span class="displaying-num"><?php printf( __( '%s items', 'woocommerce-b2b-sales-agents' ), $results->total ); ?></span>
                <span class="pagination-links">
                    <?php echo paginate_links( array(
                        'base' => '%_%',
                        'format' => '?pagination=%#%',
                        'total' => $results->max_num_pages,
                        'current' => $args['page'],
                        'show_all' => false,
                        'end_size' => 5,
                        'mid_size' => 5,
                        'prev_next' => True,
                        'prev_text' => __( '&laquo; Previous', 'woocommerce-b2b-sales-agents' ),
                        'next_text' => __( 'Next &raquo;', 'woocommerce-b2b-sales-agents' ),
                        'type' => 'plain',
                        'add_args' => false,
                        'add_fragment' => ''
                    ) ); ?>
                </span>
            </div>
        </div>

        <?php wp_nonce_field( 'page-commissions-nonce' ); ?>

    </form>
</div>