<?php

/**
 * WooCommerce B2B Sales Agents Admin Users view
 *
 * @version 1.3.0
 */

defined( 'ABSPATH' ) || exit;

$sales_agents = wcb2bsa_get_sales_agents();
$sales_agents = $sales_agents->get_results();
?>

<?php if ( wcb2bsa_has_role( $user->ID, 'customer' ) ) : ?>
<div class="wcb2b-card">
    <input type="checkbox" id="wcb2bsa-collapse_<?php echo $user->ID; ?>" checked>
    <div class="wcb2b-card-title">
        <?php esc_html_e( 'WooCommerce B2B Sales Agents', 'woocommerce-b2b-sales-agents' ); ?>
            
        <label for="wcb2bsa-collapse_<?php echo $user->ID; ?>"></label>
    </div>
    <div class="wcb2b-card-body">
        <div class="wcb2b-label"><?php esc_html_e( 'Sales Agent', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_sales_agent"><?php esc_html_e( 'Sales agent', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <select name="wcb2bsa_sales_agent" id="wcb2bsa_sales_agent" class="regular-text">
                        <option value=""><?php esc_html_e( '-- None --', 'woocommerce-b2b-sales-agents' ); ?></option>
                        
                        <?php
                            if ( ! empty( $sales_agents ) ) {
                                foreach ( $sales_agents as $sales_agent ) {
                                    printf( '<option value="%s" %s>%s</option>', 
                                        wp_kses_post( $sales_agent->ID ),
                                        selected( get_the_author_meta( 'wcb2bsa_sales_agent', $user->ID ), $sales_agent->ID, false ),
                                        wp_kses_post( $sales_agent->display_name )
                                    );
                                }
                            }
                            wp_reset_postdata();
                        ?>

                    </select>
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Select sales agent assigned to this customer', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
        </table>
    </div>
</div>
<?php endif; ?>

<?php if ( wcb2bsa_has_role( $user->ID, 'sales_agent' ) ) : ?>
<div class="wcb2b-card">
    <input type="checkbox" id="wcb2bsa-collapse_<?php echo $user->ID; ?>" checked>
    <div class="wcb2b-card-title">
        <?php esc_html_e( 'WooCommerce B2B Sales Agents', 'woocommerce-b2b-sales-agents' ); ?>
            
        <label for="wcb2bsa-collapse_<?php echo $user->ID; ?>"></label>
    </div>
    <div class="wcb2b-card-body">

        <?php if ( get_option( 'wcb2bsa_moderate_registration' ) === 'yes' ) : ?>
        <div class="wcb2b-label"><?php esc_html_e( 'Account status', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_status"><?php esc_html_e( 'Status', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>

                    <?php $statuses = WCB2BSA_Configuration::get_sales_agent_statuses(); ?>

                    <select name="wcb2bsa_status" id="wcb2bsa_status" class="regular-text">
                        <?php foreach ( $statuses as $key => $status ) : ?>
                        <option value="<?php echo $key; ?>" <?php selected( get_the_author_meta( 'wcb2bsa_status', $user->ID ), $key ); ?> ><?php echo $status; ?></option>
                        <?php endforeach; ?>
                    </select>
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Please select sales agent status to approve registration and enable account.', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
        </table>
        <?php endif; ?>

        <div class="wcb2b-label"><?php esc_html_e( 'Affiliation', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_affiliate_id"><?php esc_html_e( 'Affiliation ID', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <input type="text" name="wcb2bsa_affiliate_id" id="wcb2bsa_affiliate_id" class="regular-text" value="<?php echo esc_attr( get_the_author_meta( 'wcb2bsa_affiliate_id', $user->ID ) ); ?>" />
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Unique affiliation code used for customer assignment. By changing this value, the previous links will no longer be valid.', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
        </table>
        <div class="wcb2b-label"><?php esc_html_e( 'Hierarchy', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_sales_agent_parent"><?php esc_html_e( 'Manager', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <select name="wcb2bsa_sales_agent_parent" id="wcb2bsa_sales_agent_parent" class="regular-text">
                        <option value=""><?php esc_html_e( '-- None --', 'woocommerce-b2b-sales-agents' ); ?></option>
                        
                        <?php
                            if ( ! empty( $sales_agents ) ) {
                                $children = array();
                                $subagents = wcb2bsa_get_subagents_hierarchy_by_sales_agent( $user->ID, $children, true );
                                foreach ( $sales_agents as $sales_agent ) {
                                    if ( in_array( $sales_agent->ID, $subagents ) ) { continue; }
                                    if ( $sales_agent->ID == $user->ID ) { continue; }
                                    printf( '<option value="%s" %s>%s</option>', 
                                        wp_kses_post( $sales_agent->ID ),
                                        selected( get_the_author_meta( 'wcb2bsa_sales_agent_parent', $user->ID ), $sales_agent->ID, false ),
                                        wp_kses_post( $sales_agent->display_name )
                                    );
                                }
                            }
                            wp_reset_postdata();
                        ?>

                    </select>
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Sales agent manager.', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
        </table>
        <br />
        <div class="wcb2b-label"><?php esc_html_e( 'Commissions', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_commission"><?php esc_html_e( 'Global commission', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <input type="number" name="wcb2bsa_commission" id="wcb2bsa_commission" class="small-text" value="<?php echo esc_attr( get_the_author_meta( 'wcb2bsa_commission', $user->ID ) ); ?>" min="0" step="0.01" />
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Sales agent commission percentage for order items (Fees, Shipping cost).', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
            <tr>
                <th><label for="wcb2bsa_subcommission"><?php esc_html_e( 'Subcommission', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <input type="number" name="wcb2bsa_subcommission" id="wcb2bsa_subcommission" class="small-text" value="<?php echo esc_attr( get_the_author_meta( 'wcb2bsa_subcommission', $user->ID ) ); ?>" min="0" step="0.01" />
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Sales agent commission percentage for order items (Fees, Shipping cost) placed by subagents, calculated from subagent commission percentage.', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
        </table>
        <br />
        <div class="wcb2b-label"><?php esc_html_e( 'Orders notification', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_order_notification"><?php esc_html_e( 'Send order notification', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <label for="wcb2bsa_order_notification">
                        <input type="checkbox" name="wcb2bsa_order_notification" id="wcb2bsa_order_notification" value="1" <?php checked( get_the_author_meta( 'wcb2bsa_order_notification', $user->ID ), true ); ?> />

                        <?php esc_html_e( 'Notify this sales agent of any order placed by his customers', 'woocommerce-b2b-sales-agents' ); ?>

                    </label>
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Sales agent will receive a copy of new order email.', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
        </table>
        <br />
        <div class="wcb2b-label"><?php esc_html_e( 'Payment preferences', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_payment_method"><?php esc_html_e( 'Payment method', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <select name="wcb2bsa_payment_method" id="wcb2bsa_payment_method" class="regular-text">
                        <option value=""><?php esc_html_e( '-- None --', 'woocommerce-b2b-sales-agents' ); ?></option>
                        
                        <?php
                            $payment_methods = wcb2bsa_get_payment_methods();
                            if ( ! empty( $payment_methods ) ) {
                                foreach ( $payment_methods as $payment_method_key => $payment_method_value ) {
                                    printf( '<option value="%s" %s>%s</option>', 
                                        wp_kses_post( $payment_method_key ),
                                        selected( get_the_author_meta( 'wcb2bsa_payment_method', $user->ID ), $payment_method_key, false ),
                                        wp_kses_post( $payment_method_value )
                                    );
                                }
                            }
                        ?>

                    </select>
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Select sales agent favorite method to receive commissions payments', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
            <tr>
                <th><label for="wcb2bsa_payment_method_info"><?php esc_html_e( 'Payment informations', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>
                    <textarea name="wcb2bsa_payment_method_info" id="wcb2bsa_payment_method_info" rows="5"><?php echo esc_attr( get_the_author_meta( 'wcb2bsa_payment_method_info', $user->ID ) ); ?></textarea>
                    <br />
                    <em class="wcb2b-help"><?php esc_html_e( 'Sales agent payment method informations, such as PayPal email, bank account, etc.', 'woocommerce-b2b-sales-agents' ); ?></em>
                </td>
            </tr>
        </table>

        <?php if ( wcb2b_is_active() ) : // WCB2B compatibility ?>

        <br />
        <div class="wcb2b-label"><?php esc_html_e( 'B2B default group for new customers', 'woocommerce-b2b-sales-agents' ); ?></div>
        <table class="form-table">
            <tr>
                <th><label for="wcb2bsa_customers_default_group"><?php esc_html_e( 'Default group', 'woocommerce-b2b-sales-agents' ); ?></label></th>
                <td>

                    <?php $groups = wcb2b_get_groups(); ?>

                    <select name="wcb2bsa_customers_default_group" id="wcb2bsa_customers_default_group" class="regular-text">
                        <option value=""><?php esc_html_e( '-- None --', 'woocommerce-b2b-sales-agents' ); ?></option>
                        <option value="choose" <?php selected( get_the_author_meta( 'wcb2bsa_customers_default_group', $user->ID ), 'choose' ); ?>><?php esc_html_e( '-- Allow to choose --', 'woocommerce-b2b-sales-agents' ); ?></option>
                        <?php if ( $groups->have_posts() ) : ?>
                            <?php while ( $groups->have_posts() ) : $groups->the_post(); ?>
                            <option value="<?php the_ID(); ?>" <?php selected( get_the_author_meta( 'wcb2bsa_customers_default_group', $user->ID ), get_the_ID() ); ?> ><?php the_title(); ?></option>
                            <?php endwhile; ?>
                        <?php endif; wp_reset_postdata(); ?>
                    </select>
                    <br />
                    <em class="wcb2b-help">
                        <?php
                            esc_html_e( 'When sales agent create a new customer, this will be inserted in a default group provided by WooCommerce B2B plugin.', 'woocommerce-b2b-sales-agents' );
                            echo '<br />';
                            esc_html_e( '"Allow to choose" option, add a dropdown menu in registration, so sales agent can choose customer group among those available', 'woocommerce-b2b-sales-agents' );
                        ?>
                    </em>
                </td>
            </tr>
        </table>

        <?php endif; ?>

    </div>
</div>
<?php endif; ?>