<?php

/**
 * WooCommerce B2B Sales Agents Functions
 *
 * @version 1.3.2
 */

defined( 'ABSPATH' ) || exit;

/**
 * Check if WooCommerce B2B is installed and enabled (not pluggable)
 *
 * @return boolean
 */
function wcb2b_is_active() {
    return in_array( 'woocommerce-b2b/woocommerce-b2b.php', get_option( 'active_plugins' ) );
}

if ( ! function_exists( 'wcb2bsa_print_styles' ) ) {
    function wcb2bsa_print_styles() {
        $styles = apply_filters( 'wcb2bsa_myaccount_styles', array(
            'loader' => plugins_url( 'assets/css/loader.min.css', WCB2BSA_PLUGIN_FILE ),
            'bootstrap' => plugins_url( 'assets/vendor/bootstrap/css/bootstrap.min.css', WCB2BSA_PLUGIN_FILE ),
            'chart' => plugins_url( 'assets/vendor/chart/Chart.min.css', WCB2BSA_PLUGIN_FILE ), // Conditional
            'datatables' => plugins_url( 'assets/vendor/datatables/datatables.min.css', WCB2BSA_PLUGIN_FILE ), // Conditional
            'fontawesome' => plugins_url( 'assets/vendor/fontawesome/css/all.min.css', WCB2BSA_PLUGIN_FILE ),
            'daterangepicker' => plugins_url( 'assets/vendor/daterangepicker/css/daterangepicker.min.css', WCB2BSA_PLUGIN_FILE ),
            'wcb2bsa' => plugins_url( 'assets/css/myaccount.min.css', WCB2BSA_PLUGIN_FILE )
        ) );
        foreach ( $styles as $style ) {
            echo '<link rel="stylesheet" href="' . esc_url( $style ) . '">';
        }
    }
}

if ( ! function_exists( 'wcb2bsa_print_scripts' ) ) {
    function wcb2bsa_print_scripts() {
        $scripts = apply_filters( 'wcb2bsa_myaccount_scripts', array(
            'jquery' => plugins_url( 'assets/vendor/jquery/jquery.min.js', WCB2BSA_PLUGIN_FILE ),
            'popper' => plugins_url( 'assets/vendor/popper/popper.min.js', WCB2BSA_PLUGIN_FILE ),
            'bootstrap' => plugins_url( 'assets/vendor/bootstrap/js/bootstrap.min.js', WCB2BSA_PLUGIN_FILE ),
            'chart' => plugins_url( 'assets/vendor/chart/Chart.min.js', WCB2BSA_PLUGIN_FILE ), // Conditional
            'moment' => plugins_url( 'assets/vendor/moment/moment.min.js', WCB2BSA_PLUGIN_FILE ), // Conditional
            'datatables' => plugins_url( 'assets/vendor/datatables/datatables.min.js', WCB2BSA_PLUGIN_FILE ), // Conditional
            'dt-moment' => plugins_url( 'assets/vendor/datatables/plugins/datetime-moment.min.js', WCB2BSA_PLUGIN_FILE ), // Conditional
            'daterangepicker' => plugins_url( 'assets/vendor/daterangepicker/js/daterangepicker.min.js', WCB2BSA_PLUGIN_FILE ),
            'clipboardjs' => plugins_url( 'assets/vendor/clipboardjs/clipboard.min.js', WCB2BSA_PLUGIN_FILE ),
            'wcb2bsa' => plugins_url( 'assets/js/myaccount.min.js', WCB2BSA_PLUGIN_FILE ) // Conditional
        ) );
        foreach ( $scripts as $script ) {
            echo '<script src="' . esc_url( $script ) . '"></script>';
        }
    }
}

/**
 * Get Sales Agents area logo
 * 
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_logo' ) ) {
    function wcb2bsa_get_logo() {
        $default = plugins_url( 'assets/images/wcb2bsa-logo.png', WCB2BSA_PLUGIN_FILE );
        if ( $logo = wp_get_attachment_image_src( get_theme_mod( 'wcb2bsa_logo' ), 'full' ) ) {
            $logo = $logo[0];
        }
        if ( empty( $logo ) ) {
            $logo = $default;
        }
        return apply_filters( 'wcb2bsa_myaccount_logo_url', $logo );
    }
}

/**
 * Display Sales Agents menu items
 *
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_account_menu_items' ) ) {
    function wcb2bsa_get_account_menu_items() {
        $endpoints = array(
            'main' => array(
                'sales-agent-customers'     => get_option( 'wcb2bsa_account_customers_endpoint', 'sales-agent-customers' ),
                'sales-agent-orders'        => get_option( 'wcb2bsa_account_orders_endpoint', 'sales-agent-orders' ),
                'sales-agent-earnings'   => get_option( 'wcb2bsa_account_earnings_endpoint', 'sales-agent-earnings' ),
                'sales-agent-commissions'   => get_option( 'wcb2bsa_account_commissions_endpoint', 'sales-agent-commissions' ),
            ),
            'profile' => array(
                'sales-agent-edit-account'  => get_option( 'wcb2bsa_account_edit_account_endpoint', 'sales-agent-edit-account' ),
                'sales-agent-logout'        => get_option( 'wcb2bsa_account_logout_endpoint', 'sales-agent-logout' )
            )
        );

        $items = array(
            'main' => array(
                'label'     => esc_html__( 'Main', 'woocommerce-b2b-sales-agents' ),
                'entries'   => array(
                    'sales-agent-dashboard'     => array(
                        'label'     => esc_html__( 'Dashboard', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-tachometer-alt'
                    ),
                    'sales-agent-customers'     => array(
                        'label'     => esc_html__( 'Customers', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-users'
                    ),
                    'sales-agent-orders'        => array(
                        'label'     => esc_html__( 'Orders', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-shopping-cart'
                    ),
                    'sales-agent-earnings'   => array(
                        'label'     => esc_html__( 'Earnings', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-wallet'
                    ),
                    'sales-agent-commissions'   => array(
                        'label'     => esc_html__( 'Commissions', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-percentage'
                    ),
                    'sales-agent-notifications'   => array(
                        'label'     => esc_html__( 'Notifications', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-bell',
                        'counter'   => wcb2bsa_get_notifications_count_by_sales_agent()
                    ),
                    'sales-agent-statistics'   => array(
                        'label'     => esc_html__( 'Statistics', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-chart-bar'
                    )
                )
            ),
            'network' => array(
                'label'     => esc_html__( 'Network', 'woocommerce-b2b-sales-agents' ),
                'entries'   => array(
                    'sales-agent-subagents'     => array(
                        'label'     => esc_html__( 'Subagents', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-user-tie'
                    )
                )
            ),
            'profile' => array(
                'label'     => esc_html__( 'Profile', 'woocommerce-b2b-sales-agents' ),
                'entries'   => array(
                    'sales-agent-edit-account'  => array(
                        'label'     => esc_html__( 'Account', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'far fa-id-badge'
                    ),
                    'sales-agent-logout'        => array(
                        'label'     => esc_html__( 'Logout', 'woocommerce-b2b-sales-agents' ),
                        'icon'      => 'fas fa-sign-out-alt'
                    )
                )
            )
        );
        foreach ( $endpoints as $section => $entries ) {
            foreach ( $entries as $endpoint_id => $endpoint ) {
                if ( empty( $endpoint ) ) {
                    unset( $items[ $section ]['entries'][ $endpoint_id ] );
                }
            }
        }
        return apply_filters( 'wcb2bsa_myaccount_menu_items', $items );
    }
}

/**
 * Get sales agents area endpoint URL
 * 
 * @param string $endpoint Endpoint name
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_account_endpoint_url' ) ) {
    function wcb2bsa_get_account_endpoint_url( $endpoint ) {
        // Dashboard enpoint, go to sales agents area home
        if ( 'sales-agent-dashboard' === $endpoint ) {
            return get_permalink( get_option( 'wcb2bsa_dashboard_page' ) );
        }
        // Logout endpoint, create custom URL or default WordPress logout URL
        if ( 'sales-agent-logout' === $endpoint ) {
            if ( get_option( 'woocommerce_logout_endpoint' ) ) {
                return wp_nonce_url( wc_get_endpoint_url( 'sales-agent-logout', '', get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) ), 'sales-agent-logout' );
            }
            return wp_logout_url( get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) );
        }
        // Create endpoint URL
        return wc_get_endpoint_url( $endpoint, '', get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) );
    }
}

/**
 * Get view single order URL
 * 
 * @param int $order_id Order ID
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_view_order_url' ) ) {
    function wcb2bsa_get_view_order_url( $order_id ) {
        $endpoint = get_option( 'wcb2bsa_account_view_order_endpoint', 'sales-agent-view-order' );
        return wc_get_endpoint_url( $endpoint, $order_id, get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) );
    }
}

/**
 * Get view single customer URL
 * 
 * @param int $customer_id Customer ID
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_view_customer_url' ) ) {
    function wcb2bsa_get_view_customer_url( $customer_id ) {
        $endpoint = get_option( 'wcb2bsa_account_view_customer_endpoint', 'sales-agent-view-customer' );
        return wc_get_endpoint_url( $endpoint, $customer_id, get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) );
    }
}

/**
 * Get view single subagent URL
 * 
 * @param int $subagent_id Subagent ID
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_view_subagent_url' ) ) {
    function wcb2bsa_get_view_subagent_url( $subagent_id ) {
        $endpoint = get_option( 'wcb2bsa_account_view_subagent_endpoint', 'sales-agent-view-subagent' );
        return wc_get_endpoint_url( $endpoint, $subagent_id, get_permalink( get_option( 'wcb2bsa_dashboard_page' ) ) );
    }
}

/**
 * Retrieve all Sales Agents users
 * 
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_sales_agents' ) ) {
    function wcb2bsa_get_sales_agents() {
        return new WP_User_Query( array(
            'limit'     => -1,
            'role__in'  => get_option( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ),
            'orderby'   => 'display_name',
            'order'     => 'ASC'
        ) );
    }
}

/**
 * Retrieve sales agent name from ID
 * 
 * @param int $sales_agent_id Sales agent ID
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_sales_agent_name' ) ) {
    function wcb2bsa_get_sales_agent_name( $sales_agent_id ) {
        $sales_agent = get_userdata( $sales_agent_id );
        return sprintf( $sales_agent->display_name );
    }
}

/**
 * Retrieve all sales agent customer orders (with optional pagination)
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param int $current_page Current page number used for pagination
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_orders_by_sales_agent' ) ) {
    function wcb2bsa_get_orders_by_sales_agent( $sales_agent_id, $current_page = 1 ) {
        $parents = array();
        if ( wcb2bsa_get_subagents_full_hierarchy() ) {
            $parents = wcb2bsa_get_subagents_hierarchy_by_sales_agent( $sales_agent_id );
        }
        array_push( $parents, $sales_agent_id );
        $args = array(
            'limit'                 => -1,
            'page'                  => 1,
            'paginate'              => true,
            'type'                  => 'shop_order',
            'wcb2bsa_sales_agent'   => $parents
        );
        // Add pagination
        if ( $current_page !== false ) {
            $args['limit']    = apply_filters( 'wcb2bsa_dashboard_pagination', 20 );
            $args['page']     = $current_page;
        }
        // Only direct orders
        if ( 'yes' == get_option( 'wcb2bsa_only_direct_orders' ) ) {
            $args['meta_key'] = 'wcb2bsa_created_by';
            $args['meta_value'] = 'sales_agent';
        }

        $orders = new WC_Order_Query( apply_filters( 'wcb2bsa_get_orders_by_sales_agent_args', $args, $sales_agent_id ) );
        return $orders->get_orders();
    }
}

/**
 * Retrieve all sales agent customer (with optional pagination)
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param int $current_page Current page number used for pagination
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_customers_by_sales_agent' ) ) {
    function wcb2bsa_get_customers_by_sales_agent( $sales_agent_id, $current_page = 1 ) {
        $parents = array();
        if ( wcb2bsa_get_subagents_full_hierarchy() ) {
            $parents = wcb2bsa_get_subagents_hierarchy_by_sales_agent( $sales_agent_id );
        }
        array_push( $parents, $sales_agent_id );
        $args = array(
            'number'     => -1,
            'paged'      => 1,
            'paginate'   => true,
            'role__in'   => get_option( 'wcb2bsa_has_role_customer', array( 'customer' ) ),
            'fields'     => 'all_with_meta',
            'meta_key'   => 'wcb2bsa_sales_agent',
            'meta_value' => $parents,
            'compare'    => 'IN'
        );
        // Add pagination
        if ( $current_page !== false ) {
            $args['number']   = apply_filters( 'wcb2bsa_dashboard_pagination', 20 );
            $args['paged']    = $current_page;
        }
        $customers = new WP_User_Query( apply_filters( 'wcb2bsa_get_customers_by_sales_agent_args', $args, $sales_agent_id ) );
        return $customers;
    }
}

/**
 * Retrieve all sales agent subagents (with optional pagination)
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param int $current_page Current page number used for pagination
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_subagents_by_sales_agent' ) ) {
    function wcb2bsa_get_subagents_by_sales_agent( $sales_agent_id, $current_page = 1 ) {
        $parents = array();
        if ( wcb2bsa_get_subagents_full_hierarchy() ) {
            $parents = wcb2bsa_get_subagents_hierarchy_by_sales_agent( $sales_agent_id );
        }
        array_push( $parents, $sales_agent_id );
        $args = array(
            'number'     => -1,
            'paged'      => 1,
            'paginate'   => true,
            'role__in'   => get_option( 'wcb2bsa_has_role_sales_agent', array( 'customer' ) ),
            'fields'     => 'all_with_meta',
            'meta_key'   => 'wcb2bsa_sales_agent_parent',
            'meta_value' => $parents,
            'compare'    => 'IN'
        );
        // Add pagination
        if ( $current_page !== false ) {
            $args['number']   = apply_filters( 'wcb2bsa_dashboard_pagination', 20 );
            $args['paged']    = $current_page;
        }
        $subagents = new WP_User_Query( apply_filters( 'wcb2bsa_get_subagents_by_sales_agent_args', $args, $sales_agent_id ) );
        return $subagents;
    }
}

/**
 * Retrieve all sales agent subagents descendant hierarchy
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param array $children IDs container by reference
 * @param boolean $full_hierarchy Stop on first level or go deeply
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_subagents_hierarchy_by_sales_agent' ) ) {
    function wcb2bsa_get_subagents_hierarchy_by_sales_agent( $sales_agent_id, &$children = array(), $full_hierarchy = false ) {
        $args = array(
            'fields' => 'ID',
            'limit'     => -1,
            'role__in'  => get_option( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ),
            'meta_key'   => 'wcb2bsa_sales_agent_parent',
            'meta_value' => $sales_agent_id
        );
        $sales_agents = new WP_User_Query( $args );
        foreach ( $sales_agents->get_results() as $child ) {
            $children[] = $child;
            if ( ! $full_hierarchy && ! wcb2bsa_get_subagents_full_hierarchy() ) {
                return $children;
            }
            wcb2bsa_get_subagents_hierarchy_by_sales_agent( $child, $children );
        }
        return $children;
    }
}

/**
 * Retrieve all sales agent parents ascendant hierarchy
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param array $parents IDs container by reference
 * @param boolean $full_hierarchy Stop on first level or go deeply
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_parents_hierarchy_by_sales_agent' ) ) {
    function wcb2bsa_get_parents_hierarchy_by_sales_agent( $sales_agent_id, &$parents = array(), $full_hierarchy = false ) {
        if ( $parent = get_the_author_meta( 'wcb2bsa_sales_agent_parent', $sales_agent_id ) ) {
            $parents[] = $parent;
            if ( ! $full_hierarchy && ! wcb2bsa_get_subagents_full_hierarchy() ) {
                return $parents;
            }
            wcb2bsa_get_parents_hierarchy_by_sales_agent( $parent, $parents );
        }
        return $parents;
    }
}

/**
 * Retrieve all sales agent notifications (with optional pagination)
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param int $current_page Current page number used for pagination
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_notifications_by_sales_agent' ) ) {
    function wcb2bsa_get_notifications_by_sales_agent( $sales_agent_id, $current_page = 1 ) {
        $args = array(
            'post_type'             => 'wcb2bsa_notification',
            'status'                => 'publish',
            'limit'                 => -1,
            'meta_key'              => 'wcb2bsa_recipients'
        );
        // Add pagination
        if ( $current_page !== false ) {
            $args['page']     = $current_page;
            $args['paginate'] = true;
        }

        $notifications = array();
        $wp_query = new WP_Query( apply_filters( 'wcb2bsa_get_notifications_by_sales_agent_args', $args, $sales_agent_id ) );
        if ( $wp_query->found_posts ) {
            foreach ( $wp_query->posts as $post ) {
                $recipients = get_post_meta( $post->ID, 'wcb2bsa_recipients', true );
                if ( ! is_array( $recipients ) ) {
                    $recipients = array();
                }
                if ( in_array( $sales_agent_id, $recipients ) ) {
                    $post->unread = empty( get_user_meta( $sales_agent_id, 'wcb2bsa_read_notification_' . $post->ID, true ) );
                    $notifications[] = $post;
                }
            }
        }
        return $notifications;
    }
}

/**
 * Retrieve unread sales agent notification count
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @return int
 */
if ( ! function_exists( 'wcb2bsa_get_notifications_count_by_sales_agent' ) ) {
    function wcb2bsa_get_notifications_count_by_sales_agent() {
        $notifications = wcb2bsa_get_notifications_by_sales_agent( get_current_user_id() );
        $counter = count( array_filter( $notifications, function( $item ) {
            return ! empty( $item->unread );
        } ) );
        return $counter;
    }
}

/**
 * Retrieve and calculate commission on a cart item by sales agent
 * 
 * @param object $item An instance of current order item
 * @param int $sales_agent_id ID of sales agent assigned to order
 * @return float Commission percentage
 */
if ( ! function_exists( 'wcb2bsa_get_item_commission' ) ) {
    function wcb2bsa_get_item_commission( $item, $sales_agent_id ) {
        $product_id = 0;
        if ( $item->get_type() == 'line_item' ) {
            // Only products have dedicated commissions
            $product_id = $item->get_product_id();
        }
        return wcb2bsa_get_product_commission( $product_id, $sales_agent_id );
    }
}

/**
 * Retrieve and calculate commission on a product by sales agent
 * 
 * @param int $product_id Product ID
 * @param int $sales_agent_id ID of sales agent assigned to order
 * @return float Commission percentage
 */
if ( ! function_exists( 'wcb2bsa_get_product_commission' ) ) {
    function wcb2bsa_get_product_commission( $product_id, $sales_agent_id ) {
        // Only products have dedicated commissions
        $commissions = get_post_meta( $product_id, 'wcb2bsa_commissions', true );

        if ( isset( $commissions[$sales_agent_id] ) && $commissions[$sales_agent_id] != '' ) {
            // If commission exists for this sales agent
            return $commissions[$sales_agent_id];
        }
        // WCB2B compatibility
        if ( wcb2b_is_active() ) {
            $customer_group_id = get_option( 'wcb2b_guest_group' );
            if ( is_user_logged_in() ) {
                if ( $group_id = get_the_author_meta( 'wcb2b_group', get_current_user_id() ) ) {
                    $customer_group_id = $group_id;
                }
            }
            $group_commission = get_post_meta( $customer_group_id, 'wcb2bsa_group_commission', true );
            if ( $group_commission != '' ) {
                return $group_commission;
            }
        }
        // Fallback: global sales agent commission
        return get_the_author_meta( 'wcb2bsa_commission', $sales_agent_id );
    }
}

/**
 * Retrieve sales agent total commission amount
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param  string $status Commission type
 * @return float
 */
if ( ! function_exists( 'wcb2bsa_get_total_commission_amount_by_sales_agent' ) ) {
    function wcb2bsa_get_total_commission_amount_by_sales_agent( $sales_agent_id, $status = 'paid', $hierarchy = -1 ) {
        $sales_agents = array( 0 );
        if ( true === $hierarchy ) {
            $sales_agents = wcb2bsa_get_subagents_hierarchy_by_sales_agent( $sales_agent_id, $sales_agents, true );
        } elseif ( false === $hierarchy ) {
            array_push( $sales_agents, $sales_agent_id );
        } else {
            $sales_agents = wcb2bsa_get_subagents_hierarchy_by_sales_agent( $sales_agent_id, $sales_agents, true );
            array_push( $sales_agents, $sales_agent_id );
        }

        $amount = 0;
        $args = array(
            'limit'             => -1,
            'type'              => 'shop_order',
            'wcb2bsa_sales_agent'  => $sales_agents,
            'return' => 'ids'
        );
        // Only direct orders
        if ( 'yes' == get_option( 'wcb2bsa_only_direct_orders' ) ) {
            $args['meta_key'] = 'wcb2bsa_created_by';
            $args['meta_value'] = 'sales_agent';
        }
        $included = apply_filters( 'wcb2bsa_include_orders_in_pending_commissions_by_status', array( 'completed', 'on-hold', 'processing', 'pending' ), $sales_agent_id );
        $orders = new WC_Order_Query( apply_filters( 'wcb2bsa_get_orders_by_sales_agent', $args ) );
        foreach ( $orders->get_orders() as $order_id ) {
            $order_status = str_replace( 'wc-', '', get_post_status( $order_id ) );
            switch ( $status ) {
                // Total commissions generated, independently from order status
                case 'total' :
                    $amount += wcb2bsa_get_order_commission_amount_by_sales_agent( $order_id, $sales_agent_id );
                    break;
                // Paid commissions (aka payouts)
                case 'paid' :
                    $amount += wcb2bsa_get_order_commission_amount_paid_by_sales_agent( $order_id, $sales_agent_id );
                    break;
                // Pending commissions, not yet paid
                case 'pending' :
                    if ( in_array( $order_status, $included ) ) {
                        $amount += wcb2bsa_get_order_commission_amount_by_sales_agent( $order_id, $sales_agent_id ) - wcb2bsa_get_order_commission_amount_paid_by_sales_agent( $order_id, $sales_agent_id );
                    }
                    break;
            }
        }
        return round( $amount, 2 );
    }
}

/**
 * Retrieve sales agent total commission amount
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param  string $status Commission type
 * @return float
 */
if ( ! function_exists( 'wcb2bsa_get_total_orders_amount_by_sales_agent' ) ) {
    function wcb2bsa_get_total_orders_amount_by_sales_agent( $sales_agent_id ) {
        $amount = 0;
        $args = array(
            'limit'             => -1,
            'type'              => 'shop_order',
            'wcb2bsa_sales_agent'  => array( $sales_agent_id )
        );
        // Only direct orders
        if ( 'yes' == get_option( 'wcb2bsa_only_direct_orders' ) ) {
            $args['meta_key'] = 'wcb2bsa_created_by';
            $args['meta_value'] = 'sales_agent';
        }
        $orders = new WC_Order_Query( apply_filters( 'wcb2bsa_get_orders_by_sales_agent', $args ) );
        foreach ( $orders->get_orders() as $order ) {
            if ( ! in_array( $order->get_status(), apply_filters( 'wcb2bsa_exclude_orders_in_totals_by_status', array( 'cancelled', 'failed', 'refunded' ), $sales_agent_id ) ) ) {
                $amount += (float)$order->get_total();
            }
        }
        return round( $amount, 2 );
    }
}

/**
 * Calculate total commission amount on single order
 *
 * @param int $order_id Order ID
 * @param int $sales_agent_id Sales agent ID
 * @return float
 */
if ( ! function_exists( 'wcb2bsa_get_order_commission_amount_by_sales_agent' ) ) {
    function wcb2bsa_get_order_commission_amount_by_sales_agent( $order_id, $sales_agent_id = false ) {
        $amount = 0;
        $meta_key = 'wcb2bsa_item_commission_amount';
        if ( $sales_agent_id ) {
            $meta_key = 'wcb2bsa_item_commission_amount_' . $sales_agent_id;
        }

        if ( $order_item_types_eligible_for_commission = get_post_meta( $order_id, 'wcb2bsa_order_item_types_eligible_for_commission', true ) ) {
            if ( !empty( $order_item_types_eligible_for_commission ) ) {
                $order = new WC_Order( $order_id );
                foreach ( $order->get_items( array_keys( $order_item_types_eligible_for_commission ) ) as $item_id => $item ) {
                    $amount += (float)wc_get_order_item_meta( $item->get_id(), $meta_key, true );
                }
            }
        }
        return round( $amount, 2 );
    }
}

/**
 * Calculate total commission amount on order
 *
 * @param int $order_id Order ID
 * @return float
 */
if ( ! function_exists( 'wcb2bsa_order_commission_amount_evaluate' ) ) {
    function wcb2bsa_order_commission_amount_evaluate( $order_id ) {
        $amount = 0;

        if ( $order_item_types_eligible_for_commission = get_post_meta( $order_id, 'wcb2bsa_order_item_types_eligible_for_commission', true ) ) {
            $order = new WC_Order( $order_id );
            foreach ( $order->get_items( array_keys( $order_item_types_eligible_for_commission ) ) as $item_id => $item ) {
                $amount += wcb2bsa_order_item_commission_amount_evaluate( $item, $order );
            }
        }
        return round( $amount, 2 );
    }
}

/**
 * Calculate commission amount on order item
 *
 * @param int $order_item_id Order item ID
 * @param WC_Order $order WC_Order class instance
 * @return float
 */
if ( ! function_exists( 'wcb2bsa_order_item_commission_amount_evaluate' ) ) {
    function wcb2bsa_order_item_commission_amount_evaluate( $item, $order ) {
        $item_total_refunded = $order->get_total_refunded_for_item( $item->get_id() );
        $item_total = $item->get_total() - $item_total_refunded;
        $item_commission = wc_get_order_item_meta( $item->get_id(), 'wcb2bsa_item_commission', true );
        return (float)$item_commission * $item_total / 100;
    }
}

/**
 * Retrieve order commission paid amount
 * 
 * @param int $order_id Order ID
 * @return decimal
 */
if ( ! function_exists( 'wcb2bsa_get_order_commission_amount_paid_by_sales_agent' ) ) {
    function wcb2bsa_get_order_commission_amount_paid_by_sales_agent( $order_id, $sales_agent_id ) {
        $commission_payments = get_post_meta( $order_id, 'wcb2bsa_commission_payment', true );
        return isset( $commission_payments[$sales_agent_id]['amount'] ) ? (float)$commission_payments[$sales_agent_id]['amount'] : 0;
    }
}

/**
 * Retrieve order commission payment data by sales agent
 * 
 * @param int $order_id Order ID
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_order_commission_payout_by_sales_agent' ) ) {
    function wcb2bsa_get_order_commission_payout_by_sales_agent( $order_id, $sales_agent_id ) {
        $commission_payments = get_post_meta( $order_id, 'wcb2bsa_commission_payment', true );
        if ( ! isset( $commission_payments[$sales_agent_id] ) ) {
            return sprintf( esc_html__( 'Unpaid', 'woocommerce-b2b-sales-agents' ) );
        }

        return sprintf( esc_html__( 'Paid on %s', 'woocommerce-b2b-sales-agents' ),
            date_i18n( get_option( 'date_format' ), strtotime( $commission_payments[$sales_agent_id]['datetime'] ) )
        );
    }
}

/**
 * Retrieve statistics data
 * 
 * @param int $sales_agent_id Sales Agent ID
 * @param string $type Statistics type
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_stats' ) ) {
    function wcb2bsa_get_stats( $sales_agent_id, $type = 'bar', $data = 'orders', $groupby = 'month', $daterange = false ) {
        $params = array(
            'type' => $type,
            'options' => array(
                'aspectRatio' => 3,
                'scales' => array(
                    'yAxes' => array(
                        array(
                            'ticks' => array(
                                'beginAtZero' => true
                            )
                        )
                    )
                )
            ),
            'data' => ''
        );

        if ( empty( $daterange ) ) {
            $daterange = sprintf( '%s - %s', date( '01/01/Y' ), date( 'd/m/Y' ) );
        }

        $date = explode( ' - ', $daterange );

        $chart_datasets = array();
        $chart_values = array();
        $chart_labels = array();
        $chart_legend = '';

        $date_start = DateTime::createFromFormat( 'd/m/Y', $date[0] );
        $date_start->setTime( 00, 00, 00 );

        $date_end = DateTime::createFromFormat( 'd/m/Y', $date[1] );
        $date_end->setTime( 23, 59, 59 );

        $interval = DateInterval::createFromDateString( '1 ' . $groupby );
        $periods = new DatePeriod( $date_start, $interval, $date_end );

        switch ( $groupby ) {
            case 'year' :
                $format = 'Y';
                $label_formats = array( 'Y' );
                break;
            case 'month' :
                $format = 'Ym';
                $label_formats = array( 'M', 'Y' );
                break;
            case 'day' :
                $format = 'Ymd';
                $label_formats = array( 'd', 'M', 'Y' );
                break;
            default :
                $format = 'Ymd';
                break;
        }

        foreach ( $periods as $period ) {
            $labels = array();
            foreach ( $label_formats as $label_format ) {
                $labels[] = $period->format( $label_format );
            }
            $chart_labels[] = $labels;
            $chart_values[$period->format( $format )] = 0;
        }

        switch ( $data ) {
            case 'orders':
                $chart_legend = esc_html__( 'Orders', 'woocommerce-b2b-sales-agents' );
                $args = array(
                    'limit'             => -1,
                    'type'              => 'shop_order',
                    'wcb2bsa_sales_agent'  => array( $sales_agent_id ),
                    'date_query'    => array(
                        array(
                            'before'    => $date_end->format( 'Y-m-d H:i:s' ),
                            'after'     => $date_start->format( 'Y-m-d H:i:s' ),
                            'inclusive' => true
                        )
                    )
                );
                // Only direct orders
                if ( 'yes' == get_option( 'wcb2bsa_only_direct_orders' ) ) {
                    $args['meta_key'] = 'wcb2bsa_created_by';
                    $args['meta_value'] = 'sales_agent';
                }
                $orders = new WC_Order_Query( $args );
                foreach ( $orders->get_orders() as $order ) {
                    $chart_values[wc_format_datetime( $order->get_date_created(), $format )]++;
                }
                break;
            case 'commissions':
                $chart_legend = esc_html__( 'Commissions', 'woocommerce-b2b-sales-agents' );
                $args = array(
                    'limit'             => -1,
                    'type'              => 'shop_order',
                    'wcb2bsa_sales_agent'  => array( $sales_agent_id ),
                    'date_query'    => array(
                        array(
                            'before'    => $date_end->format( 'Y-m-d H:i:s' ),
                            'after'     => $date_start->format( 'Y-m-d H:i:s' ),
                            'inclusive' => true
                        )
                    )
                );
                // Only direct orders
                if ( 'yes' == get_option( 'wcb2bsa_only_direct_orders' ) ) {
                    $args['meta_key'] = 'wcb2bsa_created_by';
                    $args['meta_value'] = 'sales_agent';
                }
                $orders = new WC_Order_Query( $args );
                foreach ( $orders->get_orders() as $order ) {
                    $chart_values[wc_format_datetime( $order->get_date_created(), $format )] += wcb2bsa_get_order_commission_amount_by_sales_agent( $order->get_id(), $sales_agent_id );
                }
                break;
            case 'customers':
                $chart_legend = esc_html__( 'Customers', 'woocommerce-b2b-sales-agents' );
                $args = array(
                    'limit'      => -1,
                    'role__in'   => get_option( 'wcb2bsa_has_role_customer', array( 'customer' ) ),
                    'fields'     => 'all_with_meta',
                    'meta_key'   => 'wcb2bsa_sales_agent',
                    'meta_value' => array( $sales_agent_id ),
                    'date_query'    => array(
                        array(
                            'before'    => $date_end->format( 'Y-m-d H:i:s' ),
                            'after'     => $date_start->format( 'Y-m-d H:i:s' ),
                            'inclusive' => true
                        )
                    )
                );
                $customers = new WP_User_Query( $args );
                foreach ( $customers->get_results() as $customer ) {
                    $chart_values[wc_format_datetime( wc_string_to_datetime( $customer->user_registered ), $format )]++;
                }
                break;
            case 'subagents' :
                $chart_legend = esc_html__( 'Subagents', 'woocommerce-b2b-sales-agents' );
                $args = array(
                    'limit'      => -1,
                    'role__in'   => get_option( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ),
                    'fields'     => 'all_with_meta',
                    'meta_key'   => 'wcb2bsa_sales_agent_parent',
                    'meta_value' => $sales_agent_id,
                    'date_query'    => array(
                        array(
                            'before'    => $date_end->format( 'Y-m-d H:i:s' ),
                            'after'     => $date_start->format( 'Y-m-d H:i:s' ),
                            'inclusive' => true
                        )
                    )
                );
                $subagents = new WP_User_Query( $args );
                foreach ( $subagents->get_results() as $subagent ) {
                    $chart_values[wc_format_datetime( wc_string_to_datetime( $subagent->user_registered ), $format )]++;
                }
                break;
            default:
                break;
        }

        $chart_datasets[] = array(
            'label' => $chart_legend,
            'backgroundColor' => 'rgba( 0, 0, 0, 0.5 )',
            'borderColor' => 'rgba(0, 0, 0, 0.7)',
            'borderWidth' => 2,
            'data' => array_values( $chart_values )
        );

        $params['data'] = array(
            'labels' => $chart_labels,
            'datasets' => $chart_datasets
        );

        return json_encode( apply_filters( 'wcb2bsa_stats_params', $params ) );
    }
}

/**
 * Verify if user has specific roles
 */
if ( ! function_exists( 'wcb2bsa_has_role' ) ) {
    function wcb2bsa_has_role( $user_id, $role ) {
        $default = get_option( 'wcb2bsa_has_role_' . $role, array( $role ) );
        $roles = ( new WP_User( $user_id ) )->roles;

        return ! empty( array_intersect( $default, $roles ) );
    }
}

/**
 * Look for WooCommerce B2B Sales Agents template
 */
if ( ! function_exists( 'wcb2bsa_locate_template' ) ) {
    function wcb2bsa_locate_template( $template_name ) {
        $template = locate_template( WCB2BSA_OVERRIDES . '/' . $template_name );
        if ( empty( $template ) ) {
            $template = trailingslashit( plugin_dir_path( WCB2BSA_PLUGIN_FILE ) ) . 'templates/' . $template_name;
        }
        return $template;
    }
}

/**
 * Verify order created by
 * @param object $order Current order instance
 * @return string|boolean
 */
if ( ! function_exists( 'wcb2bsa_get_order_author' ) ) {
    function wcb2bsa_get_order_author( $order ) {
        $authors = WCB2BSA_Configuration::get_order_created_by();
        if ( $order_created_by = $order->get_meta( 'wcb2bsa_created_by', true ) ) {
            return $authors[$order_created_by];
        }
        if ( 'admin' == $order->get_created_via() ) {
            return $authors['admin'];
        }
        return false;
    }
}

/**
 * Verify if a sales agent can view all hierarchy
 * @return boolean
 */
if ( ! function_exists( 'wcb2bsa_get_subagents_full_hierarchy' ) ) {
    function wcb2bsa_get_subagents_full_hierarchy() {
        return apply_filters( 'wcb2bsa_subagents_full_hierarchy', true );
    }
}

/**
 * Verify if a sales agent can read all hierarchy
 * @return boolean
 */
if ( ! function_exists( 'wcb2bsa_is_subagents_hierarchy_readable' ) ) {
    function wcb2bsa_is_subagents_hierarchy_readable() {
        return apply_filters( 'wcb2bsa_subagents_hierarchy_readable', true );
    }
}

/**
 * Verify if a sales agent can write all hierarchy
 * @return boolean
 */
if ( ! function_exists( 'wcb2bsa_is_subagents_hierarchy_writable' ) ) {
    function wcb2bsa_is_subagents_hierarchy_writable() {
        return apply_filters( 'wcb2bsa_subagents_hierarchy_writable', true );
    }
}

/**
 * Verify sales agent capabilities
 * @param integer $sales_agent_id Sales agent ID
 * @param integer $user_id User ID
 * @return boolean
 */
if ( ! function_exists( 'wcb2bsa_can_read' ) ) {
    function wcb2bsa_can_read( $sales_agent_id, $user_id ) {
        $has_permission = $sales_agent_id == $user_id;
        if ( wcb2bsa_is_subagents_hierarchy_readable() ) {
            $has_permission |= in_array( $sales_agent_id, wcb2bsa_get_subagents_hierarchy_by_sales_agent( $user_id ) );
        }
        return $has_permission;
    }
}

/**
 * Verify sales agent capabilities
 * @param integer $sales_agent_id Sales agent ID
 * @param integer $user_id User ID
 * @return boolean
 */
if ( ! function_exists( 'wcb2bsa_can_write' ) ) {
    function wcb2bsa_can_write( $sales_agent_id, $user_id ) {
        $has_permission = $sales_agent_id == $user_id;
        if ( wcb2bsa_is_subagents_hierarchy_writable() ) {
            $has_permission |= in_array( $sales_agent_id, wcb2bsa_get_subagents_hierarchy_by_sales_agent( $user_id ) );
        }
        return $has_permission;
    }
}

/**
 * Generate a unique affiliation ID with user data
 * @param integer $user_id User ID
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_generate_affiliate_id' ) ) {
    function wcb2bsa_generate_affiliate_id( $user_id ) {
        if ( $userdata = get_userdata( $user_id ) ) {
            return base64_encode( sprintf( '%s|%s',
                $userdata->user_email,
                $userdata->ID
            ) );
        }
        return false;
    }
}

/**
 * Build sales agent affiliation link
 *
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_get_affiliate_link' ) ) {
    function wcb2bsa_get_affiliate_link( $sales_agent_id = false ) {
        if ( $sales_agent_id === false ) {
            if ( !is_user_logged_in() ) {
                return false;
            }
            $sales_agent_id = get_current_user_id();
        }

        if ( wcb2bsa_has_role( $sales_agent_id, 'sales_agent' ) ) {
            $parameter = apply_filters( 'wcb2bsa_affiliate_parameter', 'wcb2bsa_affiliate' );
            $link = apply_filters( 'wcb2bsa_affiliate_baseurl', get_permalink( wc_get_page_id( 'myaccount' ) ) );
            return apply_filters( 'wcb2bsa_affiliate_permalink', add_query_arg( array(
                $parameter => wcb2bsa_generate_affiliate_id( $sales_agent_id )
            ), $link ) );
        }
        return false;
    }
}

/**
 * Get allowed payment methods for sales agents
 *
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_payment_methods' ) ) {
    function wcb2bsa_get_payment_methods() {
        return apply_filters( 'wcb2bsa_payment_methods', array(
            'paypal' => __( 'Paypal', 'woocommerce-b2b-sales-agents' ),
            'banwire' => __( 'Bankwire', 'woocommerce-b2b-sales-agents' )
        ) );
    }
}

/**
 * Build product estimated earnings
 */
if ( ! function_exists( 'wcb2bsa_show_estimated_earnings' ) ) {
    function wcb2bsa_show_estimated_earnings() {
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                global $product;

                ob_start();
                wc_get_template( 'loop/estimated-earnings.php', array(
                    'earnings' => wc_price( (float)$product->get_price() * (float)wcb2bsa_get_product_commission( $product->get_id(), $switchto_id ) / 100 )
                ), WCB2BSA_OVERRIDES, WCB2BSA_ABSPATH . 'templates/' );
                $html = ob_get_contents();
                ob_end_clean();
                echo $html;
            }
        }
    }
}

/**
 * Build product stock
 */
if ( ! function_exists( 'wcb2bsa_show_stock' ) ) {
    function wcb2bsa_show_stock() {
        if ( is_user_logged_in() ) {
            $switchto_id = get_transient( 'wcb2bsa_switched_' . get_current_user_id() );
            if ( $switchto_id && isset( $_COOKIE['wcb2bsa_switched'] ) ) {
                global $product;
                $availability = $product->get_availability();

                if ( ! empty( $availability['availability'] ) ) {
                    ob_start();
                    wc_get_template( 'loop/stock.php', array(
                        'product'      => $product,
                        'class'        => $availability['class'],
                        'availability' => $availability['availability'],
                    ), WCB2BSA_OVERRIDES, WCB2BSA_ABSPATH . 'templates/' );
                    $html = ob_get_contents();
                    ob_end_clean();
                    echo $html;
                }
            }
        }
    }
}

/**
 * Get options for daterangepicker in sales agents dashboard statistics page
 *
 * @return array
 */
if ( ! function_exists( 'wcb2bsa_get_drp_i18n' ) ) {
    function wcb2bsa_get_drp_i18n() {
        return json_encode( array(
            'today' => _x( 'Today', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
            'yesterday' => _x( 'Yesterday', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
            'last7days' => _x( 'Last 7 Days', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
            'last30days' => _x( 'Last 30 Days', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
            'thismonth' => _x( 'This Month', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
            'lastmonth' => _x( 'Last Month', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
            'locale' => [
                'separator' => ' - ',
                'format' => _x( 'DD/MM/Y', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                'applyLabel' => _x( 'Apply', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                'cancelLabel' => _x( 'Cancel', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                'fromLabel' => _x( 'From', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                'toLabel' => _x( 'To', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                'customRangeLabel' => _x( 'Custom', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                'weekLabel' => _x( 'W', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                'daysOfWeek' => [
                    _x( 'Su', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'Mo', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'Tu', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'We', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'Th', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'Fr', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'Sa', 'daterangepicker', 'woocommerce-b2b-sales-agents' )
                ],
                'monthNames' => [
                    _x( 'January', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'February', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'March', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'April', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'May', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'June', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'July', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'August', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'September', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'October', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'November', 'daterangepicker', 'woocommerce-b2b-sales-agents' ),
                    _x( 'December', 'daterangepicker', 'woocommerce-b2b-sales-agents' )
                ]
            ]
        ) );
    }
}

/**
 * Outputs a new customer form field
 *
 * @param string $key Key.
 * @param mixed $args Arguments.
 * @param string $value (default: null).
 * @return string
 */
if ( ! function_exists( 'wcb2bsa_form_field' ) ) {
    function wcb2bsa_form_field( $type, $key, $args, $value = null ) {
        $defaults = array(
            'type'              => 'text',
            'label'             => '',
            'description'       => '',
            'placeholder'       => '',
            'maxlength'         => false,
            'required'          => false,
            'autocomplete'      => false,
            'id'                => $key,
            'class'             => array(),
            'label_class'       => array(),
            'input_class'       => array(),
            'return'            => false,
            'options'           => array(),
            'custom_attributes' => array(),
            'validate'          => array(),
            'default'           => '',
            'autofocus'         => '',
            'priority'          => '',
        );

        $args = wp_parse_args( $args, $defaults );
        $args = apply_filters( 'wcb2bsa_new_' . $type . '_form_fields_args', $args, $key, $value );

        if ( $args['required'] ) {
            $args['class'][] = 'validate-required';
            $required        = '&nbsp;<abbr class="required" title="' . esc_attr__( 'required', 'woocommerce-b2b-sales-agents' ) . '">*</abbr>';
        } else {
            $required = '&nbsp;<span class="optional">(' . esc_html__( 'optional', 'woocommerce-b2b-sales-agents' ) . ')</span>';
        }

        if ( is_string( $args['label_class'] ) ) {
            $args['label_class'] = array( $args['label_class'] );
        }

        if ( is_null( $value ) ) {
            $value = $args['default'];
        }

        // Custom attribute handling.
        $custom_attributes         = array();
        $args['custom_attributes'] = array_filter( (array) $args['custom_attributes'], 'strlen' );

        if ( $args['maxlength'] ) {
            $args['custom_attributes']['maxlength'] = absint( $args['maxlength'] );
        }

        if ( ! empty( $args['autocomplete'] ) ) {
            $args['custom_attributes']['autocomplete'] = $args['autocomplete'];
        }

        if ( true === $args['autofocus'] ) {
            $args['custom_attributes']['autofocus'] = 'autofocus';
        }

        if ( $args['description'] ) {
            $args['custom_attributes']['aria-describedby'] = $args['id'] . '-description';
        }

        if ( ! empty( $args['custom_attributes'] ) && is_array( $args['custom_attributes'] ) ) {
            foreach ( $args['custom_attributes'] as $attribute => $attribute_value ) {
                $custom_attributes[] = esc_attr( $attribute ) . '="' . esc_attr( $attribute_value ) . '"';
            }
        }

        if ( ! empty( $args['validate'] ) ) {
            foreach ( $args['validate'] as $validate ) {
                $args['class'][] = 'validate-' . $validate;
            }
        }

        $field           = '';
        $label_id        = $args['id'];
        $sort            = $args['priority'] ? $args['priority'] : '';
        $field_container = '<div class="form-row %1$s" id="%2$s" data-priority="' . esc_attr( $sort ) . '">%3$s</div>';

        switch ( $args['type'] ) {
            case 'country':
                $countries = 'shipping_country' === $key ? WC()->countries->get_shipping_countries() : WC()->countries->get_allowed_countries();

                if ( 1 === count( $countries ) ) {

                    $field .= '<strong>' . current( array_values( $countries ) ) . '</strong>';

                    $field .= '<input type="hidden" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" value="' . current( array_keys( $countries ) ) . '" ' . implode( ' ', $custom_attributes ) . ' class="country_to_state" readonly="readonly" />';

                } else {

                    $field = '<select name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" class="country_to_state country_select ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" ' . implode( ' ', $custom_attributes ) . '><option value="">' . esc_html__( 'Select a country / region&hellip;', 'woocommerce-b2b-sales-agents' ) . '</option>';

                    foreach ( $countries as $ckey => $cvalue ) {
                        $field .= '<option value="' . esc_attr( $ckey ) . '" ' . selected( $value, $ckey, false ) . '>' . esc_html( $cvalue ) . '</option>';
                    }

                    $field .= '</select>';

                    $field .= '<noscript><button type="submit" name="woocommerce_checkout_update_totals" value="' . esc_attr__( 'Update country / region', 'woocommerce-b2b-sales-agents' ) . '">' . esc_html__( 'Update country / region', 'woocommerce-b2b-sales-agents' ) . '</button></noscript>';

                }

                break;
            case 'state':
                /* Get country this state field is representing */
                $for_country = isset( $args['country'] ) ? $args['country'] : WC()->checkout->get_value( 'billing_state' === $key ? 'billing_country' : 'shipping_country' );
                $states      = WC()->countries->get_states( $for_country );

                if ( is_array( $states ) && empty( $states ) ) {

                    $field_container = '<div class="form-row %1$s" id="%2$s" style="display: none">%3$s</div>';

                    $field .= '<input type="hidden" class="hidden" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" value="" ' . implode( ' ', $custom_attributes ) . ' placeholder="' . esc_attr( $args['placeholder'] ) . '" readonly="readonly" data-input-classes="' . esc_attr( implode( ' ', $args['input_class'] ) ) . '"/>';

                } elseif ( ! is_null( $for_country ) && is_array( $states ) ) {

                    $field .= '<select name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" class="state_select ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" ' . implode( ' ', $custom_attributes ) . ' data-placeholder="' . esc_attr( $args['placeholder'] ? $args['placeholder'] : esc_html__( 'Select an option&hellip;', 'woocommerce-b2b-sales-agents' ) ) . '"  data-input-classes="' . esc_attr( implode( ' ', $args['input_class'] ) ) . '">
                        <option value="">' . esc_html__( 'Select an option&hellip;', 'woocommerce-b2b-sales-agents' ) . '</option>';

                    foreach ( $states as $ckey => $cvalue ) {
                        $field .= '<option value="' . esc_attr( $ckey ) . '" ' . selected( $value, $ckey, false ) . '>' . esc_html( $cvalue ) . '</option>';
                    }

                    $field .= '</select>';

                } else {

                    $field .= '<input type="text" class="input-text ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" value="' . esc_attr( $value ) . '"  placeholder="' . esc_attr( $args['placeholder'] ) . '" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" ' . implode( ' ', $custom_attributes ) . ' data-input-classes="' . esc_attr( implode( ' ', $args['input_class'] ) ) . '"/>';

                }

                break;
            case 'textarea':
                $field .= '<textarea name="' . esc_attr( $key ) . '" class="input-text ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" id="' . esc_attr( $args['id'] ) . '" placeholder="' . esc_attr( $args['placeholder'] ) . '" ' . ( empty( $args['custom_attributes']['rows'] ) ? ' rows="2"' : '' ) . ( empty( $args['custom_attributes']['cols'] ) ? ' cols="5"' : '' ) . implode( ' ', $custom_attributes ) . '>' . esc_textarea( $value ) . '</textarea>';

                break;
            case 'checkbox':
                $field_container = '<div class="form-row custom-control custom-checkbox %1$s" id="%2$s" data-priority="' . esc_attr( $sort ) . '">%3$s</div>';
                $field = '<input type="' . esc_attr( $args['type'] ) . '" class="input-checkbox custom-control-input ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" value="1" ' . checked( $value, 1, false ) . ' /> ' . $args['label'] . $required . '
                    <label class="checkbox custom-control-label ' . implode( ' ', $args['label_class'] ) . '" ' . implode( ' ', $custom_attributes ) . '></label>';

                break;
            case 'text':
            case 'password':
            case 'datetime':
            case 'datetime-local':
            case 'date':
            case 'month':
            case 'time':
            case 'week':
            case 'number':
            case 'email':
            case 'url':
            case 'tel':
                $field .= '<input type="' . esc_attr( $args['type'] ) . '" class="input-text ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" placeholder="' . esc_attr( $args['placeholder'] ) . '"  value="' . esc_attr( $value ) . '" ' . implode( ' ', $custom_attributes ) . ' />';

                break;
            case 'select':
                $field   = '';
                $options = '';

                if ( ! empty( $args['options'] ) ) {
                    foreach ( $args['options'] as $option_key => $option_text ) {
                        if ( '' === $option_key ) {
                            // If we have a blank option, select2 needs a placeholder.
                            if ( empty( $args['placeholder'] ) ) {
                                $args['placeholder'] = $option_text ? $option_text : __( 'Choose an option', 'woocommerce-b2b-sales-agents' );
                            }
                            $custom_attributes[] = 'data-allow_clear="true"';
                        }
                        $options .= '<option value="' . esc_attr( $option_key ) . '" ' . selected( $value, $option_key, false ) . '>' . esc_html( $option_text ) . '</option>';
                    }

                    $field .= '<select name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" class="select ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" ' . implode( ' ', $custom_attributes ) . ' data-placeholder="' . esc_attr( $args['placeholder'] ) . '">
                            ' . $options . '
                        </select>';
                }

                break;
            case 'radio':
                $field_container = '<div class="form-row custom-control custom-radio %1$s" id="%2$s" data-priority="' . esc_attr( $sort ) . '">%3$s</div>';
                $label_id .= '_' . current( array_keys( $args['options'] ) );

                if ( ! empty( $args['options'] ) ) {
                    foreach ( $args['options'] as $option_key => $option_text ) {
                        $field .= '<input type="radio" class="input-radio custom-control-input ' . esc_attr( implode( ' ', $args['input_class'] ) ) . '" value="' . esc_attr( $option_key ) . '" name="' . esc_attr( $key ) . '" ' . implode( ' ', $custom_attributes ) . ' id="' . esc_attr( $args['id'] ) . '_' . esc_attr( $option_key ) . '"' . checked( $value, $option_key, false ) . ' />';
                        $field .= '<label for="' . esc_attr( $args['id'] ) . '_' . esc_attr( $option_key ) . '" class="radio custom-control-label ' . implode( ' ', $args['label_class'] ) . '">' . esc_html( $option_text ) . '</label>';
                    }
                }

                break;
        }

        if ( ! empty( $field ) ) {
            $field_html = '';

            if ( $args['label'] && 'checkbox' !== $args['type'] ) {
                $field_html .= '<label for="' . esc_attr( $label_id ) . '" class="' . esc_attr( implode( ' ', $args['label_class'] ) ) . '">' . $args['label'] . $required . '</label>';
            }

            $field_html .= $field;

            if ( $args['description'] ) {
                $field_html .= '<span class="description" id="' . esc_attr( $args['id'] ) . '-description" aria-hidden="true">' . wp_kses_post( $args['description'] ) . '</span>';
            }

            $field_html .= '</span>';

            $container_class = esc_attr( implode( ' ', $args['class'] ) );
            $container_id    = esc_attr( $args['id'] ) . '_field';
            $field           = sprintf( $field_container, $container_class, $container_id, $field_html );
        }

        if ( $args['return'] ) {
            return $field;
        } else {
            echo $field; // WPCS: XSS ok.
        }
    }
}

/**
 * Build new multicheckbox field
 * 
 * @param array $field Field arguments
 */
function woocommerce_wp_multicheckbox( $field ) {
    // Retrocompatibility
    if ( ! isset( $field['value'] ) ) {
        $field['value'] = get_option( $field['id'], array() );
    }
    foreach ( $field['options'] as $key => $value ) {
        ?>
        <fieldset>
            <label for="<?php echo esc_attr( $field['id'] . '_' . $key ); ?>">
                <input
                    name="<?php echo esc_attr( $field['id'] ); ?>[<?php echo esc_attr( $key ); ?>]"
                    id="<?php echo esc_attr( $field['id'] . '_' . $key ); ?>"
                    type="checkbox"
                    value="1"
                    
                    <?php
                        if ( $field['value'] && array_key_exists( $key, $field['value'] ) ) {
                            checked( $field['value'][$key], true );
                        }
                    ?>

                /> <?php echo wp_kses_post( $value ); ?>
            </label>
        </fieldset>
        <?php
    }
}