<?php

/**
 * WooCommerce B2B Sales Agents Update Functions
 *
 * @version 1.2.0
 */

defined( 'ABSPATH' ) || exit;

function wcb2bsa_order_created_by_105() {
    global $wpdb;

    $results = $wpdb->get_results( "SELECT post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = '_created_via' AND meta_value = 'checkout'", OBJECT );
    foreach ( $results as $result ) {
        update_post_meta( $result->post_id, 'wcb2bsa_created_by', 'sales_agent' );
    }
    return true;
}

function wcb2bsa_dashboard_page_109() {
    if ( ! get_option( 'wcb2bsa_dashboard_page', false ) ) {
        update_option( 'wcb2bsa_dashboard_page', wc_get_page_id( 'myaccount' ) );
    }
    return true;
}

function wcb2bsa_edit_customer_data_109() {
    if ( ! get_option( 'wcb2bsa_edit_customer_data', false ) ) {
        update_option( 'wcb2bsa_edit_customer_data', 'yes' );
    }
    return true;
}

function wcb2bsa_add_capabilities_111() {
    $role = get_role( 'sales_agent' );
    $role->add_cap( 'edit_shop_orders', true );
    return true;
}

function wcb2bsa_order_total_commission_112() {
    global $wpdb;

    $results = $wpdb->get_results( "SELECT ID FROM {$wpdb->prefix}posts WHERE post_type = 'shop_order'", OBJECT );
    foreach ( $results as $result ) {
        $commissions = wcb2bsa_order_commission_amount_evaluate( $result->ID );
        update_post_meta( $result->ID, 'wcb2bsa_commission_amount', $commissions );
    }
    return true;
}

function wcb2bsa_earning_endpoint_112() {
    return update_option( 'wcb2bsa_account_earnings_endpoint', 'sales-agents-earnings' );
}

function wcb2bsa_order_item_commission_amount_120() {
    global $wpdb;

    // Select all orders with a sales agent assigned
    $orders = $wpdb->get_results( "SELECT post_id, meta_value FROM {$wpdb->prefix}postmeta WHERE meta_key = 'wcb2bsa_sales_agent' AND meta_value > 0", OBJECT );
    if ( count( $orders ) ) {
        foreach ( $orders as $order ) {
            $maybe_is_updated = get_metadata( 'post', $order->post_id, 'wcb2bsa_commission_amount', true );
            $sales_agent_id = $order->meta_value;
            // Get order sales agent ID
            if ( $sales_agent_id && ! $maybe_is_updated ) {
                // Inizialize order total commission amount
                $order_commission_amount = 0;
                // Get eligible items types from order
                $order_item_types_eligible = get_metadata( 'post', $order->post_id, 'wcb2bsa_order_item_types_eligible_for_commission', true );
                // Get all order items
                $order_items = $wpdb->get_results(
                    $wpdb->prepare(
                        "SELECT order_item_id, order_item_type FROM {$wpdb->prefix}woocommerce_order_items WHERE order_id = %d",
                        $order->post_id
                    )
                );

                foreach ( $order_items as $order_item ) {
                    // Check if item is eligible for commission
                    if ( isset( $order_item_types_eligible[$order_item->order_item_type] ) && $order_item_types_eligible[$order_item->order_item_type] == 1 ) {
                        // Get item commission percentage
                        $order_item_commission = floatval( get_metadata( 'order_item', $order_item->order_item_id, 'wcb2bsa_item_commission', true ) );
                        // Default meta keys
                        $total_meta_key = '_line_total';
                        $tax_meta_key = '_line_tax';
                        // Only in shipping line, meta keys are different
                        if ( 'shipping' == $order_item->order_item_type ) {
                            $total_meta_key = 'cost';
                            $tax_meta_key = 'total_tax';
                        }
                        // Get order item total amount
                        $order_item_line_total = floatval( get_metadata( 'order_item', $order_item->order_item_id, $total_meta_key, true ) );
                        // Get order item total refunded amount
                        $order_item_line_total_refunded = abs( floatval( $wpdb->get_var(
                            $wpdb->prepare(
                                "SELECT SUM( meta_value ) FROM {$wpdb->prefix}woocommerce_order_itemmeta WHERE meta_key = %s AND order_item_id IN ( SELECT order_item_id FROM {$wpdb->prefix}woocommerce_order_itemmeta WHERE meta_key = '_refunded_item_id' AND meta_value = %d )",
                                $total_meta_key,
                                $order_item->order_item_id
                            )
                        ) ) );
                        // Get order item total tax amount
                        $order_item_line_tax = floatval( get_metadata( 'order_item', $order_item->order_item_id, $tax_meta_key, true ) );
                        // Get order item total tax refunded amount
                        $order_item_line_tax_refunded = abs( floatval( $wpdb->get_var(
                            $wpdb->prepare(
                                "SELECT SUM( meta_value ) FROM {$wpdb->prefix}woocommerce_order_itemmeta WHERE meta_key = %s AND order_item_id IN ( SELECT order_item_id FROM {$wpdb->prefix}woocommerce_order_itemmeta WHERE meta_key = '_refunded_item_id' AND meta_value = %d )",
                                $tax_meta_key,
                                $order_item->order_item_id
                            )
                        ) ) );
                        // Calculate item final total paid
                        $order_item_total_amount =  ( $order_item_line_total - $order_item_line_total_refunded ) + 
                                                    ( $order_item_line_tax - $order_item_line_tax_refunded );

                        // Calculate item commission amount
                        $order_item_commission_amount = $order_item_total_amount * $order_item_commission / 100;

                        /*** Save order item commission amount ***/
                        /*** Save order item commission amount (by sales agent) ***/
                        update_metadata( 'order_item', $order_item->order_item_id, 'wcb2bsa_item_commission_amount', $order_item_commission_amount );
                        update_metadata( 'order_item', $order_item->order_item_id, 'wcb2bsa_item_commission_amount_' . $sales_agent_id, $order_item_commission_amount );

                        // Sum to order total commission amount
                        $order_commission_amount += $order_item_commission_amount;
                    }
                }

                /*** Save order commission amount ***/
                update_metadata( 'post', $order->post_id, 'wcb2bsa_commission_amount', $order_commission_amount );
            }
        }
    }
    return true;
}

function wcb2bsa_order_commission_payment_120() {
    global $wpdb;
    $results = $wpdb->get_results( "SELECT post_id, meta_value FROM {$wpdb->prefix}postmeta WHERE meta_key = 'wcb2bsa_commission_payment'", OBJECT );
    foreach ( $results as $result ) {
        $sales_agent_id = get_post_meta( $result->post_id, 'wcb2bsa_sales_agent', true );
        if ( $sales_agent_id ) {
            $value = unserialize( $result->meta_value );
            if (isset($value['amount'])) {
                $payments[$sales_agent_id] = $value;
                update_post_meta( $result->post_id, 'wcb2bsa_commission_payment', $payments );
            }
        }
    }
    return true;
}

function wcb2bsa_set_suitable_roles_132() {
    return  update_option( 'wcb2bsa_has_role_customer', apply_filters( 'wcb2bsa_has_role_customer', array( 'customer' ) ), true ) &&
            update_option( 'wcb2bsa_has_role_sales_agent', apply_filters( 'wcb2bsa_has_role_sales_agent', array( 'sales_agent' ) ), true );
}