<?php
/**
 * WooCommerce B2B Sales Agents MyAccount - Commissions products table
 *
 * @version 1.2.2
 */
?>

<?php defined( 'ABSPATH' ) || exit; ?>

<?php if ( ! $has_products ) : ?>
<div class="alert alert-dark">
    <div><?php esc_html_e( 'Sorry, there are not yet product commissions', 'woocommerce-b2b-sales-agents' ); ?></div>
</div>
<?php return; endif; ?>

<div class="table-fluid small">
    <table class="table table-striped" <?php echo $interactive ? 'data-render="datatables"' : false; ?> data-name="commissions">
        <thead>
            <tr class="text-uppercase text-muted">
                <th class="text-nowrap border-0"><?php esc_html_e( 'Name', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'SKU', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Price', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Total sales', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Commission', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Your earning', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0 no-sort"></th>
            </tr>
        </thead>
        <tfoot>
            <tr class="search">
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th class="no-search"></th>
            </tr>
        </tfoot>
        <tbody>
        
            <?php foreach ( $products as $product_id ) : ?>

                <?php
                    $product = wc_get_product( $product_id );
                    $price = floatval( $product->get_price() );
                    $sales_agent_id = get_current_user_id();
                    $commission = get_the_author_meta( 'wcb2bsa_commission', $sales_agent_id );
                    $commissions = get_post_meta( $product_id, 'wcb2bsa_commissions', true );
                    if ( isset( $commissions[$sales_agent_id] ) && $commissions[$sales_agent_id] != '' ) {
                        // If commission exists for this sales agent
                        $commission = $commissions[$sales_agent_id];
                    }
                    $commission = floatval( $commission );
                ?>

                <tr>

                    <?php
                        // Product name
                        $raw = $formatted = $product->get_title();
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Name', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Product SKU
                        $raw = $formatted = $product->get_sku();
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'SKU', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Product price
                        $raw = $price;
                        $formatted = wc_price( $raw );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Price', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Total sales
                        $raw = $formatted = $product->get_total_sales();
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Total sales', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            $raw ? wp_kses_post( sprintf( '%s', $formatted ) ) : false
                        );
                        // Commission (%s)
                        $raw = $formatted = $commission;
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Commission', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            $raw ? wp_kses_post( sprintf( '%s%%', $formatted ) ) : false
                        );
                        // Earning
                        $raw = $commission * $price / 100;
                        $formatted = wc_price( $raw );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Earning', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            $raw ? wp_kses_post( $formatted ) : false
                        );
                        // Button
                        printf('<td class="text-center"><a class="btn btn-dark btn-sm" href="%s" target="_blank">%s</a></td>',
                            esc_url( $product->get_permalink() ),
                            esc_html__( 'View', 'woocommerce-b2b-sales-agents' )
                        );
                    ?>

                </tr>

            <?php endforeach; ?>

        </tbody>
    </table>
</div>