<?php
/**
 * WooCommerce B2B Sales Agents MyAccount - Customers table
 *
 * @version 1.2.0
 */
?>

<?php defined( 'ABSPATH' ) || exit; ?>

<?php if ( ! $has_customers ) : ?>
<div class="alert alert-dark">
    <div><?php esc_html_e( 'Sorry, there are not yet customers assigned to you', 'woocommerce-b2b-sales-agents' ); ?></div>
</div>
<?php return; endif; ?>

<?php if ( ! $interactive ) { printf( '<div class="card shadow-sm my-3">' ); } ?>

<div class="table-fluid small">
    <table class="table table-striped" <?php echo $interactive ? 'data-render="datatables"' : false; ?> data-name="customers">
        <thead>
            <tr class="text-uppercase text-muted">
                <th class="text-nowrap border-0 no-sort"></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Name', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Surname', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Company', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Address', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'City', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Country', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Email', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Telephone', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Total orders', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Total spent amount', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0 no-sort"></th>
            </tr>
        </thead>
        <tfoot>
            <tr class="search">
                <th class="no-search"></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th class="no-search"></th>
            </tr>
        </tfoot>
        <tbody>
        
            <?php foreach ( $customers->get_results() as $customer ) : ?>
            <tr>

                <?php
                    // Subagent
                    $sales_agent_id = get_user_meta( $customer->ID, 'wcb2bsa_sales_agent', true );
                    $direct = get_current_user_id() == $sales_agent_id;
                    printf('<td class="text-center">%s</td>',
                        sprintf( ! $direct ? '<i class="fas fa-user-tie" data-toggle="tooltip" data-placement="top" title="%s"></a>' : false,
                            wcb2bsa_get_sales_agent_name( $sales_agent_id )
                        )
                    );
                    // Customer name
                    $raw = $formatted = $customer->user_firstname;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Name', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer lastname
                    $raw = $formatted = $customer->user_lastname;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Surname', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer company
                    $raw = $formatted = $customer->billing_company;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Company', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer address
                    $raw = $formatted = get_user_meta( $customer->ID, 'billing_address_1', true );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Address', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer city
                    $raw = $formatted = get_user_meta( $customer->ID, 'billing_city', true );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'City', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer country
                    $raw = $formatted = get_user_meta( $customer->ID, 'billing_country', true );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Country', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer email
                    $raw = $formatted = $customer->user_email;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Email', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer telephone
                    $raw = $formatted = get_user_meta( $customer->ID, 'billing_phone', true );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Telephone', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Total orders
                    $raw = $formatted = wc_get_customer_order_count( $customer->ID );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Total orders', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Total orders amount
                    $raw = wc_get_customer_total_spent( $customer->ID );
                    $formatted = wc_price( $raw );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Total orders amount', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Button
                    printf('<td class="text-center"><a class="btn btn-dark btn-sm" href="%s">%s</a></td>',
                        esc_url( wcb2bsa_get_view_customer_url( $customer->ID ) ),
                        esc_html__( 'Details', 'woocommerce-b2b-sales-agents' )
                    );
                ?>

            </tr>    
            <?php endforeach; ?>

        </tbody>
    </table>
</div>

<?php if ( ! $interactive ) { printf( '</div>' ); } ?>
