<?php
/**
 * WooCommerce B2B Sales Agents MyAccount - Earnings table
 *
 * @version 1.2.0
 */
?>

<?php defined( 'ABSPATH' ) || exit; ?>

<?php if ( ! $has_orders ) : ?>
<div class="alert alert-dark">
    <div><?php esc_html_e( 'Sorry, there are not yet earnings collected', 'woocommerce-b2b-sales-agents' ); ?></div>
</div>
<?php return; endif; ?>

<div class="table-fluid small">
    <table class="table table-striped" <?php echo $interactive ? 'data-render="datatables"' : false; ?> data-name="earnings">
        <thead>
            <tr class="text-uppercase text-muted">
                <th class="text-nowrap border-0 no-sort"></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Order reference', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Customer', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Company', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Order amount', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Order date', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Order status', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Your earning', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Earning payment', 'woocommerce-b2b-sales-agents' ); ?></th>
            </tr>
        </thead>
        <tfoot>
            <tr class="search">
                <th class="no-search"></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
            </tr>
        </tfoot>
        <tbody>
        
            <?php foreach ( $orders->orders as $order ) : ?>

                <?php
                    $order_id = $order->get_id();
                    $currency = $order->get_currency();
                ?>

                <tr>

                    <?php
                        // Subagent
                        $sales_agent_id = $order->get_meta( 'wcb2bsa_sales_agent' );
                        $direct = get_current_user_id() == $sales_agent_id;
                        printf('<td class="text-center">%s</td>',
                            sprintf( ! $direct ? '<i class="fas fa-user-tie" data-toggle="tooltip" data-placement="top" title="%s"></a>' : false,
                                wcb2bsa_get_sales_agent_name( $sales_agent_id )
                            )
                        );
                        // Order reference
                        $raw = $formatted = $order->get_order_number();
                        printf('<td data-title="%s" data-sort="%s">#%s</td>',
                            esc_attr__( 'Order reference', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Customer
                        $raw = $formatted = sprintf( '%s %s',
                            $order->get_billing_last_name(),
                            $order->get_billing_first_name()
                        );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Customer', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Company
                        $raw = $formatted = $order->get_billing_company();
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Company', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Total amount
                        $raw = $order->get_total();
                        $formatted = $order->get_formatted_order_total( '', false );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Total amount', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Datetime
                        $raw = wc_format_datetime( $order->get_date_created(), 'YmdHis' );
                        $formatted = wc_format_datetime( $order->get_date_created() );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Date', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Status
                        $raw = $formatted = wc_get_order_status_name( $order->get_status() );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Date', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Earning
                        $raw = wcb2bsa_get_order_commission_amount_by_sales_agent( $order->get_id(), get_current_user_id() );
                        $formatted = wc_price( $raw );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Earning', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                        // Earning payment
                        $raw = $formatted = wcb2bsa_get_order_commission_payout_by_sales_agent( $order_id, $sales_agent_id );
                        printf('<td data-title="%s" data-sort="%s">%s</td>',
                            esc_attr__( 'Earning payment', 'woocommerce-b2b-sales-agents' ),
                            esc_html( $raw ),
                            wp_kses_post( $formatted )
                        );
                    ?>

                </tr>

            <?php endforeach; ?>

        </tbody>
    </table>
</div>

<?php if ( ! $interactive ) { printf( '</div>' ); } ?>
