<?php
/**
 * WooCommerce B2B Sales Agents MyAccount - Subagents table
 *
 * @version 1.2.0
 */
?>

<?php defined( 'ABSPATH' ) || exit; ?>

<?php if ( ! $has_subagents ) : ?>
<div class="alert alert-dark">
    <div><?php esc_html_e( 'Sorry, there are not yet subagents assigned to you', 'woocommerce-b2b-sales-agents' ); ?></div>
</div>
<?php else: ?>

<div class="table-fluid small">
    <table class="table table-striped" data-render="datatables" data-name="subagents">
        <thead>
            <tr class="text-uppercase text-muted">
                <th class="text-nowrap border-0 no-sort"></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Name', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Surname', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'City', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Country', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Email', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Total customers', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Total orders', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Total orders amount', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0"><?php esc_html_e( 'Total commissions', 'woocommerce-b2b-sales-agents' ); ?></th>
                <th class="text-nowrap border-0 no-sort"></th>
            </tr>
        </thead>
        <tfoot>
            <tr class="search">
                <th class="no-search"></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th class="no-search"></th>
            </tr>
        </tfoot>
        <tbody>
        
            <?php foreach ( $subagents->get_results() as $subagent ) : ?>
            <tr>
                <td></td>

                <?php
                    // Customer name
                    $raw = $formatted = $subagent->user_firstname;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Name', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer lastname
                    $raw = $formatted = $subagent->user_lastname;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Surname', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer city
                    $raw = $formatted = get_user_meta( $subagent->ID, 'billing_city', true );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'City', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer country
                    $raw = $formatted = get_user_meta( $subagent->ID, 'billing_country', true );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Country', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Customer email
                    $raw = $formatted = $subagent->user_email;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Email', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Total customers
                    $raw = $formatted = wcb2bsa_get_customers_by_sales_agent( $subagent->ID )->total_users;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Total customers', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Total orders
                    $raw = $formatted = wcb2bsa_get_orders_by_sales_agent( $subagent->ID )->total;
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Total orders', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Total orders amount
                    $raw = wcb2bsa_get_total_orders_amount_by_sales_agent( $subagent->ID );
                    $formatted = wc_price( $raw );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Total orders amount', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Total commissions amount
                    $raw = wcb2bsa_get_total_commission_amount_by_sales_agent( $subagent->ID, 'total' );
                    $formatted = wc_price( $raw );
                    printf('<td data-title="%s" data-sort="%s">%s</td>',
                        esc_attr__( 'Total orders amount', 'woocommerce-b2b-sales-agents' ),
                        esc_html( $raw ),
                        wp_kses_post( $formatted )
                    );
                    // Button
                    printf('<td class="text-center"><a class="btn btn-dark btn-sm" href="%s">%s</a></td>',
                        esc_url( wcb2bsa_get_view_subagent_url( $subagent->ID ) ),
                        esc_html__( 'Details', 'woocommerce-b2b-sales-agents' )
                    );
                ?>

            </tr>    
            <?php endforeach; ?>

        </tbody>
    </table>
</div>
<?php endif; ?>