<?php
/**
 * WooCommerce B2B Sales Agents MyAccount - View order details
 *
 * @version 1.2.0
 */
?>

<?php defined( 'ABSPATH' ) || exit; ?>

<?php if ( ! $has_permission ) : ?>
<div class="alert alert-dark">
    <div><?php esc_html_e( 'This order is not from a customer assigned to you', 'woocommerce-b2b-sales-agents' ); ?></div>
</div>
<?php return; endif; ?>

<?php
    $order_item_types_eligible_for_commission = get_post_meta( $order->get_id(), 'wcb2bsa_order_item_types_eligible_for_commission', true );
    $currency = $order->get_currency();
?>

<div class="card-deck">
    <div class="card shadow-sm mb-4">
        <div class="card-body">
            <div class="card-title text-uppercase text-muted mb-2"><?php esc_html_e( 'Order reference', 'woocommerce-b2b-sales-agents' ); ?></div>
            <div class="row align-items-center">
                <div class="col">
                    <span class="h2 mb-0">#<?php echo wp_kses_post( $order->get_order_number() ); ?></span>
                </div>
                <div class="col-auto">
                    <div class="text-muted"><i class="fas fa-qrcode fa-3x"></i></div>
                </div>
            </div>
        </div>
    </div>
    <div class="card shadow-sm mb-4">
        <div class="card-body">
            <div class="card-title text-uppercase text-muted mb-2"><?php esc_html_e( 'Order status', 'woocommerce-b2b-sales-agents' ); ?></div>
            <div class="row align-items-center">
                <div class="col">
                    <span class="h2 mb-0"><?php echo wp_kses_post( wc_get_order_status_name( $order->get_status() ) ); ?></span>
                </div>
                <div class="col-auto">
                    <div class="text-muted"><i class="fas fa-info-circle fa-3x"></i></div>
                </div>
            </div>
        </div>
    </div>
    <div class="card shadow-sm mb-4">
        <div class="card-body">
            <div class="card-title text-uppercase text-muted mb-2"><?php esc_html_e( 'Payment method', 'woocommerce-b2b-sales-agents' ); ?></div>
            <div class="row align-items-center">
                <div class="col">
                    <span class="h2 mb-0"><?php echo wp_kses_post( $order->get_payment_method_title() ); ?></span>
                </div>
                <div class="col-auto">
                    <div class="text-muted"><i class="fas fa-credit-card fa-3x"></i></div>
                </div>
            </div>
        </div>
    </div>
</div>
<div class="card-deck">
    <div class="card shadow-sm mb-4">
        <div class="card-body">
            <div class="card-title text-uppercase text-muted mb-2"><?php esc_html_e( 'Order notes', 'woocommerce-b2b-sales-agents' ); ?></div>
            <div class="row align-items-center">
                <div class="col">
                    <ul class="list-unstyled">
                        <li><?php printf( '<strong>%s</strong>: %s', esc_html__( 'Date created', 'woocommerce-b2b-sales-agents' ), wp_kses_post( wc_format_datetime( $order->get_date_created() ) ) ); ?></li>
                        <li><?php printf( '<strong>%s</strong>: %s', esc_html__( 'Commissions applied to', 'woocommerce-b2b-sales-agents' ), $order_item_types_eligible_for_commission ? wp_kses_post( implode( ', ', array_intersect_key( WCB2BSA_Configuration::get_order_item_types_eligible_for_commission( false ), $order_item_types_eligible_for_commission ) ) ) : '-' ); ?></li>
                        <li><?php printf( '<strong>%s</strong>: %s', esc_html__( 'Commissions payment', 'woocommerce-b2b-sales-agents' ), wp_kses_post( wcb2bsa_get_order_commission_payout_by_sales_agent( $order->get_id(), get_post_meta( $order->get_id(), 'wcb2bsa_sales_agent', true ) ) ) ); ?></li>
                        <li><?php printf( '<strong>%s</strong>: %s', esc_html__( 'Checkout customer note', 'woocommerce-b2b-sales-agents' ), $order->get_customer_note() ); ?></li>
                        
                        <?php if ( $notes = wc_get_order_notes( array( 'order_id' => $order->get_ID() ) ) ) : ?>
                        <li>
                            <?php printf( '<strong>%s</strong>:', esc_html__( 'Other notes', 'woocommerce-b2b-sales-agents' ) ); ?>
                            <ul>
                                <?php foreach ( $notes as $note ) : ?>

                                    <?php if ( 'system' == $note->added_by ) { continue; } ?>
                                    <li>
                                        <div class="text-muted small"><?php printf( __( 'Added by %s on %s', 'woocommerce-b2b-sales-agents' ), $note->added_by, $note->date_created->date_i18n( esc_html__( 'l jS \o\f F Y, h:ia', 'woocommerce-b2b-sales-agents' ) ) ); ?></div>
                                        <div><?php printf( '<em class="small">[%s]</em> %s', ( empty( $note->customer_note ) ? esc_html__( 'Private', 'woocommerce-b2b-sales-agents' ) : esc_html__( 'Customer', 'woocommerce-b2b-sales-agents' ) ), wp_kses_post( wptexturize( $note->content ) ) ); ?></div>
                                    </li>

                                <?php endforeach; ?>
                            </ul>
                        </li>
                        <?php endif; ?>

                    </ul>
                </div>
            </div>
        </div>
    </div>
    <div class="card shadow-sm mb-4">
        <div class="card-body">
            <div class="card-title text-uppercase text-muted mb-2"><?php esc_html_e( 'Billing address', 'woocommerce-b2b-sales-agents' ); ?></div>
            <div class="row align-items-center">
                <div class="col">

                    <?php echo wp_kses_post( $order->get_formatted_billing_address( esc_html__( 'N/A', 'woocommerce-b2b-sales-agents' ) ) ); ?>
                        
                    <?php if ( $order->get_billing_phone() ) : ?>
                    <p><?php echo wp_kses_post( $order->get_billing_phone() ); ?></p>
                    <?php endif; ?>

                    <?php if ( $order->get_billing_email() ) : ?>
                    <p><?php echo wp_kses_post( $order->get_billing_email() ); ?></p>
                    <?php endif; ?>

                </div>
            </div>
        </div>
    </div>
    <div class="card shadow-sm mb-4">
        <div class="card-body">
            <div class="card-title text-uppercase text-muted mb-2"><?php esc_html_e( 'Shipping address', 'woocommerce-b2b-sales-agents' ); ?></div>
            <div class="row align-items-center">
                <div class="col">
                    <?php echo wp_kses_post( $order->get_formatted_shipping_address( esc_html__( 'N/A', 'woocommerce-b2b-sales-agents' ) ) ); ?>
                </div>
            </div>
        </div>
    </div>
</div>
<div class="card shadow-sm my-3">
    <table class="table text-right">
        <thead>
            <tr>
                <td class="text-left"><?php esc_html_e( 'Product', 'woocommerce-b2b-sales-agents' ); ?></td>
                <td><?php esc_html_e( 'Unit price', 'woocommerce-b2b-sales-agents' ); ?></td>
                <td><?php esc_html_e( 'Quantity', 'woocommerce-b2b-sales-agents' ); ?></td>
                <td><?php esc_html_e( 'Subtotal', 'woocommerce-b2b-sales-agents' ); ?></td>
                <td><?php esc_html_e( 'Tax', 'woocommerce-b2b-sales-agents' ); ?></td>
                <td><?php esc_html_e( 'Total', 'woocommerce-b2b-sales-agents' ); ?></td>
                <td><?php esc_html_e( 'Order commission', 'woocommerce-b2b-sales-agents' ); ?></td>
                <td><?php esc_html_e( 'Your earning', 'woocommerce-b2b-sales-agents' ); ?></td>
            </tr>
        </thead>
        <tbody>

            <?php foreach ( $order->get_items( 'line_item' ) as $item_id => $item ) : ?>

                <?php
                    $product = $item->get_product();
                    $product_permalink = $product && $product->is_visible() ? $product->get_permalink( $item ) : '';

                    $taxes = $item->get_taxes();
                    
                    $item_quantity = $item->get_quantity();

                    $item_subtotal_cost = $order->get_item_subtotal( $item, false, true );
                    $item_total_cost = $order->get_item_total( $item, false, true );

                    $item_subtotal = $item->get_subtotal();
                    $item_subtotal_tax = $item->get_subtotal_tax();

                    $item_total = $item->get_total();
                    $item_total_tax = $item->get_total_tax();

                    $item_refunded_qty = $order->get_qty_refunded_for_item( $item_id );
                    $item_refunded_total = -1 * $order->get_total_refunded_for_item( $item_id );
                    $item_refunded_tax = 0;
                    foreach ( $taxes['total'] as $tax_id => $tax_value ) {
                        $item_refunded_tax -= $order->get_tax_refunded_for_item( $item_id, $tax_id );
                    }

                    $has_discounts = $item_subtotal !== $item_total;
                    $has_refunds = (bool)$item_refunded_qty;
                ?>

                <tr>
                    <td class="text-left">
                        <?php
                            echo $product_permalink ? sprintf( '<a class="text-dark" href="%s">%s</a>', esc_url( $product_permalink ), wp_kses_post( $item->get_name() ) ) : wp_kses_post( $item->get_name() );
                            wc_display_item_meta( $item, array(
                                'before'    => '<ul class="list-unstyled"><li>',
                                'after'     => '</li></ul>',
                                'separator' => '</li><li>',
                            ) );
                        ?>
                    </td>
                    <td>
                        <?php
                            if ( $has_discounts ) {
                                printf( '<del class="text-muted small">%s</del> ', wp_kses_post( wc_price( $item_subtotal_cost, array( 'currency' => $currency ) ) ) );
                            }
                            printf( '%s', wp_kses_post( wc_price( $item_total_cost, array( 'currency' => $currency ) ) ) );
                        ?>
                    </td>
                    <td>
                        <?php
                            printf( '%s', wp_kses_post( $item_quantity + $item_refunded_qty ) );
                            if ( $has_refunds ) {
                                printf( '<div class="text-danger small">%s</div>', wp_kses_post( $item_refunded_qty ) );
                            }
                        ?>
                    </td>
                    <td>
                        <?php
                            if ( $has_discounts ) {
                                printf( '<del class="text-muted small">%s</del> ', wp_kses_post( wc_price( $item_subtotal, array( 'currency' => $currency ) ) ) );
                            }
                            printf( '%s', wc_price( $item_total, array( 'currency' => $currency ) ) );
                            if ( $has_refunds ) {
                                printf( '<div class="text-danger small">%s</div>', wp_kses_post( wc_price( $item_refunded_total, array( 'currency' => $currency ) ) ) );
                            }
                        ?>
                    </td>
                    <td>
                        <?php
                            if ( $has_discounts ) {
                                printf( '<del class="text-muted small">%s</del> ', wp_kses_post( wc_price( $item_subtotal_tax, array( 'currency' => $currency ) ) ) );
                            }
                            printf( '%s', wp_kses_post( wc_price( $item_total_tax, array( 'currency' => $currency ) ) ) );
                            if ( $has_refunds ) {
                                printf( '<div class="text-danger small">%s</div>', wp_kses_post( wc_price( $item_refunded_tax, array( 'currency' => $currency ) ) ) );
                            }
                        ?>
                    </td>
                    <td>
                        <?php
                            if ( $has_discounts ) {
                                printf( '<del class="text-muted small">%s</del> ', wp_kses_post( wc_price( $item_subtotal + $item_subtotal_tax, array( 'currency' => $currency ) ) ) );
                            }
                            printf( '%s', wp_kses_post( wc_price( $item_total + $item_total_tax, array( 'currency' => $currency ) ) ) );
                            if ( $has_refunds ) {
                                printf( '<div class="text-danger small">%s</div>', wp_kses_post( wc_price( $item_refunded_total + $item_refunded_tax, array( 'currency' => $currency ) ) ) );
                            }
                        ?>
                    </td>
                    <td>
                        <?php printf( '%s%%', wp_kses_post( wc_get_order_item_meta( $item_id, 'wcb2bsa_item_commission', true ) ) ?: 0 ); ?>    
                    </td>
                    <td>
                        <?php
                            $earning = 0;
                            if ( $order_item_types_eligible_for_commission && in_array( 'line_item', array_keys( $order_item_types_eligible_for_commission ) ) ) {
                                $earning = wc_get_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount_' . get_current_user_id(), true );
                            }
                            printf( '%s', wp_kses_post( wc_price( (float)$earning, array( 'currency' => $currency ) ) ) );
                        ?>
                    </td>
                </tr>

            <?php endforeach; ?>

        </tbody>
        <tfoot>

            <?php foreach ( $order->get_items( 'fee' ) as $item_id => $item ) : ?>

                <?php
                    $item_total = $item->get_total();
                    $item_total_tax = $item->get_total_tax();
                ?>

                <tr>
                    <td class="text-left"><?php printf( '%s <div class="text-muted small">%s</div>', esc_html__( 'Fee', 'woocommerce-b2b-sales-agents' ), wp_kses_post( $item->get_name() ) ); ?></td>
                    <td></td>
                    <td></td>
                    <td><?php echo wp_kses_post( wc_price( $item_total, array( 'currency' => $currency ) ) ); ?></td>
                    <td><?php echo wp_kses_post( wc_price( $item_total_tax, array( 'currency' => $currency ) ) ); ?></td>
                    <td><?php echo wp_kses_post( wc_price( $item_total + $item_total_tax, array( 'currency' => $currency ) ) ); ?></td>
                    <td>
                        <?php
                            if ( $order_item_types_eligible_for_commission && in_array( 'fee', array_keys( $order_item_types_eligible_for_commission ) ) ) {
                                printf( '%s%%', wp_kses_post( wc_get_order_item_meta( $item_id, 'wcb2bsa_item_commission', true ) ) ?: 0 );
                            }
                        ?>
                    </td>
                    <td>
                        <?php
                            if ( $order_item_types_eligible_for_commission && in_array( 'fee', array_keys( $order_item_types_eligible_for_commission ) ) ) {
                                printf( '%s', wp_kses_post( wc_price( (float)wc_get_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount_' . get_current_user_id(), true ), array( 'currency' => $currency ) ) ) );
                            }
                        ?>
                    </td>
                </tr>
            <?php endforeach; ?>

            <?php foreach ( $order->get_items( 'shipping' ) as $item_id => $item ) : ?>

                <?php
                    $item_total = $item->get_total();
                    $item_total_tax = $item->get_total_tax();
                ?>

                <tr>
                    <td class="text-left"><?php printf( '%s <div class="text-muted small">%s</div>', esc_html__( 'Shippings', 'woocommerce-b2b-sales-agents' ), wp_kses_post( $item->get_name() ) ); ?></td>
                    <td></td>
                    <td></td>
                    <td><?php echo wp_kses_post( wc_price( $item_total, array( 'currency' => $currency ) ) ); ?></td>
                    <td><?php echo wp_kses_post( wc_price( $item_total_tax, array( 'currency' => $currency ) ) ); ?></td>
                    <td><?php echo wp_kses_post( wc_price( $item_total + $item_total_tax, array( 'currency' => $currency ) ) ); ?></td>
                    <td>
                        <?php
                            if ( $order_item_types_eligible_for_commission && in_array( 'shipping', array_keys( $order_item_types_eligible_for_commission ) ) ) {
                                printf( '%s%%', wp_kses_post( wc_get_order_item_meta( $item_id, 'wcb2bsa_item_commission', true ) ) ?: 0 );
                            }
                        ?>
                    </td>
                    <td>
                        <?php
                            if ( $order_item_types_eligible_for_commission && in_array( 'shipping', array_keys( $order_item_types_eligible_for_commission ) ) ) {
                                printf( '%s', wp_kses_post( wc_price( (float)wc_get_order_item_meta( $item_id, 'wcb2bsa_item_commission_amount_' . get_current_user_id(), true ), array( 'currency' => $currency ) ) ) );
                            }
                        ?>
                    </td>
                </tr>
            <?php endforeach; ?>
    
        </tfoot>
    </table>
</div>
<br />
<div class="row justify-content-end">
    <div class="col-sm-4">
        <div class="card shadow-sm mb-4">
            <div class="card-body">
                <div class="card-title text-uppercase text-muted mb-2"><?php esc_html_e( 'Order totals', 'woocommerce-b2b-sales-agents' ); ?></div>
                <table class="table table-sm table-striped table-borderless">
                    <tbody>
                        
                        <?php foreach ( $order->get_order_item_totals( 'excl' ) as $key => $total ) : ?>
                        <tr>
                            <th><?php echo wp_kses_post( $total['label'] ); ?></th>
                            <td><?php echo wp_kses_post( $total['value'] ); ?></td>
                        </tr>
                        <?php endforeach; ?>

                        <tr>
                            <th><?php esc_html_e( 'Your earning:', 'woocommerce-b2b-sales-agents' ); ?>
                            <td><?php echo wp_kses_post( wc_price( wcb2bsa_get_order_commission_amount_by_sales_agent( $order->get_id(), get_current_user_id() ), array( 'currency' => $order->get_currency() ) ) ); ?></td>
                        </tr>
                    </tbody>
                </table>
            </div>
        </div>
    </div>
</div>
<br />
<div class="text-center text-sm-left">
    
    <?php if ( $order->has_status( apply_filters( 'woocommerce_valid_order_statuses_for_order_again', array( 'completed' ) ) ) ) : ?>
    <a class="btn btn-dark" href="<?php echo wp_nonce_url( add_query_arg( 'order_again', $order->get_id(), wc_get_cart_url() ), 'woocommerce-order_again' ); ?>">
        <i class="fas fa-redo mr-2"></i> 
        <?php esc_html_e( 'Order again', 'woocommerce-b2b-sales-agents' ); ?>
    </a>
    <?php endif; ?>

    <?php if ( $order->needs_payment() && $checkout_payment_url = $order->get_checkout_payment_url( true ) ) : ?>
    <a class="btn btn-dark" href="<?php echo esc_url( $checkout_payment_url ); ?>">
        <i class="fas fa-handshake mr-2"></i> 
        <?php esc_html_e( 'Click to complete payment', 'woocommerce-b2b-sales-agents' ); ?>
    </a>
    <?php endif; ?>

    <?php if ( 'yes' == get_option( 'wcb2bsa_update_order_status' ) && wcb2bsa_is_subagents_hierarchy_writable() ) : ?>
    <a class="btn btn-dark" href="#modal-update-order-status" data-toggle="modal">
        <i class="fas fa-sync-alt mr-2"></i>
        <?php esc_html_e( 'Update status', 'woocommerce-b2b-sales-agents' ); ?>
    </a>
    <?php endif; ?>

    <?php if ( 'yes' == get_option( 'wcb2bsa_add_order_note' )  && wcb2bsa_is_subagents_hierarchy_writable() ) : ?>
    <a class="btn btn-dark" href="#modal-add-order-note" data-toggle="modal">
        <i class="fas fa-sticky-note mr-2"></i>
        <?php esc_html_e( 'Add note', 'woocommerce-b2b-sales-agents' ); ?>
    </a>
    <?php endif; ?>

</div>
